@@ -14,8 +14,6 @@ See the License for the specific language governing permissions and
 limitations under the License.
 """
 
-from __future__ import division
-
 import operator as op
 from functools import reduce
 from typing import List, Tuple
@@ -30,12 +28,15 @@ import cvxpy.utilities as u
 from cvxpy.atoms.affine.add_expr import AddExpression
 from cvxpy.atoms.affine.affine_atom import AffAtom
 from cvxpy.atoms.affine.conj import conj
-from cvxpy.atoms.affine.reshape import deep_flatten
+from cvxpy.atoms.affine.reshape import deep_flatten, reshape
 from cvxpy.atoms.affine.sum import sum as cvxpy_sum
 from cvxpy.constraints.constraint import Constraint
 from cvxpy.error import DCPError
-from cvxpy.expressions.constants.parameter import (is_param_affine,
-                                                   is_param_free,)
+from cvxpy.expressions.constants.parameter import (
+    is_param_affine,
+    is_param_free,
+)
+from cvxpy.expressions.expression import Expression
 
 
 class BinaryOperator(AffAtom):
@@ -108,7 +109,7 @@ class MulExpression(BinaryOperator):
     def numeric(self, values):
         """Matrix multiplication.
         """
-        if self.args[0].shape == () or self.args[1].shape == () or \
+        if values[0].shape == () or values[1].shape == () or \
            intf.is_sparse(values[0]) or intf.is_sparse(values[1]):
             return values[0] * values[1]
         else:
@@ -338,12 +339,16 @@ class DivExpression(BinaryOperator):
         """
         for i in range(2):
             if sp.issparse(values[i]):
-                values[i] = values[i].todense().A
+                values[i] = values[i].toarray()
         return np.divide(values[0], values[1])
 
     def is_quadratic(self) -> bool:
         return self.args[0].is_quadratic() and self.args[1].is_constant()
 
+    def has_quadratic_term(self) -> bool:
+        """Can be a quadratic term if divisor is constant."""
+        return self.args[0].has_quadratic_term() and self.args[1].is_constant()
+
     def is_qpwa(self) -> bool:
         return self.args[0].is_qpwa() and self.args[1].is_constant()
 
@@ -415,7 +420,7 @@ class DivExpression(BinaryOperator):
         return (lu.div_expr(arg_objs[0], arg_objs[1]), [])
 
 
-def scalar_product(x, y):
+def vdot(x, y):
     """
     Return the standard inner product (or "scalar product") of (x,y).
 
@@ -445,3 +450,40 @@ def scalar_product(x, y):
     y = deep_flatten(y)
     prod = multiply(conj(x), y)
     return cvxpy_sum(prod)
+
+
+def scalar_product(x, y):
+    """
+    Alias for vdot.
+    """
+    return vdot(x, y)
+
+
+def outer(x, y):
+    """
+    Return the outer product of (x,y).
+
+    Parameters
+    ----------
+    x : Expression, int, float, NumPy ndarray, or nested list thereof.
+        Input is flattened if not already a vector.
+        The linear argument to the outer product.
+    y : Expression, int, float, NumPy ndarray, or nested list thereof.
+        Input is flattened if not already a vector.
+        The transposed-linear argument to the outer product.
+
+    Returns
+    -------
+    expr : Expression
+        The outer product of (x,y), linear in x and transposed-linear in y.
+    """
+    x = Expression.cast_to_const(x)
+    if x.ndim > 1:
+        raise ValueError("x must be a vector.")
+    y = Expression.cast_to_const(y)
+    if y.ndim > 1:
+        raise ValueError("y must be a vector.")
+    
+    x = reshape(x, (x.size, 1), order='F')
+    y = reshape(y, (1, y.size), order='F')
+    return x @ y
