@@ -55,6 +55,11 @@ class transpose(AffAtom):
         """
         return self.args[0].is_symmetric()
 
+    def is_skew_symmetric(self) -> bool:
+        """Is the expression skew-symmetric?
+        """
+        return self.args[0].is_skew_symmetric()
+
     def is_hermitian(self) -> bool:
         """Is the expression Hermitian?
         """
@@ -89,6 +94,4 @@ class transpose(AffAtom):
         tuple
             (LinOp for objective, list of constraints)
         """
-        # TODO(akshakya): This will need to be updated when we add support
-        # for >2D arrays.
         return (lu.transpose(arg_objs[0]), [])
