@@ -13,14 +13,16 @@ Copyright 2017 Steven Diamond
    See the License for the specific language governing permissions and
    limitations under the License.
 """
-import numbers
+from __future__ import annotations
+
+import os
 
 import numpy as np
-import scipy.sparse
+import scipy.sparse as sp
 
-import cvxpy.cvxcore.python.cvxcore as cvxcore
 import cvxpy.settings as s
 from cvxpy.lin_ops import lin_op as lo
+from cvxpy.lin_ops.canon_backend import CanonBackend
 
 
 def get_parameter_vector(param_size,
@@ -89,7 +91,7 @@ def reduce_problem_data_tensor(A, var_length, quad_form: bool = False):
 
     and the problem data matrix can be constructed with
 
-        problem_data : = scipy.sparse.csc_matrix(
+        problem_data : = sp.csc_matrix(
             (data, indices, indptr), shape = shape)
 
     Parameters
@@ -114,7 +116,7 @@ def reduce_problem_data_tensor(A, var_length, quad_form: bool = False):
     reduced_A_shape = (unique_old_row.size, A_coo.shape[1])
 
     # remap the rows
-    reduced_A = scipy.sparse.coo_matrix(
+    reduced_A = sp.coo_matrix(
         (A_coo.data, (reduced_row, A_coo.col)), shape=reduced_A_shape)
 
     # convert reduced_A to csr
@@ -125,7 +127,7 @@ def reduce_problem_data_tensor(A, var_length, quad_form: bool = False):
     # add one more column for the offset if not quad_form
     if not quad_form:
         n_cols += 1
-    n_constr = A.shape[0] // (n_cols)
+    n_constr, _ = np.divmod(A.shape[0], n_cols, dtype=np.int64)
     shape = (n_constr, n_cols)
     indices = nonzero_rows % (n_constr)
 
@@ -134,7 +136,7 @@ def reduce_problem_data_tensor(A, var_length, quad_form: bool = False):
 
     # construction of the indptr: scan through cols, and find
     # the structure of the column index pointer
-    indptr = np.zeros(n_cols + 1, dtype=np.int32)
+    indptr = np.zeros(n_cols + 1, dtype=np.int64)
     positions, counts = np.unique(cols, return_counts=True)
     indptr[positions+1] = counts
     indptr = np.cumsum(indptr)
@@ -214,13 +216,13 @@ def get_matrix_from_tensor(problem_data_tensor, param_vec,
     elif problem_data_index is not None:
         flat_problem_data = problem_data_tensor @ param_vec
     else:
-        param_vec = scipy.sparse.csc_matrix(param_vec[:, None])
+        param_vec = sp.csc_matrix(param_vec[:, None])
         flat_problem_data = problem_data_tensor @ param_vec
 
 
     if problem_data_index is not None:
         indices, indptr, shape = problem_data_index
-        M = scipy.sparse.csc_matrix(
+        M = sp.csc_matrix(
             (flat_problem_data, indices, indptr), shape=shape)
     else:
         n_cols = var_length
@@ -241,33 +243,18 @@ def get_matrix_from_tensor(problem_data_tensor, param_vec,
         A_vals = np.append(A.data, np.zeros(nonzero_rows.size))
         A_rows = np.append(A_rows, nonzero_rows % A_nrows)
         A_cols = np.append(A_cols, nonzero_rows // A_nrows)
-        A = scipy.sparse.csc_matrix((A_vals, (A_rows, A_cols)),
+        A = sp.csc_matrix((A_vals, (A_rows, A_cols)),
                                     shape=A.shape)
 
     return (A, b)
 
 
-def get_matrix_and_offset_from_unparameterized_tensor(problem_data_tensor,
-                                                      var_length):
-    """Converts unparameterized tensor to matrix offset representation
-
-    problem_data_tensor _must_ have been obtained from calling
-    get_problem_matrix on a problem with 0 parameters.
-
-    Parameters
-    ----------
-        problem_data_tensor: tensor returned from get_problem_matrix,
-            representing an affine map
-        var_length: the number of variables
-
-    Returns
-    -------
-        A tuple (A, b), where A is a matrix with `var_length` columns
-        and b is a flattened NumPy array representing the constant offset.
+def get_default_canon_backend() -> str:
     """
-    assert problem_data_tensor.shape[1] == 1
-    return get_matrix_and_offset_from_tensor(
-        problem_data_tensor, None, var_length)
+    Returns the default canonicalization backend, which can be set globally using an
+    environment variable.
+    """
+    return os.environ.get('CVXPY_DEFAULT_CANON_BACKEND', s.DEFAULT_CANON_BACKEND)
 
 
 def get_problem_matrix(linOps,
@@ -275,7 +262,9 @@ def get_problem_matrix(linOps,
                        id_to_col,
                        param_to_size,
                        param_to_col,
-                       constr_length):
+                       constr_length,
+                       canon_backend: str | None = None
+                       ):
     """
     Builds a sparse representation of the problem data.
 
@@ -287,6 +276,10 @@ def get_problem_matrix(linOps,
         param_to_size: A map from parameter id to parameter size.
         param_to_col: A map from parameter id to column in tensor.
         constr_length: Summed sizes of constraints input.
+        canon_backend :
+            'CPP' (default) | 'SCIPY'
+            Specifies which backend to use for canonicalization, which can affect
+            compilation time. Defaults to None, i.e., selecting the default backend.
 
     Returns
     -------
@@ -294,214 +287,29 @@ def get_problem_matrix(linOps,
         param_size + 1 columns (where param_size is the length of the
         parameter vector).
     """
-    lin_vec = cvxcore.ConstLinOpVector()
 
-    id_to_col_C = cvxcore.IntIntMap()
-    for id, col in id_to_col.items():
-        id_to_col_C[int(id)] = int(col)
+    # Allow to switch default backends through an environment variable for CI
+    default_canon_backend = get_default_canon_backend()
+    canon_backend = default_canon_backend if not canon_backend else canon_backend
 
-    param_to_size_C = cvxcore.IntIntMap()
-    for id, size in param_to_size.items():
-        param_to_size_C[int(id)] = int(size)
+    if canon_backend == s.CPP_CANON_BACKEND:
+        from cvxpy.cvxcore.python.cppbackend import build_matrix
+        return build_matrix(id_to_col, param_to_size, param_to_col, var_length, constr_length, linOps)
 
-    # dict to memoize construction of C++ linOps, and to keep Python references
-    # to them to prevent their deletion
-    linPy_to_linC = {}
-    for lin in linOps:
-        build_lin_op_tree(lin, linPy_to_linC)
-        tree = linPy_to_linC[lin]
-        lin_vec.push_back(tree)
-
-    problemData = cvxcore.build_matrix(lin_vec,
-                                       int(var_length),
-                                       id_to_col_C,
-                                       param_to_size_C,
-                                       s.get_num_threads())
-
-    # Populate tensors with info from problemData.
-    tensor_V = {}
-    tensor_I = {}
-    tensor_J = {}
-    for param_id, size in param_to_size.items():
-        tensor_V[param_id] = []
-        tensor_I[param_id] = []
-        tensor_J[param_id] = []
-        problemData.param_id = param_id
-        for i in range(size):
-            problemData.vec_idx = i
-            prob_len = problemData.getLen()
-            tensor_V[param_id].append(problemData.getV(prob_len))
-            tensor_I[param_id].append(problemData.getI(prob_len))
-            tensor_J[param_id].append(problemData.getJ(prob_len))
-
-    # Reduce tensors to a single sparse CSR matrix.
-    V = []
-    I = []
-    J = []
-    # one of the 'parameters' in param_to_col is a constant scalar offset,
-    # hence 'plus_one'
-    param_size_plus_one = 0
-    for param_id, col in param_to_col.items():
-        size = param_to_size[param_id]
-        param_size_plus_one += size
-        for i in range(size):
-            V.append(tensor_V[param_id][i])
-            I.append(tensor_I[param_id][i] +
-                     tensor_J[param_id][i]*constr_length)
-            J.append(tensor_J[param_id][i]*0 + (i + col))
-    V = np.concatenate(V)
-    I = np.concatenate(I)
-    J = np.concatenate(J)
-    A = scipy.sparse.csc_matrix(
-        (V, (I, J)),
-        shape=(np.int64(constr_length)*np.int64(var_length+1),
-               param_size_plus_one))
-    return A
-
-
-def format_matrix(matrix, shape=None, format='dense'):
-    """Returns the matrix in the appropriate form for SWIG wrapper"""
-    if (format == 'dense'):
-        # Ensure is 2D.
-        if len(shape) == 0:
-            shape = (1, 1)
-        elif len(shape) == 1:
-            shape = shape + (1,)
-        return np.reshape(matrix, shape, order='F')
-    elif(format == 'sparse'):
-        return scipy.sparse.coo_matrix(matrix)
-    elif(format == 'scalar'):
-        return np.asfortranarray([[matrix]])
-    else:
-        raise NotImplementedError()
-
-
-TYPE_MAP = {
-    "VARIABLE": cvxcore.VARIABLE,
-    "PARAM": cvxcore.PARAM,
-    "PROMOTE": cvxcore.PROMOTE,
-    "MUL": cvxcore.MUL,
-    "RMUL": cvxcore.RMUL,
-    "MUL_ELEM": cvxcore.MUL_ELEM,
-    "DIV": cvxcore.DIV,
-    "SUM": cvxcore.SUM,
-    "NEG": cvxcore.NEG,
-    "INDEX": cvxcore.INDEX,
-    "TRANSPOSE": cvxcore.TRANSPOSE,
-    "SUM_ENTRIES": cvxcore.SUM_ENTRIES,
-    "TRACE": cvxcore.TRACE,
-    "RESHAPE": cvxcore.RESHAPE,
-    "DIAG_VEC": cvxcore.DIAG_VEC,
-    "DIAG_MAT": cvxcore.DIAG_MAT,
-    "UPPER_TRI": cvxcore.UPPER_TRI,
-    "CONV": cvxcore.CONV,
-    "HSTACK": cvxcore.HSTACK,
-    "VSTACK": cvxcore.VSTACK,
-    "SCALAR_CONST": cvxcore.SCALAR_CONST,
-    "DENSE_CONST": cvxcore.DENSE_CONST,
-    "SPARSE_CONST": cvxcore.SPARSE_CONST,
-    "NO_OP": cvxcore.NO_OP,
-    "KRON_R": cvxcore.KRON_R,
-    "KRON_L": cvxcore.KRON_L
-}
-
-
-def get_type(linPy):
-    ty = linPy.type.upper()
-    if ty in TYPE_MAP:
-        return TYPE_MAP[ty]
-    else:
-        raise NotImplementedError("Type %s is not supported." % ty)
-
-
-def set_matrix_data(linC, linPy) -> None:
-    """Calls the appropriate cvxcore function to set the matrix data field of
-       our C++ linOp.
-    """
-    if get_type(linPy) == cvxcore.SPARSE_CONST:
-        coo = format_matrix(linPy.data, format='sparse')
-        linC.set_sparse_data(coo.data, coo.row.astype(float),
-                             coo.col.astype(float), coo.shape[0],
-                             coo.shape[1])
-    else:
-        linC.set_dense_data(format_matrix(linPy.data, shape=linPy.shape))
-        linC.set_data_ndim(len(linPy.data.shape))
-
-
-def set_slice_data(linC, linPy) -> None:
-    """
-    Loads the slice data, start, stop, and step into our C++ linOp.
-    The semantics of the slice operator is treated exactly the same as in
-    Python.  Note that the 'None' cases had to be handled at the wrapper level,
-    since we must load integers into our vector.
-    """
-    for i, sl in enumerate(linPy.data):
-        slice_vec = cvxcore.IntVector()
-        for var in [sl.start, sl.stop, sl.step]:
-            slice_vec.push_back(int(var))
-        linC.push_back_slice_vec(slice_vec)
-
-
-def set_linC_data(linC, linPy) -> None:
-    """Sets numerical data fields in linC."""
-    assert linPy.data is not None
-    if isinstance(linPy.data, tuple) and isinstance(linPy.data[0], slice):
-        slice_data = set_slice_data(linC, linPy)
-    elif isinstance(linPy.data, float) or isinstance(linPy.data,
-                                                   numbers.Integral):
-        linC.set_dense_data(format_matrix(linPy.data, format='scalar'))
-        linC.set_data_ndim(0)
-    else:
-        set_matrix_data(linC, linPy)
-
-
-def make_linC_from_linPy(linPy, linPy_to_linC) -> None:
-    """Construct a C++ LinOp corresponding to LinPy.
-
-    Children of linPy are retrieved from linPy_to_linC.
-    """
-    if linPy in linPy_to_linC:
-        return
-    typ = get_type(linPy)
-    shape = cvxcore.IntVector()
-    lin_args_vec = cvxcore.ConstLinOpVector()
-    for dim in linPy.shape:
-        shape.push_back(int(dim))
-    for argPy in linPy.args:
-        lin_args_vec.push_back(linPy_to_linC[argPy])
-    linC = cvxcore.LinOp(typ, shape, lin_args_vec)
-    linPy_to_linC[linPy] = linC
-
-    if linPy.data is not None:
-        if isinstance(linPy.data, lo.LinOp):
-            linC_data = linPy_to_linC[linPy.data]
-            linC.set_linOp_data(linC_data)
-            linC.set_data_ndim(len(linPy.data.shape))
+    elif canon_backend in {s.SCIPY_CANON_BACKEND, s.RUST_CANON_BACKEND,
+                           s.NUMPY_CANON_BACKEND}:
+        param_size_plus_one = sum(param_to_size.values())
+        output_shape = (np.int64(constr_length)*np.int64(var_length+1),
+                   param_size_plus_one)
+        if len(linOps) > 0:
+            backend = CanonBackend.get_backend(canon_backend, id_to_col,
+                                                          param_to_size, param_to_col,
+                                                          param_size_plus_one, var_length)
+            A_py = backend.build_matrix(linOps)
         else:
-            set_linC_data(linC, linPy)
+            A_py = sp.csc_matrix(((), ((), ())), output_shape)
+        assert A_py.shape == output_shape
+        return A_py
+    else:
+        raise ValueError(f'Unknown backend: {canon_backend}')
 
-
-def build_lin_op_tree(root_linPy, linPy_to_linC) -> None:
-    """Construct C++ LinOp tree from Python LinOp tree.
-
-    Constructed C++ linOps are stored in the linPy_to_linC dict,
-    which maps Python linOps to their corresponding C++ linOps.
-
-    Parameters
-    ----------
-        linPy_to_linC: a dict for memoizing construction and storing
-            the C++ LinOps
-    """
-    bfs_stack = [root_linPy]
-    post_order_stack = []
-    while bfs_stack:
-        linPy = bfs_stack.pop()
-        if linPy not in linPy_to_linC:
-            post_order_stack.append(linPy)
-            for arg in linPy.args:
-                bfs_stack.append(arg)
-            if isinstance(linPy.data, lo.LinOp):
-                bfs_stack.append(linPy.data)
-    while post_order_stack:
-        linPy = post_order_stack.pop()
-        make_linC_from_linPy(linPy, linPy_to_linC)
