@@ -17,7 +17,7 @@ limitations under the License.
 import abc
 import warnings
 from functools import wraps
-from typing import Tuple
+from typing import List, Literal, Tuple
 
 import numpy as np
 
@@ -29,6 +29,7 @@ from cvxpy import error
 from cvxpy.constraints import PSD, Equality, Inequality
 from cvxpy.expressions import cvxtypes
 from cvxpy.utilities import scopes
+from cvxpy.utilities.shape import size_from_shape
 
 
 def _cast_other(binary_op):
@@ -66,6 +67,29 @@ You're calling a NumPy function on a CVXPY expression. This is prone to causing
 errors or code that doesn't behave as expected. Consider using one of the
 functions documented here: https://www.cvxpy.org/tutorial/functions/index.html
 """
+
+__INPLACE_MUTATION_ERROR__ = """
+You're trying to mutate a CVXPY expression inplace. This is prone to errors or
+code that doesn't behave as expected. Consider alternatives. For example, replace
+    x += 1
+with
+    x = x + 1
+"""
+
+__ABS_ERROR__ = """
+You're calling the built-in abs function on a CVXPY expression. This is not
+supported. Consider using the abs function provided by CVXPY.
+"""
+
+DEFAULT_ORDER_DEPRECATION_MSG = (
+    """
+    You didn't specify the order of the FUNC_NAME expression. The default order
+    used in CVXPY is Fortran ('F') order. This default will change to match NumPy's
+    default order ('C') in a future version of CVXPY.
+    To suppress this warning, please specify the order explicitly.
+    """
+)
+
 __BINARY_EXPRESSION_UFUNCS__ = {
         np.add: lambda self, a: self.__radd__(a),
         np.subtract: lambda self, a: self.__rsub__(a),
@@ -85,6 +109,9 @@ __BINARY_EXPRESSION_UFUNCS__ = {
 }
 
 
+ExpressionLike = "Expression | np.typing.ArrayLike"
+
+
 class Expression(u.Canonical):
     """A mathematical expression in a convex optimization problem.
 
@@ -92,12 +119,11 @@ class Expression(u.Canonical):
     expressions (e.g., the sum of two expressions) and constraints.
     """
 
-    __metaclass__ = abc.ABCMeta
-
     # Handles arithmetic operator overloading with Numpy.
     __array_priority__ = 100
 
-    @abc.abstractproperty
+    @property
+    @abc.abstractmethod
     def value(self):
         """NumPy.ndarray or None : The numeric value of the expression.
         """
@@ -108,7 +134,8 @@ class Expression(u.Canonical):
         """
         return self.value
 
-    @abc.abstractproperty
+    @property
+    @abc.abstractmethod
     def grad(self):
         """Gives the (sub/super)gradient of the expression w.r.t. each variable.
 
@@ -122,7 +149,8 @@ class Expression(u.Canonical):
         """
         raise NotImplementedError()
 
-    @abc.abstractproperty
+    @property
+    @abc.abstractmethod
     def domain(self):
         """list : The constraints describing the closure of the region
            where the expression is finite.
@@ -153,6 +181,27 @@ class Expression(u.Canonical):
         return self
 
     # Curvature properties.
+    @property
+    def curvatures(self) -> List[str]:
+        """List : Returns a list of the curvatures of the expression."""
+        curvatures = [
+            (self.is_constant, s.CONSTANT),
+            (self.is_affine, s.AFFINE),
+            (self.is_convex, s.CONVEX),
+            (self.is_concave, s.CONCAVE),
+            (self.is_log_log_constant, s.LOG_LOG_CONSTANT),
+            (self.is_log_log_affine, s.LOG_LOG_AFFINE),
+            (self.is_log_log_convex, s.LOG_LOG_CONVEX),
+            (self.is_log_log_concave, s.LOG_LOG_CONCAVE),
+            (self.is_quasilinear, s.QUASILINEAR),
+            (self.is_quasiconvex, s.QUASICONVEX),
+            (self.is_quasiconcave, s.QUASICONCAVE),
+        ]
+        curvatures = [curvature for condition, curvature in curvatures if condition()]
+        if not curvatures:
+            return [s.UNKNOWN]
+        return curvatures
+
     @property
     def curvature(self) -> str:
         """str : The curvature of the expression.
@@ -334,12 +383,27 @@ class Expression(u.Canonical):
         # Defaults to is constant.
         return self.is_constant()
 
+    def has_quadratic_term(self) -> bool:
+        """Does the affine head of the expression contain a quadratic term?
+
+        The affine head is all nodes with a path to the root node
+        that does not pass through any non-affine atom. If the root node
+        is non-affine, then the affine head is the root alone.
+        """
+        # Defaults to constant.
+        return self.is_constant()
+
     def is_symmetric(self) -> bool:
         """Is the expression symmetric?
         """
         # Defaults to false unless scalar.
         return self.is_scalar()
 
+    def is_skew_symmetric(self) -> bool:
+        """Is this Expression, X, a real matrix that satisfies X + X.T == 0?
+        """
+        return False
+
     def is_pwl(self) -> bool:
         """Is the expression piecewise linear?
         """
@@ -385,7 +449,8 @@ class Expression(u.Canonical):
         """
         raise NotImplementedError()
 
-    @abc.abstractproperty
+    @property
+    @abc.abstractmethod
     def shape(self) -> Tuple[int, ...]:
         """tuple : The expression dimensions.
         """
@@ -396,7 +461,8 @@ class Expression(u.Canonical):
         """
         return not self.is_complex()
 
-    @abc.abstractproperty
+    @property
+    @abc.abstractmethod
     def is_imag(self) -> bool:
         """Is the Leaf imaginary?
         """
@@ -412,7 +478,7 @@ class Expression(u.Canonical):
     def size(self) -> int:
         """int : The number of entries in the expression.
         """
-        return np.prod(self.shape, dtype=int)
+        return size_from_shape(self.shape)
 
     @property
     def ndim(self) -> int:
@@ -420,10 +486,18 @@ class Expression(u.Canonical):
         """
         return len(self.shape)
 
-    def flatten(self):
-        """Vectorizes the expression.
+    def flatten(self, order: Literal["F", "C", None] = None):
         """
-        return cvxtypes.vec()(self)
+        Vectorizes the expression.
+
+        order: column-major ('F') or row-major ('C') order.
+        """
+        if order is None:
+            flatten_order_warning = DEFAULT_ORDER_DEPRECATION_MSG.replace("FUNC_NAME", "flatten")
+            warnings.warn(flatten_order_warning, FutureWarning)
+            order = 'F'
+        assert order in ['F', 'C']
+        return cvxtypes.vec()(self, order)
 
     def is_scalar(self) -> bool:
         """Is the expression a scalar?
@@ -440,7 +514,7 @@ class Expression(u.Canonical):
         """
         return self.ndim == 2 and self.shape[0] > 1 and self.shape[1] > 1
 
-    def __getitem__(self, key):
+    def __getitem__(self, key) -> "Expression":
         """Return a slice/index into the expression.
         """
         # Returning self for scalars causes
@@ -453,7 +527,7 @@ class Expression(u.Canonical):
             return cvxtypes.index()(self, key)
 
     @property
-    def T(self):
+    def T(self) -> "Expression":
         """Expression : The transpose of the expression.
         """
         # Transpose of a scalar is that scalar.
@@ -463,8 +537,8 @@ class Expression(u.Canonical):
             return cvxtypes.transpose()(self)
 
     @property
-    def H(self):
-        """Expression : The transpose of the expression.
+    def H(self) -> "Expression":
+        """Expression : The conjugate-transpose of the expression.
         """
         if self.is_real():
             return self.T
@@ -533,7 +607,7 @@ class Expression(u.Canonical):
         return lh_expr, rh_expr
 
     @_cast_other
-    def __add__(self, other: "Expression") -> "Expression":
+    def __add__(self, other: ExpressionLike) -> "Expression":
         """Expression : Sum two expressions.
         """
         if isinstance(other, cvxtypes.constant()) and other.is_zero():
@@ -542,7 +616,7 @@ class Expression(u.Canonical):
         return cvxtypes.add_expr()([self, other])
 
     @_cast_other
-    def __radd__(self, other: "Expression") -> "Expression":
+    def __radd__(self, other: ExpressionLike) -> "Expression":
         """Expression : Sum two expressions.
         """
         if isinstance(other, cvxtypes.constant()) and other.is_zero():
@@ -550,19 +624,19 @@ class Expression(u.Canonical):
         return other + self
 
     @_cast_other
-    def __sub__(self, other: "Expression") -> "Expression":
+    def __sub__(self, other: ExpressionLike) -> "Expression":
         """Expression : The difference of two expressions.
         """
         return self + -other
 
     @_cast_other
-    def __rsub__(self, other: "Expression") -> "Expression":
+    def __rsub__(self, other: ExpressionLike) -> "Expression":
         """Expression : The difference of two expressions.
         """
         return other - self
 
     @_cast_other
-    def __mul__(self, other: "Expression") -> "Expression":
+    def __mul__(self, other: ExpressionLike) -> "Expression":
         """Expression : The product of two expressions.
         """
         if self.shape == () or other.shape == ():
@@ -596,21 +670,30 @@ class Expression(u.Canonical):
             return cvxtypes.matmul_expr()(self, other)
 
     @_cast_other
-    def __matmul__(self, other: "Expression") -> "Expression":
+    def __matmul__(self, other: ExpressionLike) -> "Expression":
         """Expression : Matrix multiplication of two expressions.
         """
         if self.shape == () or other.shape == ():
             raise ValueError("Scalar operands are not allowed, use '*' instead")
+
+        if isinstance(self, cvxtypes.matmul_expr()):
+            # LHS is matrix multiplication expr, so candidate for QuadForm:
+            # Specifically, iff the matrix multiplication is of the form x.T @ A @ y
+            # such that x == y, A is constant matrix and x is a variable, then it is a QuadForm.
+            if self.args[0] is other and not other.is_constant() and self.args[1].is_constant():
+                from cvxpy.expressions.cvxtypes import quad_form
+                return quad_form()(other, self.args[1])
+
         return cvxtypes.matmul_expr()(self, other)
 
     @_cast_other
-    def __truediv__(self, other: "Expression") -> "Expression":
+    def __truediv__(self, other: ExpressionLike) -> "Expression":
         """Expression : One expression divided by another.
         """
         return self.__div__(other)
 
     @_cast_other
-    def __div__(self, other: "Expression") -> "Expression":
+    def __div__(self, other: ExpressionLike) -> "Expression":
         """Expression : One expression divided by another.
         """
         self, other = self.broadcast(self, other)
@@ -621,56 +704,56 @@ class Expression(u.Canonical):
                              self.shape, other.shape))
 
     @_cast_other
-    def __rdiv__(self, other: "Expression") -> "Expression":
+    def __rdiv__(self, other: ExpressionLike) -> "Expression":
         """Expression : Called for Number / Expression.
         """
         return other / self
 
     @_cast_other
-    def __rtruediv__(self, other: "Expression") -> "Expression":
+    def __rtruediv__(self, other: ExpressionLike) -> "Expression":
         """Expression : Called for Number / Expression.
         """
         return other / self
 
     @_cast_other
-    def __rmul__(self, other: "Expression") -> "Expression":
+    def __rmul__(self, other: ExpressionLike) -> "Expression":
         """Expression : Called for Number * Expression.
         """
         return other * self
 
     @_cast_other
-    def __rmatmul__(self, other: "Expression") -> "Expression":
+    def __rmatmul__(self, other: ExpressionLike) -> "Expression":
         """Expression : Called for matrix @ Expression.
         """
         if self.shape == () or other.shape == ():
             raise ValueError("Scalar operands are not allowed, use '*' instead")
         return cvxtypes.matmul_expr()(other, self)
 
-    def __neg__(self):
+    def __neg__(self) -> "Expression":
         """Expression : The negation of the expression.
         """
         return cvxtypes.neg_expr()(self)
 
     @_cast_other
-    def __rshift__(self, other: "Expression") -> PSD:
+    def __rshift__(self, other: ExpressionLike) -> PSD:
         """PSD : Creates a positive semidefinite inequality.
         """
         return PSD(self - other)
 
     @_cast_other
-    def __rrshift__(self, other: "Expression") -> PSD:
+    def __rrshift__(self, other: ExpressionLike) -> PSD:
         """PSD : Creates a positive semidefinite inequality.
         """
         return PSD(other - self)
 
     @_cast_other
-    def __lshift__(self, other: "Expression") -> PSD:
+    def __lshift__(self, other: ExpressionLike) -> PSD:
         """PSD : Creates a negative semidefinite inequality.
         """
         return PSD(other - self)
 
     @_cast_other
-    def __rlshift__(self, other: "Expression") -> PSD:
+    def __rlshift__(self, other: ExpressionLike) -> PSD:
         """PSD : Creates a negative semidefinite inequality.
         """
         return PSD(self - other)
@@ -681,31 +764,25 @@ class Expression(u.Canonical):
 
     # Comparison operators.
     @_cast_other
-    def __eq__(self, other: "Expression"):
+    def __eq__(self, other: ExpressionLike) -> Equality:
         """Equality : Creates a constraint ``self == other``.
         """
         return Equality(self, other)
 
     @_cast_other
-    def __le__(self, other: "Expression"):
+    def __le__(self, other: ExpressionLike) -> Inequality:
         """Inequality : Creates an inequality constraint ``self <= other``.
         """
         return Inequality(self, other)
 
-    def __lt__(self, other: "Expression"):
-        """Unsupported.
-        """
+    def __lt__(self, other: ExpressionLike):
         raise NotImplementedError("Strict inequalities are not allowed.")
 
     @_cast_other
-    def __ge__(self, other: "Expression"):
-        """NonPos : Creates an inequality constraint.
-        """
-        return other.__le__(self)
+    def __ge__(self, other: ExpressionLike) -> Inequality:
+        return Inequality(other, self)
 
-    def __gt__(self, other: "Expression"):
-        """Unsupported.
-        """
+    def __gt__(self, other: ExpressionLike):
         raise NotImplementedError("Strict inequalities are not allowed.")
 
     def __array_ufunc__(self, ufunc, method, *args, **kwargs):
@@ -715,7 +792,113 @@ class Expression(u.Canonical):
                     len(args) == 2 and \
                     args[1] is self:
                 return ufunc_handler(self, args[0])
+            elif kwargs.keys() == {'out'} and \
+                    len(args) == 2 and \
+                    args[1] is self and \
+                    isinstance(kwargs['out'], tuple) and \
+                    len(kwargs['out']) == 1 and \
+                    args[0] is kwargs['out'][0]:
+                raise RuntimeError(__INPLACE_MUTATION_ERROR__)
+
         except KeyError:
             pass
 
         raise RuntimeError(__NUMPY_UFUNC_ERROR__)
+
+    def __abs__(self):
+        raise TypeError(__ABS_ERROR__)
+
+    def conj(self) -> "Expression":
+        """
+        Equivalent to `cp.conj(self)`.
+        """
+        from cvxpy import conj
+        return conj(self)
+
+    def conjugate(self) -> "Expression":
+        """
+        Equivalent to `cp.conj(self)`.
+        """
+        from cvxpy import conj
+        return conj(self)
+
+    def cumsum(self, axis=0) -> "Expression":
+        """
+        Equivalent to `cp.cumsum(self, axis)`.
+        """
+        from cvxpy import cumsum
+        return cumsum(self, axis)
+
+    def max(self, axis=None, *, keepdims=False) -> "Expression":
+        """
+        Equivalent to `cp.max(self, axis, keepdims)`.
+        """
+        from cvxpy import max as max_
+        return max_(self, axis, keepdims)
+
+    def mean(self, axis=None, *, keepdims=False) -> "Expression":
+        """
+        Equivalent to `cp.mean(self, axis, keepdims)`.
+        """
+        from cvxpy import mean
+        return mean(self, axis, keepdims)
+
+    def min(self, axis=None, *, keepdims=False) -> "Expression":
+        """
+        Equivalent to `cp.min(self, axis, keepdims)`.
+        """
+        from cvxpy import min as min_
+        return min_(self, axis, keepdims)
+
+    def prod(self, axis=None, *, keepdims=False) -> "Expression":
+        """
+        Equivalent to `cp.prod(self, axis, keepdims)`.
+        """
+        from cvxpy import prod
+        return prod(self, axis, keepdims)
+
+    def ptp(self, axis=None, *, keepdims=False) -> "Expression":
+        """
+        Equivalent to `cp.ptp(self, axis, keepdims)`.
+        """
+        from cvxpy import ptp
+        return ptp(self, axis, keepdims)
+
+    def reshape(self, shape, order: Literal["F", "C", None] = None) -> "Expression":
+        """
+        Equivalent to `cp.reshape(self, shape, order)`.
+        """
+        if order is None:
+            reshape_order_warning = DEFAULT_ORDER_DEPRECATION_MSG.replace("FUNC_NAME", "reshape")
+            warnings.warn(reshape_order_warning, FutureWarning)
+            order = 'F'
+        from cvxpy import reshape
+        return reshape(self, shape, order)
+
+    def std(self, axis=None, *, ddof=0, keepdims=False) -> "Expression":
+        """
+        Equivalent to `cp.std(self, axis, keepdims)`.
+        """
+        from cvxpy import std
+        return std(self, axis=axis, ddof=ddof, keepdims=keepdims)
+ 
+    def sum(self, axis=None, *, keepdims=False) -> "Expression":
+        """
+        Equivalent to `cp.sum(self, axis, keepdims)`.
+        """
+        from cvxpy import sum as sum_
+        return sum_(self, axis, keepdims)
+
+    def trace(self) -> "Expression":
+        """
+        Equivalent to `cp.trace(self)`.
+        """
+        from cvxpy import trace
+        return trace(self)
+
+    def var(self, *, ddof=0) -> "Expression":
+        """
+        Equivalent to `cp.var(self)`.
+        """
+        from cvxpy import var
+        return var(self, ddof=ddof)
