@@ -13,25 +13,31 @@ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 """
+from __future__ import annotations
 
-import abc
-from typing import TYPE_CHECKING, List, Tuple, Union
+from typing import TYPE_CHECKING, Iterable, Optional
 
 if TYPE_CHECKING:
-    from cvxpy import Constant, Variable
+    from cvxpy import Constant, Parameter, Variable
     from cvxpy.atoms.atom import Atom
 
 import numbers
+import warnings
 
 import numpy as np
 import numpy.linalg as LA
 import scipy.sparse as sp
 
 import cvxpy.interface as intf
+import cvxpy.settings as s
 from cvxpy.constraints.constraint import Constraint
 from cvxpy.expressions import expression
-from cvxpy.settings import (GENERAL_PROJECTION_TOL, PSD_NSD_PROJECTION_TOL,
-                            SPARSE_PROJECTION_TOL,)
+from cvxpy.settings import (
+    GENERAL_PROJECTION_TOL,
+    PSD_NSD_PROJECTION_TOL,
+    SPARSE_PROJECTION_TOL,
+)
+from cvxpy.utilities.coo_array_compat import get_coords
 
 
 class Leaf(expression.Expression):
@@ -49,10 +55,8 @@ class Leaf(expression.Expression):
 
     Parameters
     ----------
-    shape : tuple or int
-        The leaf dimensions. Either an integer n for a 1D shape, or a
-        tuple where the semantics are the same as NumPy ndarray shapes.
-        **Shapes cannot be more than 2D**.
+    shape : int or tuple of ints
+        Shape of the leaf, e.g., ``(3, 2)`` or ``2``.
     value : numeric type
         A value to assign to the leaf.
     nonneg : bool
@@ -71,42 +75,44 @@ class Leaf(expression.Expression):
         Is the variable constrained to be negative semidefinite?
     Hermitian : bool
         Is the variable Hermitian?
-    boolean : bool or list of tuple
+    boolean : bool or Iterable
         Is the variable boolean? True, which constrains
         the entire Variable to be boolean, False, or a list of
         indices which should be constrained as boolean, where each
         index is a tuple of length exactly equal to the
         length of shape.
-    integer : bool or list of tuple
+    integer : bool or Iterable
         Is the variable integer? The semantics are the same as the
         boolean argument.
-    sparsity : list of tuplewith
-        Fixed sparsity pattern for the variable.
+    sparsity : Iterable
+        Is the variable sparse?
     pos : bool
         Is the variable positive?
     neg : bool
         Is the variable negative?
+    bounds : Iterable
+        An iterable of length two specifying lower and upper bounds.
     """
 
-    __metaclass__ = abc.ABCMeta
-
     def __init__(
-        self, shape: Union[int, Tuple[int, ...]], value=None, nonneg: bool = False,
+        self, shape: int | tuple[int, ...], value = None, nonneg: bool = False,
         nonpos: bool = False, complex: bool = False, imag: bool = False,
         symmetric: bool = False, diag: bool = False, PSD: bool = False,
         NSD: bool = False, hermitian: bool = False,
-        boolean: bool = False, integer: bool = False,
-        sparsity=None, pos: bool = False, neg: bool = False
+        boolean: Iterable | bool = False, integer: Iterable | bool = False,
+        sparsity: Iterable | bool = False, pos: bool = False, neg: bool = False,
+        bounds: Iterable | None = None
     ) -> None:
         if isinstance(shape, numbers.Integral):
             shape = (int(shape),)
-        elif len(shape) > 2:
+        elif not s.ALLOW_ND_EXPR and len(shape) > 2:
             raise ValueError("Expressions of dimension greater than 2 "
                              "are not supported.")
         for d in shape:
             if not isinstance(d, numbers.Integral) or d <= 0:
                 raise ValueError("Invalid dimensions %s." % (shape,))
-        self._shape = tuple(np.int32(d) for d in shape)
+        shape = tuple(shape)
+        self._shape = shape
 
         if (PSD or NSD or symmetric or diag or hermitian) and (len(shape) != 2
                                                                or shape[0] != shape[1]):
@@ -119,37 +125,64 @@ class Leaf(expression.Expression):
                            'complex': complex, 'imag': imag,
                            'symmetric': symmetric, 'diag': diag,
                            'PSD': PSD, 'NSD': NSD,
-                           'hermitian': hermitian, 'boolean': bool(boolean),
-                           'integer':  integer, 'sparsity': sparsity}
-
-        if boolean:
-            self.boolean_idx = boolean if not type(boolean) == bool else list(
-                np.ndindex(max(shape, (1,))))
-        else:
+                           'hermitian': hermitian, 'boolean': boolean,
+                           'integer':  integer, 'sparsity': sparsity, 'bounds': bounds}
+        if boolean is True:
+            shape = max(shape, (1,))
+            flat_idx = np.arange(np.prod(shape))
+            self.boolean_idx = np.unravel_index(flat_idx, shape, order='F')
+        elif boolean is False:
             self.boolean_idx = []
-
-        if integer:
-            self.integer_idx = integer if not type(integer) == bool else list(
-                np.ndindex(max(shape, (1,))))
         else:
+            self.boolean_idx = boolean
+        if integer is True:
+            shape = max(shape, (1,))
+            flat_idx = np.arange(np.prod(shape))
+            self.integer_idx = np.unravel_index(flat_idx, shape, order='F')
+        elif integer is False:
             self.integer_idx = []
-
+        else:
+            self.integer_idx = integer
+        if sparsity:
+            self.sparse_idx = self._validate_indices(sparsity)
+        else:
+            self.sparse_idx = None
         # Only one attribute be True (except can be boolean and integer).
         true_attr = sum(1 for k, v in self.attributes.items() if v)
+        # HACK we should remove this feature or allow multiple attributes in general.
         if boolean and integer:
             true_attr -= 1
         if true_attr > 1:
             raise ValueError("Cannot set more than one special attribute in %s."
                              % self.__class__.__name__)
-
         if value is not None:
             self.value = value
 
         self.args = []
+        self.bounds = self._ensure_valid_bounds(bounds)
+
+    def _validate_indices(self, indices: list[tuple[int]] | tuple[np.ndarray]) -> tuple[np.ndarray]:
+        """
+        Validate the sparsity pattern for a leaf node.
+
+        Parameters:
+        indices: List or tuple of indices indicating the positions of non-zero elements.
+        """
+        if self._shape == ():
+            if indices != []:
+                raise ValueError("Indices should have 0 dimensions.")
+            return []
+        # Attempt to form a COO_array with the indices matrix provided;
+        # this will raise errors if invalid.
+        validator = sp.coo_array((np.empty(len(indices[0])), indices), shape=self._shape)
+        # Apply an in-place transformation to the coordinates to reduce the
+        # validator to canonical form
+        validator.sum_duplicates()
+        # Return the canonicalized coordinates
+        return get_coords(validator)
 
     def _get_attr_str(self) -> str:
-        """Get a string representing the attributes.
-        """
+        """Get a string representing the attributes."""
         attr_str = ""
         for attr, val in self.attributes.items():
             if attr != 'real' and val:
@@ -177,112 +210,153 @@ class Leaf(expression.Expression):
         return self  # Leaves are not deep copied.
 
     def get_data(self) -> None:
-        """Leaves are not copied.
-        """
-        pass
+        """Leaves are not copied."""
 
     @property
-    def shape(self) -> Tuple[int, ...]:
-        """ tuple : The dimensions of the expression.
-        """
+    def shape(self) -> tuple[int, ...]:
+        """The dimensions of the expression."""
         return self._shape
 
-    def variables(self) -> List['Variable']:
-        """Default is empty list of Variables.
-        """
+    def variables(self) -> list[Variable]:
+        """Default is empty list of Variables."""
         return []
 
-    def parameters(self):
-        """Default is empty list of Parameters.
-        """
+    def parameters(self) -> list[Parameter]:
+        """Default is empty list of Parameters."""
         return []
 
-    def constants(self) -> List['Constant']:
-        """Default is empty list of Constants.
-        """
+    def constants(self) -> list[Constant]:
+        """Default is empty list of Constants."""
         return []
 
     def is_convex(self) -> bool:
-        """Is the expression convex?
-        """
+        """Is the expression convex?"""
         return True
 
     def is_concave(self) -> bool:
-        """Is the expression concave?
-        """
+        """Is the expression concave?"""
         return True
 
     def is_log_log_convex(self) -> bool:
-        """Is the expression log-log convex?
-        """
+        """Is the expression log-log convex?"""
         return self.is_pos()
 
     def is_log_log_concave(self) -> bool:
-        """Is the expression log-log concave?
-        """
+        """Is the expression log-log concave?"""
         return self.is_pos()
 
     def is_nonneg(self) -> bool:
-        """Is the expression nonnegative?
-        """
+        """Is the expression nonnegative?"""
         return (self.attributes['nonneg'] or self.attributes['pos'] or
                 self.attributes['boolean'])
 
     def is_nonpos(self) -> bool:
-        """Is the expression nonpositive?
-        """
+        """Is the expression nonpositive?"""
         return self.attributes['nonpos'] or self.attributes['neg']
 
     def is_pos(self) -> bool:
-        """Is the expression positive?
-        """
+        """Is the expression positive?"""
         return self.attributes['pos']
 
     def is_neg(self) -> bool:
-        """Is the expression negative?
-        """
+        """Is the expression negative?"""
         return self.attributes['neg']
 
     def is_hermitian(self) -> bool:
-        """Is the Leaf hermitian?
-        """
+        """Is the Leaf hermitian?"""
         return (self.is_real() and self.is_symmetric()) or \
             self.attributes['hermitian'] or self.is_psd() or self.is_nsd()
 
     def is_symmetric(self) -> bool:
-        """Is the Leaf symmetric?
-        """
+        """Is the Leaf symmetric?"""
         return self.is_scalar() or \
             any(self.attributes[key] for key in ['diag', 'symmetric', 'PSD', 'NSD'])
 
     def is_imag(self) -> bool:
-        """Is the Leaf imaginary?
-        """
+        """Is the Leaf imaginary?"""
         return self.attributes['imag']
 
     def is_complex(self) -> bool:
-        """Is the Leaf complex valued?
-        """
+        """Is the Leaf complex valued?"""
         return self.attributes['complex'] or self.is_imag() or self.attributes['hermitian']
 
+    def _has_lower_bounds(self) -> bool:
+        """Does the variable have lower bounds?"""
+        if self.is_nonneg():
+            return True
+        elif self.attributes['bounds'] is not None:
+            lower_bound = self.attributes['bounds'][0]
+            if np.isscalar(lower_bound):
+                return lower_bound != -np.inf
+            else:
+                return np.any(lower_bound != -np.inf)
+        else:
+            return False
+
+    def _has_upper_bounds(self) -> bool:
+        """Does the variable have upper bounds?"""
+        if self.is_nonpos():
+            return True
+        elif self.attributes['bounds'] is not None:
+            upper_bound = self.attributes['bounds'][1]
+            if np.isscalar(upper_bound):
+                return upper_bound != np.inf
+            else:
+                return np.any(upper_bound != np.inf)
+        else:
+            return False
+
+    def _bound_domain(self, term: expression.Expression, constraints: list[Constraint]) -> None:
+        """A utility function to append constraints from lower and upper bounds.
+
+        Parameters
+        ----------
+        term: The term to encode in the constraints.
+        constraints: An existing list of constraitns to append to.
+        """
+        if self.attributes['nonneg'] or self.attributes['pos']:
+            constraints.append(term >= 0)
+        elif self.attributes['nonpos'] or self.attributes['neg']:
+            constraints.append(term <= 0)
+        elif self.attributes['bounds']:
+            bounds = self.bounds
+            lower_bounds, upper_bounds = bounds
+            # Create masks if -inf or inf is present in the bounds
+            lower_bound_mask = (lower_bounds != -np.inf)
+            upper_bound_mask = (upper_bounds != np.inf)
+
+            if np.any(lower_bound_mask):
+                # At least one valid lower bound,
+                # so we apply the constraint only to those entries
+                if self.ndim > 0:
+                    constraints.append(term[lower_bound_mask] >= lower_bounds[lower_bound_mask])
+                else:
+                    constraints.append(term >= lower_bounds)
+            if np.any(upper_bound_mask):
+                # At least one valid upper bound,
+                # so we apply the constraint only to those entries
+                if self.ndim > 0:
+                    constraints.append(term[upper_bound_mask] <= upper_bounds[upper_bound_mask])
+                else:
+                    constraints.append(term <= upper_bounds)
+
     @property
-    def domain(self) -> List[Constraint]:
+    def domain(self) -> list[Constraint]:
         """A list of constraints describing the closure of the region
            where the expression is finite.
         """
         # Default is full domain.
         domain = []
-        if self.attributes['nonneg'] or self.attributes['pos']:
-            domain.append(self >= 0)
-        elif self.attributes['nonpos'] or self.attributes['neg']:
-            domain.append(self <= 0)
-        elif self.attributes['PSD']:
+        # Add constraints from bounds.
+        self._bound_domain(self, domain)
+        # Add positive/negative semidefiniteness constraints.
+        if self.attributes['PSD']:
             domain.append(self >> 0)
         elif self.attributes['NSD']:
             domain.append(self << 0)
         return domain
 
-    def project(self, val):
+    def project(self, val, sparse_path=False):
         """Project value onto the attribute set of the leaf.
 
         A sensible idiom is ``leaf.value = leaf.project(val)``.
@@ -308,6 +382,8 @@ class Leaf(expression.Expression):
             return np.minimum(val, 0.)
         elif self.attributes['nonneg'] or self.attributes['pos']:
             return np.maximum(val, 0.)
+        elif self.attributes['bounds']:
+            return np.clip(val, self.bounds[0], self.bounds[1])
         elif self.attributes['imag']:
             return np.imag(val)*1j
         elif self.attributes['complex']:
@@ -348,29 +424,82 @@ class Leaf(expression.Expression):
                     return val
                 w[bad] = 0
             return (V * w).dot(V.T)
+        elif self.attributes['sparsity'] and not sparse_path:
+            warnings.warn('Accessing a sparse CVXPY expression via a dense representation.'
+                          ' Please report this as a bug to the CVXPY Discord or GitHub.',
+                          RuntimeWarning, 3)
+            new_val = np.zeros(self.shape)
+            new_val[self.sparse_idx] = val[self.sparse_idx]
+            return new_val
         else:
             return val
 
     # Getter and setter for parameter value.
-    def save_value(self, val) -> None:
-        self._value = val
+    def save_value(self, val, sparse_path=False) -> None:
+        if self.sparse_idx is not None and not sparse_path:
+            self._value = sp.coo_array((val[self.sparse_idx], self.sparse_idx), shape=self.shape)
+        elif self.sparse_idx is not None and sparse_path:
+            self._value = val.data
+        else:
+            self._value = val
 
     @property
-    def value(self):
-        """NumPy.ndarray or None: The numeric value of the parameter.
-        """
-        return self._value
+    def value(self) -> Optional[np.ndarray]:
+        """The numeric value of the expression."""
+        if self.sparse_idx is None:
+            return self._value
+        else:
+            warnings.warn('Reading from a sparse CVXPY expression via `.value` is discouraged.'
+                          ' Use `.value_sparse` instead', RuntimeWarning, 1)
+            if self._value is None:
+                return None
+            val = np.zeros(self.shape)
+            val[self.sparse_idx] = self._value.data
+            return val
 
     @value.setter
     def value(self, val) -> None:
+        if self.sparse_idx is not None:
+            warnings.warn('Writing to a sparse CVXPY expression via `.value` is discouraged.'
+                          ' Use `.value_sparse` instead', RuntimeWarning, 1)
         self.save_value(self._validate_value(val))
 
+    @property
+    def value_sparse(self) -> Optional[...]:
+        """The numeric value of the expression if it is a sparse variable."""
+        if self._value is None:
+            return None
+        if isinstance(self._value, np.ndarray):
+            return sp.coo_array((self._value, self.sparse_idx), shape=self.shape)
+        else:
+            return self._value
+
+    @value_sparse.setter
+    def value_sparse(self, val) -> None:
+        if isinstance(val, sp.spmatrix):
+            val = sp.coo_array(val)
+        elif not isinstance(val, sp.coo_array):
+            if isinstance(val, (np.ndarray)) \
+                    and val.shape == (len(self.sparse_idx[0]),):
+                raise ValueError(
+                    'Invalid type for assigning value_sparse.'
+                    'Try using `'
+                    'expr.value_sparse = scipy.sparse.coo_array((values, expr)) instead.')
+            raise ValueError(
+                'Invalid type for assigning value_sparse.'
+                f'Recieved: {type(val)} Expected scipy.sparse.coo_array.'
+                f' Instantiate with scipy.sparse.coo_array((value_array, coordinates))'
+                )
+        self.save_value(self._validate_value(val, True), True)
+
+
+
     def project_and_assign(self, val) -> None:
         """Project and assign a value to the variable.
         """
         self.save_value(self.project(val))
 
-    def _validate_value(self, val):
+    def _validate_value(self, val, sparse_path=False):
         """Check that the value satisfies the leaf's symbolic attributes.
 
         Parameters
@@ -391,7 +520,16 @@ class Leaf(expression.Expression):
                     "Invalid dimensions %s for %s value." %
                     (intf.shape(val), self.__class__.__name__)
                 )
-            projection = self.project(val)
+            if sparse_path:
+                val.sum_duplicates()
+                coords_val = get_coords(val)
+                if len(coords_val) != len(self.sparse_idx) or \
+                     any((a != b).any() for a, b in zip(coords_val, self.sparse_idx)):
+                    raise ValueError(
+                        'Invalid sparsity pattern %s for %s value.' %
+                        (get_coords(val), self.__class__.__name__)
+                    )
+            projection = self.project(val, sparse_path)
             # ^ might be a numpy array, or sparse scipy matrix.
             delta = np.abs(val - projection)
             # ^ might be a numpy array, scipy matrix, or sparse scipy matrix.
@@ -425,6 +563,8 @@ class Leaf(expression.Expression):
                     attr_str = 'nonpositive'
                 elif self.attributes['neg']:
                     attr_str = 'negative'
+                elif self.attributes['sparsity']:
+                    attr_str = 'zero outside of sparsity pattern'
                 elif self.attributes['diag']:
                     attr_str = 'diagonal'
                 elif self.attributes['PSD']:
@@ -433,6 +573,8 @@ class Leaf(expression.Expression):
                     attr_str = 'negative semidefinite'
                 elif self.attributes['imag']:
                     attr_str = 'imaginary'
+                elif self.attributes['bounds']:
+                    attr_str = 'in bounds'
                 else:
                     attr_str = ([k for (k, v) in self.attributes.items() if v] + ['real'])[0]
                 raise ValueError(
@@ -441,23 +583,27 @@ class Leaf(expression.Expression):
         return val
 
     def is_psd(self) -> bool:
-        """Is the expression a positive semidefinite matrix?
-        """
+        """Is the expression a positive semidefinite matrix?"""
         return self.attributes['PSD']
 
     def is_nsd(self) -> bool:
-        """Is the expression a negative semidefinite matrix?
-        """
+        """Is the expression a negative semidefinite matrix?"""
         return self.attributes['NSD']
 
+    def is_diag(self) -> bool:
+        """Is the expression a diagonal matrix?"""
+        return self.attributes['diag']
+
     def is_quadratic(self) -> bool:
-        """Leaf nodes are always quadratic.
-        """
+        """Leaf nodes are always quadratic."""
         return True
 
+    def has_quadratic_term(self) -> bool:
+        """Leaf nodes are not quadratic terms."""
+        return False
+
     def is_pwl(self) -> bool:
-        """Leaf nodes are always piecewise linear.
-        """
+        """Leaf nodes are always piecewise linear."""
         return True
 
     def is_dpp(self, context: str = 'dcp') -> bool:
@@ -467,5 +613,49 @@ class Leaf(expression.Expression):
         """
         return True
 
-    def atoms(self) -> List['Atom']:
+    def atoms(self) -> list[Atom]:
         return []
+
+    def _ensure_valid_bounds(self, value) -> Iterable | None:
+        # In case for a constant or no bounds
+        if value is None:
+            return
+
+        # Check that bounds is an iterable of two items
+        if not isinstance(value, Iterable) or len(value) != 2:
+            raise ValueError("Bounds should be a list of two items.")
+
+        # Check that bounds contains two scalars or two arrays with matching shapes.
+        for val in value:
+            valid_array = isinstance(val, np.ndarray) and val.shape == self.shape
+            if not (val is None or np.isscalar(val) or valid_array):
+                raise ValueError(
+                    "Bounds should be None, scalars, or arrays with the "
+                    "same dimensions as the variable/parameter."
+                )
+
+        # Promote upper and lower bounds to arrays.
+        none_bounds = [-np.inf, np.inf]
+        for idx, val in enumerate(value):
+            if val is None:
+                value[idx] = np.full(self.shape, none_bounds[idx])
+            elif np.isscalar(val):
+                value[idx] = np.full(self.shape, val)
+
+        # Upper bound cannot be -np.inf.
+        if np.any(value[1] == -np.inf):
+            raise ValueError("-np.inf is not feasible as an upper bound.")
+        # Lower bound cannot be np.inf.
+        if np.any(value[0] == np.inf):
+            raise ValueError("np.inf is not feasible as a lower bound.")
+
+        # Check that upper_bound >= lower_bound
+        if np.any(value[0] > value[1]):
+            raise ValueError("Invalid bounds: some upper bounds are less "
+                             "than corresponding lower bounds.")
+
+        if np.any(np.isnan(value[0])) or np.any(np.isnan(value[1])):
+            raise ValueError("np.nan is not feasible as lower "
+                                "or upper bound.")
+
+        return value
