@@ -13,10 +13,12 @@ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 """
+from __future__ import annotations
 
 import time
 import warnings
 from collections import namedtuple
+from dataclasses import dataclass
 from typing import Dict, List, Optional, Union
 
 import numpy as np
@@ -46,9 +48,12 @@ from cvxpy.reductions.solvers.conic_solvers.conic_solver import ConicSolver
 from cvxpy.reductions.solvers.defines import SOLVER_MAP_CONIC, SOLVER_MAP_QP
 from cvxpy.reductions.solvers.qp_solvers.qp_solver import QpSolver
 from cvxpy.reductions.solvers.solver import Solver
-from cvxpy.reductions.solvers.solving_chain import (SolvingChain,
-                                                    construct_solving_chain,)
+from cvxpy.reductions.solvers.solving_chain import (
+    SolvingChain,
+    construct_solving_chain,
+)
 from cvxpy.settings import SOLVERS
+from cvxpy.utilities import debug_tools
 from cvxpy.utilities.deterministic import unique_list
 
 SolveResult = namedtuple(
@@ -102,8 +107,8 @@ class Cache:
         self.param_prog = None
         self.inverse_data = None
 
-    def make_key(self, solver, gp, ignore_dpp):
-        return (solver, gp, ignore_dpp)
+    def make_key(self, solver, gp, ignore_dpp, use_quad_obj):
+        return (solver, gp, ignore_dpp, use_quad_obj)
 
     def gp(self):
         return self.key is not None and self.key[1]
@@ -148,7 +153,17 @@ class Problem(u.Canonical):
             raise error.DCPError("Problem objective must be Minimize or Maximize.")
         # Constraints and objective are immutable.
         self._objective = objective
+        # Raise warning if objective has too many subexpressions.
+        if debug_tools.node_count(self._objective) >= debug_tools.MAX_NODES:
+            warnings.warn("Objective contains too many subexpressions. "
+                          "Consider vectorizing your CVXPY code to speed up compilation.")
         self._constraints = [_validate_constraint(c) for c in constraints]
+        # Raise warning if constraint has too many subexpressions.
+        for i, constraint in enumerate(self._constraints):
+            if debug_tools.node_count(constraint) >= debug_tools.MAX_NODES:
+                warnings.warn(f"Constraint #{i} contains too many subexpressions. "
+                              "Consider vectorizing your CVXPY code to speed up compilation.")
+
         self._value = None
         self._status: Optional[str] = None
         self._solution = None
@@ -242,6 +257,20 @@ class Problem(u.Canonical):
         return all(
           expr.is_dcp(dpp) for expr in self.constraints + [self.objective])
 
+    @perf.compute_once
+    def _max_ndim(self) -> int:
+        """
+        Returns the maximum number of dimensions of any argument in the problem.
+        """
+        return max(expr._max_ndim() for expr in self.constraints + [self.objective.expr])
+
+    @perf.compute_once
+    def _supports_cpp(self) -> bool:
+        """
+        Returns True if all the arguments in the problem support cpp backend.
+        """
+        return all(expr._all_support_cpp() for expr in self.constraints + [self.objective.expr])
+
     @perf.compute_once
     def is_dgp(self, dpp: bool = False) -> bool:
         """Does the problem satisfy DGP rules?
@@ -394,6 +423,61 @@ class Problem(u.Canonical):
         """
         return self._solver_stats
 
+    @property
+    def compilation_time(self) -> float | None:
+        """float : The number of seconds it took to compile the problem the
+                   last time it was compiled.
+        """
+        return self._compilation_time
+
+    def _solve_solver_path(self, solve_func, solvers:List[tuple[str, Dict] | str],
+                                args, kwargs):
+        """Solve a problem using multiple solvers.
+
+        Arguments
+        ---------
+        solvers : list of (str, dict) tuples or strings.
+            The solvers to use. For example, ['SCS', ('OSQP', {'max_iter':10000})]
+        kwargs : keywords, optional
+            Additional solver specific arguments.
+
+        Returns
+        -------
+        solution : Solution
+            The solution of the first solver that succeeds.
+
+        Raises
+        ------
+        SolverError
+            If all solvers fail to find a solution.
+        ValueError
+            If the input solvers format is incorrect.
+        """
+
+        ENTRY_ERROR_MSG ="Solver path entry must be list of str or tuple[str, dict[str, Any]]"
+        if not isinstance(solvers, list):
+            raise ValueError(ENTRY_ERROR_MSG)
+        if not solvers:
+            raise ValueError("Solver path must contain at least one solver.")
+        for solver in solvers:
+            try:
+                if isinstance(solver, str):
+                    solver_name = solver
+                    solution = solve_func(self, *args, solver=solver_name, **kwargs)
+                elif isinstance(solver, tuple) and len(solver) == 2:
+                        solver_name, solver_kwargs = solver
+                        if not isinstance(solver_name, str) or not isinstance(solver_kwargs, dict):
+                            raise ValueError(ENTRY_ERROR_MSG)
+                        solution = solve_func(
+                            self, *args, solver=solver_name, **solver_kwargs, **kwargs)
+                else:
+                    raise ValueError(ENTRY_ERROR_MSG)
+                s.LOGGER.info("Solver %s succeeds", solver_name)
+                return solution
+            except error.SolverError as e:
+                s.LOGGER.info("Solver %s failed: %s", solver_name, e)
+        raise error.SolverError(f"All solvers failed: {solvers}")
+
     def solve(self, *args, **kwargs):
         """Compiles and solves the problem using the specified method.
 
@@ -403,7 +487,12 @@ class Problem(u.Canonical):
         Arguments
         ---------
         solver : str, optional
-            The solver to use. For example, 'ECOS', 'SCS', or 'OSQP'.
+            The solver to use. For example, 'CLARABEL', 'SCS', or 'OSQP'.
+        solver_path : list of (str, dict) tuples or strings, optional
+            The solvers to use with optional arguments.
+            The function tries the solvers in the given order and
+            returns the first solver's solution that succeeds.
+            For example, ['SCS', ('OSQP', {'max_iter':10000})]
         verbose : bool, optional
             Overrides the default of hiding solver output, and prints
             logging information describing CVXPY's compilation process.
@@ -478,6 +567,13 @@ class Problem(u.Canonical):
             solve_func = Problem.REGISTERED_SOLVE_METHODS[func_name]
         else:
             solve_func = Problem._solve
+        solver_path = kwargs.pop("solver_path", None)
+        if solver_path is not None:
+            solver = kwargs.get("solver", None)
+            if solver is not None:
+                raise ValueError(
+                    "Cannot specify both 'solver' and 'solver_path'. Please choose one.")
+            return self._solve_solver_path(solve_func,solver_path, args, kwargs)
         return solve_func(self, *args, **kwargs)
 
     @classmethod
@@ -499,7 +595,9 @@ class Problem(u.Canonical):
         gp: bool = False,
         enforce_dpp: bool = False,
         ignore_dpp: bool = False,
-        verbose: bool = False
+        verbose: bool = False,
+        canon_backend: str | None = None,
+        solver_opts: Optional[dict] = None
     ):
         """Returns the problem data used in the call to the solver.
 
@@ -577,8 +675,17 @@ class Problem(u.Canonical):
         ignore_dpp : bool, optional
             When True, DPP problems will be treated as non-DPP,
             which may speed up compilation. Defaults to False.
+        canon_backend : str, optional
+            'CPP' (default) | 'SCIPY'
+            Specifies which backend to use for canonicalization, which can affect
+            compilation time. Defaults to None, i.e., selecting the default
+            backend.
         verbose : bool, optional
             If True, print verbose output related to problem compilation.
+        solver_opts : dict, optional
+            A dict of options that will be passed to the specific solver.
+            In general, these options will override any default settings
+            imposed by cvxpy.
 
         Returns
         -------
@@ -594,20 +701,28 @@ class Problem(u.Canonical):
         cvxpy.error.DPPError
             Raised if DPP settings are invalid.
         """
+
         # Invalid DPP setting.
         # Must be checked here to avoid cache issues.
         if enforce_dpp and ignore_dpp:
             raise DPPError("Cannot set enforce_dpp = True and ignore_dpp = True.")
 
         start = time.time()
-        # Cache includes ignore_dpp because it alters compilation.
-        key = self._cache.make_key(solver, gp, ignore_dpp)
+        # Cache includes ignore_dpp and solver_opts['use_quad_obj']
+        # because they alter compilation.
+        if solver_opts is None:
+            use_quad_obj = None
+        else:
+            use_quad_obj = solver_opts.get('use_quad_obj', None)
+        key = self._cache.make_key(solver, gp, ignore_dpp, use_quad_obj)
         if key != self._cache.key:
             self._cache.invalidate()
             solving_chain = self._construct_chain(
                 solver=solver, gp=gp,
                 enforce_dpp=enforce_dpp,
-                ignore_dpp=ignore_dpp)
+                ignore_dpp=ignore_dpp,
+                canon_backend=canon_backend,
+                solver_opts=solver_opts)
             self._cache.key = key
             self._cache.solving_chain = solving_chain
             self._solver_cache = {}
@@ -712,7 +827,6 @@ class Problem(u.Canonical):
                       'conic_solvers': []}
         if isinstance(solver, Solver):
             return self._add_custom_solver_candidates(solver)
-
         if solver is not None:
             if solver not in slv_def.INSTALLED_SOLVERS:
                 raise error.SolverError("The solver %s is not installed." % solver)
@@ -798,7 +912,7 @@ class Problem(u.Canonical):
         """
         if custom_solver.name() in SOLVERS:
             message = "Custom solvers must have a different name than the officially supported ones"
-            raise(error.SolverError(message))
+            raise error.SolverError(message)
 
         candidates = {'qp_solvers': [], 'conic_solvers': []}
         if not self.is_mixed_integer() or custom_solver.MIP_CAPABLE:
@@ -811,8 +925,13 @@ class Problem(u.Canonical):
         return candidates
 
     def _construct_chain(
-        self, solver: Optional[str] = None, gp: bool = False,
-        enforce_dpp: bool = False, ignore_dpp: bool = False
+            self,
+            solver: Optional[str] = None,
+            gp: bool = False,
+            enforce_dpp: bool = False,
+            ignore_dpp: bool = False,
+            canon_backend: str | None = None,
+            solver_opts: Optional[dict] = None
     ) -> SolvingChain:
         """
         Construct the chains required to reformulate and solve the problem.
@@ -835,6 +954,13 @@ class Problem(u.Canonical):
         ignore_dpp : bool, optional
             When True, DPP problems will be treated as non-DPP,
             which may speed up compilation. Defaults to False.
+        canon_backend : str, optional
+            'CPP' (default) | 'SCIPY'
+            Specifies which backend to use for canonicalization, which can affect
+            compilation time. Defaults to None, i.e., selecting the default
+            backend.
+        solver_opts: dict, optional
+            Additional arguments to pass to the solver.
 
         Returns
         -------
@@ -844,7 +970,10 @@ class Problem(u.Canonical):
         self._sort_candidate_solvers(candidate_solvers)
         return construct_solving_chain(self, candidate_solvers, gp=gp,
                                        enforce_dpp=enforce_dpp,
-                                       ignore_dpp=ignore_dpp)
+                                       ignore_dpp=ignore_dpp,
+                                       canon_backend=canon_backend,
+                                       solver_opts=solver_opts,
+                                       specified_solver=solver)
 
     @staticmethod
     def _sort_candidate_solvers(solvers) -> None:
@@ -883,6 +1012,7 @@ class Problem(u.Canonical):
                requires_grad: bool = False,
                enforce_dpp: bool = False,
                ignore_dpp: bool = False,
+               canon_backend: str | None = None,
                **kwargs):
         """Solves a DCP compliant optimization problem.
 
@@ -913,6 +1043,11 @@ class Problem(u.Canonical):
         ignore_dpp : bool, optional
             When True, DPP problems will be treated as non-DPP,
             which may speed up compilation. Defaults to False.
+        canon_backend : str, optional
+            'CPP' (default) | 'SCIPY'
+            Specifies which backend to use for canonicalization, which can affect
+            compilation time. Defaults to None, i.e., selecting the default
+            backend.
         kwargs : dict, optional
             A dict of options that will be passed to the specific solver.
             In general, these options will override any default settings
@@ -924,6 +1059,7 @@ class Problem(u.Canonical):
             The optimal value for the problem, or a string indicating
             why the problem could not be solved.
         """
+
         if verbose:
             print(_HEADER)
 
@@ -935,12 +1071,14 @@ class Problem(u.Canonical):
                     "values before solving a problem." % parameter.name())
 
         if verbose:
-            n_variables = sum(np.prod(v.shape) for v in self.variables())
+            n_variables = sum(len(v.sparse_idx[0]) if v.sparse_idx else
+                              np.prod(v.shape) for v in self.variables())
+            n_constraints = sum(np.prod(c.shape) for c in self.constraints)
             n_parameters = sum(np.prod(p.shape) for p in self.parameters())
             s.LOGGER.info(
                     'Your problem has %d variables, '
                     '%d constraints, and ' '%d parameters.',
-                    n_variables, len(self.constraints), n_parameters)
+                    n_variables, n_constraints, n_parameters)
             curvatures = []
             if self.is_dcp():
                 curvatures.append('DCP')
@@ -958,7 +1096,9 @@ class Problem(u.Canonical):
             s.LOGGER.info(
                     'CVXPY will first compile your problem; then, it will '
                     'invoke a numerical solver to obtain a solution.')
-
+            s.LOGGER.info(
+                    "Your problem is compiled with the %s canonicalization backend.",
+                    s.DEFAULT_CANON_BACKEND if canon_backend is None else canon_backend)
         if requires_grad:
             dpp_context = 'dgp' if gp else 'dcp'
             if qcp:
@@ -1005,7 +1145,8 @@ class Problem(u.Canonical):
                 return self.value
 
         data, solving_chain, inverse_data = self.get_problem_data(
-            solver, gp, enforce_dpp, ignore_dpp, verbose)
+            solver, gp, enforce_dpp, ignore_dpp, verbose, canon_backend, kwargs
+        )
 
         if verbose:
             print(_NUM_SOLVER_STR)
@@ -1013,15 +1154,18 @@ class Problem(u.Canonical):
                     'Invoking solver %s  to obtain a solution.',
                     solving_chain.reductions[-1].name())
         start = time.time()
+        solver_verbose = kwargs.pop('solver_verbose', verbose)
+        if solver_verbose and (not verbose):
+            print(_NUM_SOLVER_STR)
         solution = solving_chain.solve_via_data(
-            self, data, warm_start, verbose, kwargs)
+            self, data, warm_start, solver_verbose, kwargs)
         end = time.time()
         self._solve_time = end - start
         self.unpack_results(solution, solving_chain, inverse_data)
         if verbose:
             print(_FOOTER)
             s.LOGGER.info('Problem status: %s', self.status)
-            val = self.value if self.value is not None else np.NaN
+            val = self.value if self.value is not None else np.nan
             s.LOGGER.info('Optimal value: %.3e', val)
             s.LOGGER.info('Compilation took %.3e seconds', self._compilation_time)
             s.LOGGER.info(
@@ -1348,7 +1492,7 @@ class Problem(u.Canonical):
                     "information.")
 
         self.unpack(solution)
-        self._solver_stats = SolverStats(self._solution.attr,
+        self._solver_stats = SolverStats.from_dict(self._solution.attr,
                                          chain.solver.name())
 
     def __str__(self) -> str:
@@ -1413,6 +1557,7 @@ class Problem(u.Canonical):
     __truediv__ = __div__
 
 
+@dataclass
 class SolverStats:
     """Reports some of the miscellaneous information that is returned
     by the solver after solving but that is not captured directly by
@@ -1433,22 +1578,36 @@ class SolverStats:
         returned directly from the solver, without modification by CVXPY.
         This object may be a dict, or a custom Python object.
     """
-    def __init__(self, results_dict, solver_name: str) -> None:
-        self.solver_name = solver_name
-        self.solve_time = None
-        self.setup_time = None
-        self.num_iters = None
-        self.extra_stats = None
 
-        if s.SOLVE_TIME in results_dict:
-            self.solve_time = results_dict[s.SOLVE_TIME]
-        if s.SETUP_TIME in results_dict:
-            self.setup_time = results_dict[s.SETUP_TIME]
-        if s.NUM_ITERS in results_dict:
-            self.num_iters = results_dict[s.NUM_ITERS]
-        if s.EXTRA_STATS in results_dict:
-            self.extra_stats = results_dict[s.EXTRA_STATS]
+    solver_name: str
+    solve_time: Optional[float] = None
+    setup_time: Optional[float] = None
+    num_iters: Optional[int] = None
+    extra_stats: Optional[dict] = None
 
+    @classmethod
+    def from_dict(cls, attr: dict, solver_name: str) -> "SolverStats":
+        """Construct a SolverStats object from a dictionary of attributes.
+
+        Parameters
+        ----------
+        attr : dict
+            A dictionary of attributes returned by the solver.
+        solver_name : str
+            The name of the solver.
+
+        Returns
+        -------
+        SolverStats
+            A SolverStats object.
+        """
+        return cls(
+            solver_name,
+            solve_time=attr.get(s.SOLVE_TIME),
+            setup_time=attr.get(s.SETUP_TIME),
+            num_iters=attr.get(s.NUM_ITERS),
+            extra_stats=attr.get(s.EXTRA_STATS),
+        )
 
 class SizeMetrics:
     """Reports various metrics regarding the problem.
