@@ -44,6 +44,19 @@ class LinearOperator:
     def __call__(self, X):
         return self._matmul(X)
 
+class IdentityOperator(LinearOperator):
+    """A wrapper for the identity operator."""
+    def __init__(self, n):
+        self.shape = (n,n)
+    def __call__(self, X):
+        return X
+
+class NegativeIdentityOperator(LinearOperator):
+    """A wrapper for the negative identity operator."""
+    def __init__(self, n):
+        self.shape = (n,n)
+    def __call__(self, X):
+        return -X
 
 def as_linear_operator(linear_op):
     if isinstance(linear_op, LinearOperator):
@@ -102,6 +115,11 @@ class ConicSolver(Solver):
     # Whenever a solver uses this convention, EXP_CONE_ORDER should be [0, 1, 2].
     EXP_CONE_ORDER = None
 
+    def supports_quad_obj(self) -> bool:
+        """By default does not support a quadratic objective.
+        """
+        return False
+
     def accepts(self, problem):
         return (isinstance(problem, ParamConeProg)
                 and (self.MIP_CAPABLE or not problem.is_mixed_integer())
@@ -147,7 +165,8 @@ class ConicSolver(Solver):
         # Default is identity.
         return sp.eye(constr.size, format='csc')
 
-    def format_constraints(self, problem, exp_cone_order):
+    @classmethod
+    def format_constraints(cls, problem, exp_cone_order):
         """
         Returns a ParamConeProg whose problem data tensors will yield the
         coefficient "A" and offset "b" for the constraint in the following
@@ -182,9 +201,9 @@ class ConicSolver(Solver):
         for constr in problem.constraints:
             total_height = sum([arg.size for arg in constr.args])
             if type(constr) == Zero:
-                restruct_mat.append(-sp.eye(constr.size, format='csr'))
+                restruct_mat.append(NegativeIdentityOperator(constr.size))
             elif type(constr) == NonNeg:
-                restruct_mat.append(sp.eye(constr.size, format='csr'))
+                restruct_mat.append(IdentityOperator(constr.size))
             elif type(constr) == SOC:
                 # Group each t row with appropriate X rows.
                 assert constr.axis == 0, 'SOC must be lowered to axis == 0'
@@ -231,7 +250,7 @@ class ConicSolver(Solver):
                     arg_mats.append(space_mat)
                 restruct_mat.append(sp.hstack(arg_mats))
             elif type(constr) == PSD:
-                restruct_mat.append(self.psd_format_mat(constr))
+                restruct_mat.append(cls.psd_format_mat(constr))
             else:
                 raise ValueError("Unsupported constraint type.")
 
@@ -243,9 +262,8 @@ class ConicSolver(Solver):
             # this is equivalent to but _much_ faster than:
             #    restruct_mat_rep = sp.block_diag([restruct_mat]*(problem.x.size + 1))
             #    restruct_A = restruct_mat_rep * problem.A
-            unspecified, remainder = divmod(problem.A.shape[0] *
-                                            problem.A.shape[1],
-                                            restruct_mat.shape[1])
+            unspecified, _ = np.divmod(problem.A.shape[0] * problem.A.shape[1],
+                                        restruct_mat.shape[1], dtype=np.int64)
             reshaped_A = problem.A.reshape(restruct_mat.shape[1],
                                            unspecified, order='F').tocsr()
             restructured_A = restruct_mat(reshaped_A).tocoo()
@@ -258,15 +276,20 @@ class ConicSolver(Solver):
                 problem.A.shape[1], order='F')
         else:
             restructured_A = problem.A
-        new_param_cone_prog = ParamConeProg(problem.c,
-                                            problem.x,
-                                            restructured_A,
-                                            problem.variables,
-                                            problem.var_id_to_col,
-                                            problem.constraints,
-                                            problem.parameters,
-                                            problem.param_id_to_col,
-                                            formatted=True)
+        new_param_cone_prog = ParamConeProg(
+            problem.c,
+            problem.x,
+            restructured_A,
+            problem.variables,
+            problem.var_id_to_col,
+            problem.constraints,
+            problem.parameters,
+            problem.param_id_to_col,
+            P=problem.P,
+            formatted=True,
+            lower_bounds=problem.lower_bounds,
+            upper_bounds=problem.upper_bounds,
+        )
         return new_param_cone_prog
 
     def invert(self, solution, inverse_data):
@@ -333,7 +356,11 @@ class ConicSolver(Solver):
 
         # Apply parameter values.
         # Obtain A, b such that Ax + s = b, s \in cones.
-        c, d, A, b = problem.apply_parameters()
+        if problem.P is None:
+            c, d, A, b = problem.apply_parameters()
+        else:
+            P, c, d, A, b = problem.apply_parameters(quad_obj=True)
+            data[s.P] = P
         data[s.C] = c
         inv_data[s.OFFSET] = d
         data[s.A] = -A
