@@ -22,7 +22,9 @@ from cvxpy.reductions.solution import Solution, failure_solution
 from cvxpy.reductions.solvers import utilities
 from cvxpy.reductions.solvers.conic_solvers.conic_solver import ConicSolver
 from cvxpy.reductions.solvers.conic_solvers.ecos_conif import (
-    ECOS, dims_to_solver_dict,)
+    ECOS,
+    dims_to_solver_dict,
+)
 
 
 class ECOS_BB(ECOS):
@@ -31,6 +33,7 @@ class ECOS_BB(ECOS):
 
     # Solver capabilities.
     MIP_CAPABLE = True
+    MI_SUPPORTED_CONSTRAINTS = ECOS.SUPPORTED_CONSTRAINTS
 
     # Exit flags from ECOS_BB
     # ECOS_BB found optimal solution.
