@@ -73,8 +73,7 @@ class ECOS(ConicSolver):
     def import_solver(self) -> None:
         """Imports the solver.
         """
-        import ecos
-        ecos  # For flake8
+        import ecos  # noqa F401
 
     def name(self):
         """The name of the solver.
@@ -116,13 +115,13 @@ class ECOS(ConicSolver):
         data[s.A] = -A[:len_eq]
         if data[s.A].shape[0] == 0:
             data[s.A] = None
-        data[s.B] = b[:len_eq].flatten()
+        data[s.B] = b[:len_eq].flatten(order='F')
         if data[s.B].shape[0] == 0:
             data[s.B] = None
         data[s.G] = -A[len_eq:]
         if 0 in data[s.G].shape:
             data[s.G] = None
-        data[s.H] = b[len_eq:].flatten()
+        data[s.H] = b[len_eq:].flatten(order='F')
         if 0 in data[s.H].shape:
             data[s.H] = None
         return data, inv_data
