@@ -38,8 +38,7 @@ class GLPK(CVXOPT):
     def import_solver(self) -> None:
         """Imports the solver.
         """
-        from cvxopt import glpk
-        glpk  # For flake8
+        from cvxopt import glpk  # noqa F401
 
     def invert(self, solution, inverse_data):
         """Returns the solution to the original problem given the inverse_data.
