@@ -18,10 +18,12 @@ import numpy as np
 import scipy.sparse as sp
 
 import cvxpy.settings as s
-from cvxpy.constraints import NonPos, Zero
+from cvxpy.constraints import NonNeg, Zero
 from cvxpy.reductions.cvx_attr2constr import convex_attributes
-from cvxpy.reductions.qp2quad_form.qp_matrix_stuffing import (ConeDims,
-                                                              ParamQuadProg,)
+from cvxpy.reductions.qp2quad_form.qp_matrix_stuffing import (
+    ConeDims,
+    ParamQuadProg,
+)
 from cvxpy.reductions.solvers.solver import Solver
 from cvxpy.reductions.utilities import group_constraints
 
@@ -30,8 +32,8 @@ class QpSolver(Solver):
     """
     A QP solver interface.
     """
-    # Every QP solver supports Zero and NonPos constraints.
-    SUPPORTED_CONSTRAINTS = [Zero, NonPos]
+    # Every QP solver supports Zero and NonNeg constraints.
+    SUPPORTED_CONSTRAINTS = [Zero, NonNeg]
 
     # Some solvers cannot solve problems that do not have constraints.
     # For such solvers, REQUIRES_CONSTR should be set to True.
@@ -79,7 +81,7 @@ class QpSolver(Solver):
         # Get number of variables
         n = problem.x.size
         len_eq = data[QpSolver.DIMS].zero
-        len_leq = data[QpSolver.DIMS].nonpos
+        len_leq = data[QpSolver.DIMS].nonneg
 
         if len_eq > 0:
             A = AF[:len_eq, :]
@@ -88,8 +90,8 @@ class QpSolver(Solver):
             A, b = sp.csr_matrix((0, n)), -np.array([])
 
         if len_leq > 0:
-            F = AF[len_eq:, :]
-            g = -bg[len_eq:]
+            F = -AF[len_eq:, :]
+            g = bg[len_eq:]
         else:
             F, g = sp.csr_matrix((0, n)), -np.array([])
 
@@ -102,6 +104,8 @@ class QpSolver(Solver):
         data[s.G] = g
         data[s.BOOL_IDX] = [t[0] for t in problem.x.boolean_idx]
         data[s.INT_IDX] = [t[0] for t in problem.x.integer_idx]
+        data[s.LOWER_BOUNDS] = problem.lower_bounds
+        data[s.UPPER_BOUNDS] = problem.upper_bounds
         data['n_var'] = n
         data['n_eq'] = A.shape[0]
         data['n_ineq'] = F.shape[0]
