@@ -30,10 +30,9 @@ class Solver(Reduction):
     #   There are separate ConeDims classes for cone programs vs QPs.
     #   See cone_matrix_stuffing.py and qp_matrix_stuffing.py for details.
 
-    __metaclass__ = abc.ABCMeta
-
     # Solver capabilities.
     MIP_CAPABLE = False
+    BOUNDED_VARIABLES = False
 
     # Keys for inverse data.
     VAR_ID = 'var_id'
