@@ -1,31 +1,86 @@
+from __future__ import annotations
+
 import warnings
-from typing import Any, List
 
 import numpy as np
 
 from cvxpy.atoms import EXP_ATOMS, NONPOS_ATOMS, PSD_ATOMS, SOC_ATOMS
-from cvxpy.constraints import (PSD, SOC, Equality, ExpCone, Inequality, NonNeg,
-                               NonPos, PowCone3D, Zero,)
+from cvxpy.constraints import (
+    PSD,
+    SOC,
+    Equality,
+    ExpCone,
+    FiniteSet,
+    Inequality,
+    NonNeg,
+    NonPos,
+    PowCone3D,
+    Zero,
+)
+from cvxpy.constraints.exponential import OpRelEntrConeQuad, RelEntrConeQuad
 from cvxpy.error import DCPError, DGPError, DPPError, SolverError
 from cvxpy.problems.objective import Maximize
 from cvxpy.reductions.chain import Chain
 from cvxpy.reductions.complex2real import complex2real
-from cvxpy.reductions.cone2cone.exotic2common import (EXOTIC_CONES,
-                                                      Exotic2Common,)
+from cvxpy.reductions.cone2cone.approximations import APPROX_CONES, QuadApprox
+from cvxpy.reductions.cone2cone.exotic2common import (
+    EXOTIC_CONES,
+    Exotic2Common,
+)
+from cvxpy.reductions.cone2cone.soc2psd import SOC2PSD
 from cvxpy.reductions.cvx_attr2constr import CvxAttr2Constr
 from cvxpy.reductions.dcp2cone.cone_matrix_stuffing import ConeMatrixStuffing
 from cvxpy.reductions.dcp2cone.dcp2cone import Dcp2Cone
 from cvxpy.reductions.dgp2dcp.dgp2dcp import Dgp2Dcp
+from cvxpy.reductions.discrete2mixedint.valinvec2mixedint import (
+    Valinvec2mixedint,
+)
 from cvxpy.reductions.eval_params import EvalParams
 from cvxpy.reductions.flip_objective import FlipObjective
 from cvxpy.reductions.qp2quad_form import qp2symbolic_qp
 from cvxpy.reductions.qp2quad_form.qp_matrix_stuffing import QpMatrixStuffing
+from cvxpy.reductions.reduction import Reduction
 from cvxpy.reductions.solvers import defines as slv_def
 from cvxpy.reductions.solvers.constant_solver import ConstantSolver
 from cvxpy.reductions.solvers.solver import Solver
-from cvxpy.settings import PARAM_THRESHOLD
+from cvxpy.settings import (
+    CLARABEL,
+    CPP_CANON_BACKEND,
+    NUMPY_CANON_BACKEND,
+    PARAM_THRESHOLD,
+    SCIPY_CANON_BACKEND,
+)
 from cvxpy.utilities.debug_tools import build_non_disciplined_error_msg
 
+DPP_ERROR_MSG = (
+    "You are solving a parameterized problem that is not DPP. "
+    "Because the problem is not DPP, subsequent solves will not be "
+    "faster than the first one. For more information, see the "
+    "documentation on Disciplined Parametrized Programming, at "
+    "https://www.cvxpy.org/tutorial/dpp/index.html"
+)
+
+ECOS_DEP_DEPRECATION_MSG = (
+    """
+    You specified your problem should be solved by ECOS. Starting in
+    CXVPY 1.6.0, ECOS will no longer be installed by default with CVXPY.
+    Please either add ECOS as an explicit install dependency to your project
+    or switch to our new default solver, Clarabel, by either not specifying a
+    solver argument or specifying ``solver=cp.CLARABEL``. To suppress this
+    warning while continuing to use ECOS, you can filter this warning using
+    Python's ``warnings`` module until you are using 1.6.0.
+    """
+)
+
+ECOS_DEPRECATION_MSG = (
+    """
+    Your problem is being solved with the ECOS solver by default. Starting in
+    CVXPY 1.5.0, Clarabel will be used as the default solver instead. To continue
+    using ECOS, specify the ECOS solver explicitly using the ``solver=cp.ECOS``
+    argument to the ``problem.solve`` method.
+    """
+)
+
 
 def _is_lp(self):
     """Is problem a linear program?
@@ -49,7 +104,8 @@ def _solve_as_qp(problem, candidates):
     return candidates['qp_solvers'] and qp2symbolic_qp.accepts(problem)
 
 
-def _reductions_for_problem_class(problem, candidates, gp: bool = False) -> List[Any]:
+def _reductions_for_problem_class(problem, candidates, gp: bool = False, solver_opts=None) \
+        -> list[Reduction]:
     """
     Builds a chain that rewrites a problem into an intermediate
     representation suitable for numeric reductions.
@@ -107,23 +163,31 @@ def _reductions_for_problem_class(problem, candidates, gp: bool = False) -> List
     if type(problem.objective) == Maximize:
         reductions += [FlipObjective()]
 
-    if _solve_as_qp(problem, candidates):
-        reductions += [CvxAttr2Constr(), qp2symbolic_qp.Qp2SymbolicQp()]
-    else:
-        # Canonicalize it to conic problem.
-        if not candidates['conic_solvers']:
-            raise SolverError("Problem could not be reduced to a QP, and no "
-                              "conic solvers exist among candidate solvers "
-                              "(%s)." % candidates)
-        else:
-            reductions += [Dcp2Cone(), CvxAttr2Constr()]
+    # Special reduction for finite set constraint,
+    # used by both QP and conic pathways.
+    constr_types = {type(c) for c in problem.constraints}
+    if FiniteSet in constr_types:
+        reductions += [Valinvec2mixedint()]
+
+    use_quad = True if solver_opts is None else solver_opts.get('use_quad_obj', True)
+    valid_qp = _solve_as_qp(problem, candidates) and use_quad
+    valid_conic = len(candidates['conic_solvers']) > 0
+    if not valid_qp and not valid_conic:
+        raise SolverError("Problem could not be reduced to a QP, and no "
+                            "conic solvers exist among candidate solvers "
+                            "(%s)." % candidates)
+
     return reductions
 
 
 def construct_solving_chain(problem, candidates,
                             gp: bool = False,
                             enforce_dpp: bool = False,
-                            ignore_dpp: bool = False) -> "SolvingChain":
+                            ignore_dpp: bool = False,
+                            canon_backend: str | None = None,
+                            solver_opts: dict | None = None,
+                            specified_solver: str | None = None,
+                            ) -> "SolvingChain":
     """Build a reduction chain from a problem to an installed solver.
 
     Note that if the supplied problem has 0 variables, then the solver
@@ -145,6 +209,15 @@ def construct_solving_chain(problem, candidates,
     ignore_dpp : bool, optional
         When True, DPP problems will be treated as non-DPP,
         which may speed up compilation. Defaults to False.
+    canon_backend : str, optional
+        'CPP' (default) | 'SCIPY'
+        Specifies which backend to use for canonicalization, which can affect
+        compilation time. Defaults to None, i.e., selecting the default
+        backend.
+    solver_opts : dict, optional
+        Additional arguments to pass to the solver.
+    specified_solver: str, optional
+        A solver specified by the user.
 
     Returns
     -------
@@ -157,28 +230,22 @@ def construct_solving_chain(problem, candidates,
         Raised if no suitable solver exists among the installed solvers, or
         if the target solver is not installed.
     """
+    canon_backend = _get_canon_backend(problem, canon_backend)
     if len(problem.variables()) == 0:
         return SolvingChain(reductions=[ConstantSolver()])
-    reductions = _reductions_for_problem_class(problem, candidates, gp)
+    reductions = _reductions_for_problem_class(problem, candidates, gp, solver_opts)
 
     # Process DPP status of the problem.
     dpp_context = 'dcp' if not gp else 'dgp'
-    dpp_error_msg = (
-            "You are solving a parameterized problem that is not DPP. "
-            "Because the problem is not DPP, subsequent solves will not be "
-            "faster than the first one. For more information, see the "
-            "documentation on Discplined Parametrized Programming, at\n"
-            "\thttps://www.cvxpy.org/tutorial/advanced/index.html#"
-            "disciplined-parametrized-programming")
     if ignore_dpp or not problem.is_dpp(dpp_context):
         # No warning for ignore_dpp.
         if ignore_dpp:
             reductions = [EvalParams()] + reductions
         elif not enforce_dpp:
-            warnings.warn(dpp_error_msg)
+            warnings.warn(DPP_ERROR_MSG)
             reductions = [EvalParams()] + reductions
         else:
-            raise DPPError(dpp_error_msg)
+            raise DPPError(DPP_ERROR_MSG)
     elif any(param.is_complex() for param in problem.parameters()):
         reductions = [EvalParams()] + reductions
     else:  # Compilation with DPP.
@@ -192,12 +259,17 @@ def construct_solving_chain(problem, candidates,
     # Conclude with matrix stuffing; choose one of the following paths:
     #   (1) QpMatrixStuffing --> [a QpSolver],
     #   (2) ConeMatrixStuffing --> [a ConicSolver]
-    if _solve_as_qp(problem, candidates):
+    use_quad = True if solver_opts is None else solver_opts.get('use_quad_obj', True)
+    if _solve_as_qp(problem, candidates) and use_quad:
         # Canonicalize as a QP
         solver = candidates['qp_solvers'][0]
         solver_instance = slv_def.SOLVER_MAP_QP[solver]
-        reductions += [QpMatrixStuffing(),
-                       solver_instance]
+        reductions += [
+            CvxAttr2Constr(reduce_bounds=not solver_instance.BOUNDED_VARIABLES),
+            qp2symbolic_qp.Qp2SymbolicQp(),
+            QpMatrixStuffing(canon_backend=canon_backend),
+            solver_instance,
+        ]
         return SolvingChain(reductions=reductions)
 
     # Canonicalize as a cone program
@@ -212,12 +284,18 @@ def construct_solving_chain(problem, candidates,
     for c in problem.constraints:
         constr_types.add(type(c))
     ex_cos = [ct for ct in constr_types if ct in EXOTIC_CONES]
+    approx_cos = [ct for ct in constr_types if ct in APPROX_CONES]
     # ^ The way we populate "ex_cos" will need to change if and when
     # we have atoms that require exotic cones.
     for co in ex_cos:
         sim_cos = EXOTIC_CONES[co]  # get the set of required simple cones
         constr_types.update(sim_cos)
         constr_types.remove(co)
+
+    for co in approx_cos:
+        app_cos = APPROX_CONES[co]
+        constr_types.update(app_cos)
+        constr_types.remove(co)
     # We now go over individual elementary cones support by CVXPY (
     # SOC, ExpCone, NonNeg, Zero, PSD, PowCone3D) and check if
     # they've appeared in constr_types or if the problem has an atom
@@ -245,16 +323,56 @@ def construct_solving_chain(problem, candidates,
 
     # Here, we make use of the observation that canonicalization only
     # increases the number of constraints in our problem.
-    has_constr = len(cones) > 0 or len(problem.constraints) > 0
+    var_domains = sum([var.domain for var in problem.variables()], start = [])
+    has_constr = len(cones) > 0 or len(problem.constraints) > 0 or len(var_domains) > 0
+
+    if PSD in cones \
+            and slv_def.DISREGARD_CLARABEL_SDP_SUPPORT_FOR_DEFAULT_RESOLUTION \
+            and specified_solver is None:
+        candidates['conic_solvers'] = [s for s in candidates['conic_solvers'] if s != CLARABEL]
 
     for solver in candidates['conic_solvers']:
         solver_instance = slv_def.SOLVER_MAP_CONIC[solver]
-        if (all(c in solver_instance.SUPPORTED_CONSTRAINTS for c in cones)
-                and (has_constr or not solver_instance.REQUIRES_CONSTR)):
+        # Cones supported for MI problems may differ from non MI.
+        if problem.is_mixed_integer():
+            supported_constraints = solver_instance.MI_SUPPORTED_CONSTRAINTS
+        else:
+            supported_constraints = solver_instance.SUPPORTED_CONSTRAINTS
+        unsupported_constraints = [
+            cone for cone in cones if cone not in supported_constraints
+        ]
+
+        if has_constr or not solver_instance.REQUIRES_CONSTR:
             if ex_cos:
                 reductions.append(Exotic2Common())
-            reductions += [ConeMatrixStuffing(), solver_instance]
-            return SolvingChain(reductions=reductions)
+            if RelEntrConeQuad in approx_cos or OpRelEntrConeQuad in approx_cos:
+                reductions.append(QuadApprox())
+
+            # Should the objective be canonicalized to a quadratic?
+            if solver_opts is None:
+                use_quad_obj = True
+            else:
+                use_quad_obj = solver_opts.get("use_quad_obj", True)
+            quad_obj = use_quad_obj and solver_instance.supports_quad_obj() and \
+                problem.objective.expr.has_quadratic_term()
+            reductions += [
+                Dcp2Cone(quad_obj=quad_obj),
+                CvxAttr2Constr(reduce_bounds=not solver_instance.BOUNDED_VARIABLES),
+            ]
+            if all(c in supported_constraints for c in cones):
+                # Return the reduction chain.
+                reductions += [
+                    ConeMatrixStuffing(quad_obj=quad_obj, canon_backend=canon_backend),
+                    solver_instance
+                ]
+                return SolvingChain(reductions=reductions)
+            elif all(c==SOC for c in unsupported_constraints) and PSD in supported_constraints:
+                reductions += [
+                    SOC2PSD(),
+                    ConeMatrixStuffing(quad_obj=quad_obj, canon_backend=canon_backend),
+                    solver_instance
+                ]
+                return SolvingChain(reductions=reductions)
 
     raise SolverError("Either candidate conic solvers (%s) do not support the "
                       "cones output by the problem (%s), or there are not "
@@ -262,6 +380,51 @@ def construct_solving_chain(problem, candidates,
                           candidates['conic_solvers'],
                           ", ".join([cone.__name__ for cone in cones])))
 
+def _get_canon_backend(problem, canon_backend):
+    """
+    This function checks if the problem has expressions of dimension greater
+    than 2 or if it lacks C++ support, then raises a warning if the default
+    backend is not specified or raises an error if the backend is specified
+    as 'CPP'.
+
+    Parameters
+    ----------
+    problem : Problem
+        The problem for which to build a chain.
+    canon_backend : str
+        'CPP' (default) | 'SCIPY'
+        Specifies which backend to use for canonicalization, which can affect
+        compilation time. Defaults to None, i.e., selecting the default
+        backend.
+    Returns
+    -------
+    canon_backend : str
+        The canonicalization backend to use.
+    """
+
+    if not problem._supports_cpp():
+        if canon_backend is None:
+            warnings.warn(UserWarning(
+                f"The problem includes expressions that don't support {CPP_CANON_BACKEND} backend. "
+                f"Defaulting to the {SCIPY_CANON_BACKEND} backend for canonicalization."))
+            return SCIPY_CANON_BACKEND
+        if canon_backend == CPP_CANON_BACKEND:
+            raise ValueError(f"The {CPP_CANON_BACKEND} backend cannot be used with problems "
+                             f"that have expressions which do not support it.")
+        return canon_backend  # Use the specified backend (e.g., SCIPY_CANON_BACKEND)
+
+    if problem._max_ndim() > 2:
+        if canon_backend is None:
+            warnings.warn(UserWarning(
+                f"The problem has an expression with dimension greater than 2. "
+                f"Defaulting to the {SCIPY_CANON_BACKEND} backend for canonicalization."))
+            return SCIPY_CANON_BACKEND
+        if canon_backend == CPP_CANON_BACKEND:
+            raise ValueError(f"Only the {SCIPY_CANON_BACKEND} and {NUMPY_CANON_BACKEND} "
+                             f"backends are supported for problems with expressions of "
+                             f"dimension greater than 2.")
+    return canon_backend
+
 
 class SolvingChain(Chain):
     """A reduction chain that ends with a solver.
