@@ -1,5 +1,3 @@
-from typing import Any, Dict
-
 """
 Copyright 2013 Steven Diamond
 
@@ -16,11 +14,35 @@ See the License for the specific language governing permissions and
 limitations under the License.
 """
 
+from typing import Any, Dict
+
 import numpy as np
 
 import cvxpy.interface as intf
 
 
+def stack_vals(variables: list, default: float, order: str = "F") -> np.ndarray:
+    """Stacks the values of the given variables.
+
+    Parameters
+    ----------
+    variables: list of cvxpy variables.
+    default: value to use when variable value is None.
+    order: unravel values in C or Fortran ("F") order
+
+    Returns
+    -------
+    An initial guess for the solution.
+    """
+    value = []
+    for variable in variables:
+        if variable.value is not None:
+            value.append(np.ravel(variable.value, order))
+        else:  # unknown values.
+            value.append(np.full(variable.size, default))
+    return np.concatenate(value)
+
+
 def expcone_permutor(n_cones, exp_cone_order) -> np.ndarray:
     order = np.tile(np.array(exp_cone_order), n_cones)  # e.g. [1,0,2, 1,0,2, 1,0,2,...
     offsets = 3 * np.repeat(np.arange(n_cones), 3)  # [0,0,0, 3,3,3, 6,6,6, ...
