@@ -74,18 +74,11 @@ class SolverTestHelper:
         #   (e.g. X = Variable(shape=(n,n), PSD=True)), check
         #   domains for dual variables of the attribute constraint.
         for con in self.constraints:
-            if isinstance(con, cp.constraints.PSD):
-                # TODO: move this to PSD.dual_violation
-                dv = con.dual_value
-                eigs = np.linalg.eigvalsh(dv)
-                min_eig = np.min(eigs)
-                self.tester.assertGreaterEqual(min_eig, -(10**(-places)))
-            elif isinstance(con, cp.constraints.ExpCone):
-                # TODO: implement this (preferably with ExpCone.dual_violation)
-                raise NotImplementedError()
-            elif isinstance(con, cp.constraints.SOC):
-                # TODO: implement this (preferably with SOC.dual_violation)
-                raise NotImplementedError()
+            if isinstance(con, cp.constraints.Cone):
+                dual_violation = con.dual_residual
+                if isinstance(con, cp.constraints.SOC):
+                    dual_violation = np.linalg.norm(dual_violation)
+                self.tester.assertLessEqual(dual_violation, 10**(-places))
             elif isinstance(con, cp.constraints.Inequality):
                 # TODO: move this to Inequality.dual_violation
                 dv = con.dual_value
@@ -99,8 +92,6 @@ class SolverTestHelper:
                     self.tester.assertEqual(contents, float)
                 else:
                     self.tester.assertIsInstance(dv, float)
-            elif isinstance(con, cp.constraints.PowCone3D):
-                raise NotImplementedError()
             else:
                 raise ValueError('Unknown constraint type %s.' % type(con))
 
@@ -110,27 +101,131 @@ class SolverTestHelper:
         #   complementarity against the dual variable of the
         #   attribute constraint.
         for con in self.constraints:
-            if isinstance(con, cp.constraints.PSD):
-                dv = con.dual_value
-                pv = con.args[0].value
-                comp = cp.scalar_product(pv, dv).value
+            if isinstance(con, (cp.constraints.Inequality,
+                                cp.constraints.Equality)):
+                comp = cp.vdot(con.expr, con.dual_value).value
             elif isinstance(con, (cp.constraints.ExpCone,
                                   cp.constraints.SOC,
-                                  cp.constraints.NonPos,
-                                  cp.constraints.Zero)):
-                comp = cp.scalar_product(con.args, con.dual_value).value
-            elif isinstance(con, cp.constraints.PowCone3D):
-                comp = cp.scalar_product(con.args[:3], con.dual_value).value
-            elif isinstance(con, (cp.constraints.Inequality,
-                                  cp.constraints.Equality)):
-                comp = cp.scalar_product(con.expr, con.dual_value).value
-            elif isinstance(con, cp.constraints.PowConeND):
-                msg = '\nPowConeND dual variables not implemented;' \
+                                  cp.constraints.NonNeg,
+                                  cp.constraints.Zero,
+                                  cp.constraints.PSD,
+                                  cp.constraints.PowCone3D,
+                                  cp.constraints.PowConeND)):
+                comp = cp.vdot(con.args, con.dual_value).value
+            elif isinstance(con, cp.RelEntrConeQuad) or isinstance(con, cp.OpRelEntrConeQuad):
+                msg = '\nDual variables not implemented for quadrature based approximations;' \
                        + '\nSkipping complementarity check.'
                 warnings.warn(msg)
             else:
                 raise ValueError('Unknown constraint type %s.' % type(con))
             self.tester.assertAlmostEqual(comp, 0, places)
+            
+    def check_stationary_lagrangian(self, places) -> None:
+        L = self.prob.objective.expr
+        objective = self.prob.objective
+        if objective.NAME == 'minimize':
+            L = objective.expr
+        else:
+            L = -objective.expr
+        for con in self.constraints:
+            if isinstance(con, (cp.constraints.Inequality,
+                                cp.constraints.Equality)):
+                dual_var_value = con.dual_value
+                prim_var_expr = con.expr
+                L = L + cp.vdot(dual_var_value, prim_var_expr)
+            elif isinstance(con, (cp.constraints.ExpCone,
+                                  cp.constraints.SOC,
+                                  cp.constraints.Zero,
+                                  cp.constraints.NonNeg,
+                                  cp.constraints.PSD,
+                                  cp.constraints.PowCone3D,
+                                  cp.constraints.PowConeND)):
+                L = L - cp.vdot(con.args, con.dual_value)
+            else:
+                raise NotImplementedError()
+        try:
+            g = L.grad
+        except TypeError as e:
+            assert 'is not subscriptable' in str(e)
+            msg = """\n
+            CVXPY problems with `diag` variables are not supported for
+            stationarity checks as of now
+            """
+            self.tester.fail(msg)
+        bad_norms = []
+
+        r"""The convention that we follow for construting the Lagrangian is: 1) Move all
+        explicitly passed constraints to the problem (via Problem.constraints) into the
+        Lagrangian --- dLdX == 0 for any such variables 2) Constraints that have
+        implicitly been imposed on variables at the time of declaration via specific
+        flags (e.g.: PSD/symmetric etc.), in such a case we check, `dLdX\in K^{*}`, where
+        `K` is the convex cone corresponding to the implicit constraint on `X`
+        """
+        for (opt_var, v) in g.items():
+            if all(not attr for attr in list(map(lambda x: x[1], opt_var.attributes.items()))):
+                """Case when the variable doesn't have any special attributes"""
+                norm = np.linalg.norm(v.data) / np.sqrt(opt_var.size)
+                if norm > 10**(-places):
+                    bad_norms.append((norm, opt_var))
+            else:
+                if opt_var.is_psd():
+                    """The PSD cone is self-dual"""
+                    g_bad_mat = cp.Constant(np.reshape(g[opt_var].toarray(), opt_var.shape))
+                    tmp_con = g_bad_mat >> 0
+                    dual_cone_violation = tmp_con.residual
+                    if dual_cone_violation > 10**(-places):
+                        bad_norms.append((dual_cone_violation, opt_var))
+                elif opt_var.is_nsd():
+                    """The NSD cone is also self-dual"""
+                    g_bad_mat = cp.Constant(np.reshape(g[opt_var].toarray(), opt_var.shape))
+                    tmp_con = g_bad_mat << 0
+                    dual_cone_violation = tmp_con.residual
+                    if dual_cone_violation > 10**(-places):
+                        bad_norms.append((dual_cone_violation, opt_var))
+                elif opt_var.is_diag():
+                    """The dual cone to the set of diagonal matrices is the set of
+                        'Hollow' matrices i.e. matrices with diagonal entries zero"""
+                    g_bad_mat = np.reshape(g[opt_var].toarray(), opt_var.shape)
+                    diag_entries = np.diag(opt_var.value)
+                    dual_cone_violation = np.linalg.norm(diag_entries) / np.sqrt(opt_var.size)
+                    if diag_entries > 10**(-places):
+                        bad_norms.append((dual_cone_violation, opt_var))
+                elif opt_var.is_symmetric():
+                    r"""The dual cone to the set of symmetric matrices is the
+                    set of skew-symmetric matrices, so we check if dLdX \in
+                    set(skew-symmetric-matrices)
+                    g[opt_var] is the problematic gradient in question"""
+                    g_bad_mat = np.reshape(g[opt_var].toarray(), opt_var.shape)
+                    mat = g_bad_mat + g_bad_mat.T
+                    dual_cone_violation = np.linalg.norm(mat) / np.sqrt(opt_var.size)
+                    if dual_cone_violation > 10**(-places):
+                        bad_norms.append((dual_cone_violation, opt_var))
+                elif opt_var.is_nonpos():
+                    """The cone of matrices with all entries nonpos is self-dual"""
+                    g_bad_mat = cp.Constant(np.reshape(g[opt_var].toarray(), opt_var.shape))
+                    tmp_con = g_bad_mat <= 0
+                    dual_cone_violation = np.linalg.norm(tmp_con.residual) / np.sqrt(opt_var.size)
+                    if dual_cone_violation > 10**(-places):
+                        bad_norms.append((dual_cone_violation, opt_var))
+                elif opt_var.is_nonneg():
+                    """The cone of matrices with all entries nonneg is self-dual"""
+                    g_bad_mat = cp.Constant(np.reshape(g[opt_var].toarray(), opt_var.shape))
+                    tmp_con = g_bad_mat >= 0
+                    dual_cone_violation = np.linalg.norm(tmp_con.residual) / np.sqrt(opt_var.size)
+                    if dual_cone_violation > 10**(-places):
+                        bad_norms.append((dual_cone_violation, opt_var))
+
+        if len(bad_norms):
+            msg = f"""\n
+        The gradient of Lagrangian with respect to the primal variables
+        is above the threshold of 10^{-places}. The names of the problematic
+        variables and the corresponding gradient norms are as follows:
+            """
+            for norm, opt_var in bad_norms:
+                msg += f"\n\t\t\t{opt_var.name} : {norm}"
+            msg += '\n'
+            self.tester.fail(msg)
+        pass 
 
     def verify_objective(self, places) -> None:
         actual = self.prob.value
@@ -256,6 +351,43 @@ def lp_6() -> SolverTestHelper:
     return sth
 
 
+def lp_7() -> SolverTestHelper:
+    """
+    An ill-posed problem to test multiprecision ability of solvers.
+
+    This test will not pass on CVXOPT (as of v1.3.1) and on SDPA without GMP support.
+    """
+    n = 50
+    a = cp.Variable((n+1))
+    delta = cp.Variable((n))
+    b = cp.Variable((n+1))
+    objective = cp.Minimize(cp.sum(cp.pos(delta)))
+    constraints = [
+        a[1:] - a[:-1] == delta,
+        a >= cp.pos(b),
+    ]
+    con_pairs = [(constraints[0], None),
+                 (constraints[1], None)]
+    var_pairs = [(a, None),
+                 (delta, None),
+                 (b, None)]
+    obj_pair = (objective, 0.)
+    sth = SolverTestHelper(obj_pair, var_pairs, con_pairs)
+    return sth
+
+
+def qp_0() -> SolverTestHelper:
+    # univariate feasible problem
+    x = cp.Variable(1)
+    objective = cp.Minimize(cp.square(x))
+    constraints = [x[0] >= 1]
+    con_pairs = [(constraints[0], 2)]
+    obj_pair = (objective, 1)
+    var_pairs = [(x, 1)]
+    sth = SolverTestHelper(obj_pair, var_pairs, con_pairs)
+    return sth
+
+
 def socp_0() -> SolverTestHelper:
     x = cp.Variable(shape=(2,))
     obj_pair = (cp.Minimize(cp.norm(x, 2) + 1), 1)
@@ -301,7 +433,7 @@ def socp_2() -> SolverTestHelper:
     """
     x = cp.Variable(shape=(2,), name='x')
     objective = cp.Minimize(-4 * x[0] - 5 * x[1])
-    expr = cp.reshape(x[0] + 2 * x[1], (1, 1))
+    expr = cp.reshape(x[0] + 2 * x[1], (1, 1), order='F')
     constraints = [2 * x[0] + x[1] <= 3,
                    cp.constraints.SOC(cp.Constant([3]), expr),
                    x[0] >= 0,
@@ -474,6 +606,50 @@ def expcone_socp_1() -> SolverTestHelper:
     return sth
 
 
+def sdp_pcp_1() -> SolverTestHelper:
+    """
+    Example sdp and power cone.
+    """
+    Sigma = np.array([[ 0.4787481 , -0.96924914],
+                      [-0.96924914,  2.77788598]])
+
+    x = cp.Variable(shape=(2,1))
+    y = cp.Variable(shape=(2,1))
+    X = cp.Variable(shape=(2,2), symmetric=True)
+    M1 = cp.vstack([X, x.T])
+    M2 = cp.vstack([x, np.ones((1, 1))])
+    M3 = cp.hstack([M1, M2])
+
+    var_pairs = [
+        (x, np.array([[0.72128204],
+                      [0.27871796]])),
+        (y, np.array([[0.01],
+                      [0.01]])),
+        (X, np.array([[0.52024779, 0.20103426],
+                      [0.20103426, 0.0776837 ]])),        
+    ]
+    con_pairs = [
+        (cp.sum(x) == 1, -0.1503204799112807),
+        (x >= 0, np.array([[-0.],
+                           [-0.]])),
+        (y >= 0.01, np.array([[0.70705506],
+                               [0.70715844]])),
+        (M3 >> 0, np.array([[ 0.4787481 , -0.96924914, -0.07516024],
+                            [-0.96924914,  2.77788598, -0.07516024],
+                            [-0.07516024, -0.07516024,  0.07516094]])),
+        (cp.PowCone3D(x, np.ones((2,1)), y, 0.9), [np.array([[1.17878172e-09],
+                                                             [3.05162243e-09]]),
+                                                   np.array([[9.20157640e-10],
+                                                             [9.40823207e-10]]),
+                                                   np.array([[2.41053358e-10],
+                                                             [7.43432462e-10]])]),
+        ]
+    obj_expr = cp.Minimize(cp.trace(Sigma @ X) + cp.norm(y, p=2))
+    obj_pair = (obj_expr, 0.089301671322676)
+    sth = SolverTestHelper(obj_pair, var_pairs, con_pairs)
+    return sth
+
+
 def pcp_1() -> SolverTestHelper:
     """
     Use a 3D power cone formulation for
@@ -715,6 +891,100 @@ def mi_lp_4() -> SolverTestHelper:
     return sth
 
 
+def mi_lp_5() -> SolverTestHelper:
+    # infeasible boolean problem - https://trac.sagemath.org/ticket/31962#comment:48
+    z = cp.Variable(11, boolean=True)
+    constraints = [z[2] + z[1] == 1,
+                   z[4] + z[3] == 1,
+                   z[6] + z[5] == 1,
+                   z[8] + z[7] == 1,
+                   z[10] + z[9] == 1,
+                   z[4] + z[1] <= 1,
+                   z[2] + z[3] <= 1,
+                   z[6] + z[2] <= 1,
+                   z[1] + z[5] <= 1,
+                   z[8] + z[6] <= 1,
+                   z[5] + z[7] <= 1,
+                   z[10] + z[8] <= 1,
+                   z[7] + z[9] <= 1,
+                   z[9] + z[4] <= 1,
+                   z[3] + z[10] <= 1]
+    obj = cp.Minimize(0)
+    obj_pair = (obj, np.inf)
+    con_pairs = [(c, None) for c in constraints]
+    var_pairs = [(z, None)]
+    sth = SolverTestHelper(obj_pair, var_pairs, con_pairs)
+    return sth
+
+def mi_lp_6() -> SolverTestHelper:
+    "Test MILP for timelimit and no feasible solution"
+    n = 70
+    m = 70
+    x = cp.Variable((n,), boolean=True, name="x")
+    y = cp.Variable((n,), name="y")
+    z = cp.Variable((m,), pos=True, name="z")
+    A = np.random.rand(m, n)
+    b = np.random.rand(m)
+    objective = cp.Maximize(cp.sum(y))
+    constraints = [
+        A @ y <= b,
+        y <= 1,
+        cp.sum(x) >= 10,
+        cp.sum(x) <= 20,
+        z[0] + z[1] + z[2] >= 10,
+        z[3] + z[4] + z[5] >= 5,
+        z[6] + z[7] + z[8] >= 7,
+        z[9] + z[10] >= 8,
+        z[11] + z[12] >= 6,
+        z[13] + z[14] >= 3,
+        z[15] + z[16] >= 2,
+        z[17] + z[18] >= 1,
+        z[19] >= 2,
+        z[20] >= 1,
+        z[21] >= 1,
+        z[22] >= 1,
+        z[23] >= 1,
+        z[24] >= 1,
+        z[25] >= 1,
+        z[26] >= 1,
+        z[27] >= 1,
+        z[28] >= 1,
+        z[29] >= 1,
+    ]
+    return SolverTestHelper(
+        (objective, None),
+        [(x, None), (y, None), (z, None)],
+        [(con, None) for con in constraints]
+    )
+
+
+
+def mi_lp_7() -> SolverTestHelper:
+    """Problem that takes significant time to solve - for testing time/iteration limits"""
+    np.random.seed(0)
+    n = 24 * 8
+    c = cp.Variable((n,), pos=True)
+    d = cp.Variable((n,), pos=True)
+    c_or_d = cp.Variable((n,), boolean=True)
+    big = 1e3
+    s = cp.cumsum(c * 0.9 - d)
+    p = np.random.random(n)
+    objective = cp.Maximize(p @ (d - c))
+    constraints = [
+        d <= 1,
+        c <= 1,
+        s >= 0,
+        s <= 1,
+        c <= c_or_d * big,
+        d <= (1 - c_or_d) * big,
+    ]
+    return SolverTestHelper(
+        (objective, None),
+        [(c, None,), (d, None), (c_or_d, None)],
+        [(con, None) for con in constraints]
+    )
+
+
 def mi_socp_1() -> SolverTestHelper:
     """
     Formulate the following mixed-integer SOCP with cvxpy
@@ -862,6 +1132,15 @@ class StandardTestLPs:
             sth.check_dual_domains(places)
         return sth
 
+    @staticmethod
+    def test_lp_7(solver, places: int = 4, duals: bool = True, **kwargs) -> SolverTestHelper:
+        sth = lp_7()
+        import sdpap
+        if sdpap.sdpacall.sdpacall.get_backend_info()["gmp"]:
+            sth.solve(solver, **kwargs)
+            sth.verify_objective(places)
+        return sth
+
     @staticmethod
     def test_mi_lp_0(solver, places: int = 4, **kwargs) -> SolverTestHelper:
         sth = mi_lp_0()
@@ -902,6 +1181,28 @@ class StandardTestLPs:
         sth.verify_primal_values(places)
         return sth
 
+    @staticmethod
+    def test_mi_lp_5(solver, places: int = 4, **kwargs) -> SolverTestHelper:
+        sth = mi_lp_5()
+        sth.solve(solver, **kwargs)
+        sth.verify_objective(places)
+        sth.verify_primal_values(places)
+        return sth
+
+
+class StandardTestQPs:
+
+    @staticmethod
+    def test_qp_0(solver, places: int = 4, duals: bool = True, **kwargs) -> SolverTestHelper:
+        sth = qp_0()
+        sth.solve(solver, **kwargs)
+        sth.verify_primal_values(places)
+        sth.verify_objective(places)
+        if duals:
+            sth.check_complementarity(places)
+            sth.verify_dual_values(places)
+        return sth
+
 
 class StandardTestSOCPs:
 
@@ -1001,7 +1302,7 @@ class StandardTestSDPs:
         return sth
 
     @staticmethod
-    def test_sdp_2(solver, places: int = 3, duals: bool = True, **kwargs) -> SolverTestHelper:
+    def test_sdp_2(solver, places: int = 2, duals: bool = True, **kwargs) -> SolverTestHelper:
         # places is set to 3 rather than 4, because analytic solution isn't known.
         sth = sdp_2()
         sth.solve(solver, **kwargs)
@@ -1041,7 +1342,19 @@ class StandardTestMixedCPs:
             sth.verify_dual_values(places)
         return sth
 
+    @staticmethod
+    def test_sdp_pcp_1(solver, places: int = 3, duals: bool = False, **kwargs) -> SolverTestHelper:
+        sth = sdp_pcp_1()
+        sth.solve(solver, **kwargs)
+        sth.verify_objective(places)
+        sth.check_primal_feasibility(places)
+        sth.verify_primal_values(places)
+        if duals:
+            sth.check_complementarity(places)
+            sth.check_dual_domains(places)
+        return sth
 
+    
 class StandardTestPCPs:
 
     @staticmethod
@@ -1049,6 +1362,7 @@ class StandardTestPCPs:
         sth = pcp_1()
         sth.solve(solver, **kwargs)
         sth.verify_objective(places)
+        sth.check_primal_feasibility(places)
         sth.verify_primal_values(places)
         if duals:
             sth.check_complementarity(places)
@@ -1060,6 +1374,7 @@ class StandardTestPCPs:
         sth = pcp_2()
         sth.solve(solver, **kwargs)
         sth.verify_objective(places)
+        sth.check_primal_feasibility(places)
         sth.verify_primal_values(places)
         if duals:
             sth.check_complementarity(places)
@@ -1071,6 +1386,7 @@ class StandardTestPCPs:
         sth = pcp_3()
         sth.solve(solver, **kwargs)
         sth.verify_objective(places)
+        sth.check_primal_feasibility(places)
         sth.verify_primal_values(places)
         if duals:
             sth.check_complementarity(places)
@@ -1081,5 +1397,6 @@ class StandardTestPCPs:
         sth = mi_pcp_0()
         sth.solve(solver, **kwargs)
         sth.verify_objective(places)
+        sth.check_primal_feasibility(places)
         sth.verify_primal_values(places)
         return sth
