@@ -21,6 +21,7 @@ import scipy as sp
 
 import cvxpy as cp
 from cvxpy import settings as s
+from cvxpy.atoms.affine.trace import trace
 from cvxpy.constraints.exponential import ExpCone
 from cvxpy.constraints.power import PowCone3D, PowConeND
 from cvxpy.constraints.second_order import SOC
@@ -29,9 +30,9 @@ from cvxpy.reductions.cone2cone import affine2direct as a2d
 from cvxpy.reductions.cvx_attr2constr import CvxAttr2Constr
 from cvxpy.reductions.dcp2cone.cone_matrix_stuffing import ConeMatrixStuffing
 from cvxpy.reductions.dcp2cone.dcp2cone import Dcp2Cone
+from cvxpy.reductions.solution import Solution
 from cvxpy.reductions.solvers.conic_solvers.conic_solver import ConicSolver
-from cvxpy.reductions.solvers.defines import (
-    INSTALLED_MI_SOLVERS as INSTALLED_MI,)
+from cvxpy.reductions.solvers.defines import INSTALLED_MI_SOLVERS as INSTALLED_MI
 from cvxpy.reductions.solvers.defines import MI_SOCP_SOLVERS as MI_SOCP
 from cvxpy.tests import solver_test_helpers as STH
 from cvxpy.tests.base_test import BaseTest
@@ -48,7 +49,7 @@ class TestDualize(BaseTest):
 
         # Dualize the problem, reconstruct a high-level cvxpy problem for the dual.
         # Solve the problem, invert the dualize reduction.
-        cone_prog = ConicSolver().format_constraints(cone_prog, exp_cone_order=[0, 1, 2])
+        cone_prog = ConicSolver.format_constraints(cone_prog, exp_cone_order=[0, 1, 2])
         data, inv_data = a2d.Dualize.apply(cone_prog)
         A, b, c, K_dir = data[s.A], data[s.B], data[s.C], data['K_dir']
         y = cp.Variable(shape=(A.shape[1],))
@@ -88,7 +89,7 @@ class TestDualize(BaseTest):
         if K_dir[a2d.DUAL_POW3D]:
             dual_prims[a2d.DUAL_POW3D] = dual_prims[a2d.DUAL_POW3D].value
         dual_duals = {s.EQ_DUAL: constraints[0].dual_value}
-        dual_sol = cp.Solution(dual_prob.status, dual_prob.value, dual_prims, dual_duals, dict())
+        dual_sol = Solution(dual_prob.status, dual_prob.value, dual_prims, dual_duals, dict())
         cone_sol = a2d.Dualize.invert(dual_sol, inv_data)
 
         # Pass the solution back up the solving chain.
@@ -204,7 +205,7 @@ class TestSlacks(BaseTest):
 
         # apply the Slacks reduction, reconstruct a high-level problem,
         # solve the problem, invert the reduction.
-        cone_prog = ConicSolver().format_constraints(cone_prog, exp_cone_order=[0, 1, 2])
+        cone_prog = ConicSolver.format_constraints(cone_prog, exp_cone_order=[0, 1, 2])
         data, inv_data = a2d.Slacks.apply(cone_prog, affine)
         G, h, f, K_dir, K_aff = data[s.A], data[s.B], data[s.C], data['K_dir'], data['K_aff']
         G = sp.sparse.csc_matrix(G)
@@ -217,7 +218,7 @@ class TestSlacks(BaseTest):
         slack_prob = cp.Problem(objective, constraints)
         slack_prob.solve(**solve_kwargs)
         slack_prims = {a2d.FREE: y[:cone_prog.x.size].value}  # nothing else need be populated.
-        slack_sol = cp.Solution(slack_prob.status, slack_prob.value, slack_prims, None, dict())
+        slack_sol = Solution(slack_prob.status, slack_prob.value, slack_prims, None, dict())
         cone_sol = a2d.Slacks.invert(slack_sol, inv_data)
 
         # pass solution up the solving chain
@@ -290,7 +291,7 @@ class TestSlacks(BaseTest):
         # typical LP
         sth = STH.lp_2()
         for affine in TestSlacks.AFF_LP_CASES:
-            TestSlacks.simulate_chain(sth.prob, affine, solver='ECOS')
+            TestSlacks.simulate_chain(sth.prob, affine, solver='CLARABEL')
             sth.verify_objective(places=4)
             sth.verify_primal_values(places=4)
 
@@ -298,41 +299,41 @@ class TestSlacks(BaseTest):
         # unbounded LP
         sth = STH.lp_3()
         for affine in TestSlacks.AFF_LP_CASES:
-            TestSlacks.simulate_chain(sth.prob, affine, solver='ECOS')
+            TestSlacks.simulate_chain(sth.prob, affine, solver='CLARABEL')
             sth.verify_objective(places=4)
 
     def test_lp_4(self):
         # infeasible LP
         sth = STH.lp_4()
         for affine in TestSlacks.AFF_LP_CASES:
-            TestSlacks.simulate_chain(sth.prob, affine, solver='ECOS')
+            TestSlacks.simulate_chain(sth.prob, affine, solver='CLARABEL')
             sth.verify_objective(places=4)
 
     def test_socp_2(self):
         sth = STH.socp_2()
         for affine in TestSlacks.AFF_SOCP_CASES:
-            TestSlacks.simulate_chain(sth.prob, affine, solver='ECOS')
+            TestSlacks.simulate_chain(sth.prob, affine, solver='CLARABEL')
             sth.verify_objective(places=4)
             sth.verify_primal_values(places=4)
 
     def test_socp_3(self):
         for axis in [0, 1]:
             sth = STH.socp_3(axis)
-            TestSlacks.simulate_chain(sth.prob, [], solver='ECOS')
+            TestSlacks.simulate_chain(sth.prob, [], solver='CLARABEL')
             sth.verify_objective(places=4)
             sth.verify_primal_values(places=4)
 
     def test_expcone_1(self):
         sth = STH.expcone_1()
         for affine in TestSlacks.AFF_EXP_CASES:
-            TestSlacks.simulate_chain(sth.prob, affine, solver='ECOS')
+            TestSlacks.simulate_chain(sth.prob, affine, solver='CLARABEL')
             sth.verify_objective(places=4)
             sth.verify_primal_values(places=4)
 
     def test_expcone_socp_1(self):
         sth = STH.expcone_socp_1()
         for affine in TestSlacks.AFF_MIXED_CASES:
-            TestSlacks.simulate_chain(sth.prob, affine, solver='ECOS')
+            TestSlacks.simulate_chain(sth.prob, affine, solver='SCS')
             sth.verify_objective(places=4)
             sth.verify_primal_values(places=4)
 
@@ -350,10 +351,14 @@ class TestSlacks(BaseTest):
             sth.verify_objective(places=3)
             sth.verify_primal_values(places=3)
 
+    @pytest.mark.skipif(
+        "HIGHS" not in INSTALLED_MI, 
+        reason='HiGHS solver is not installed.'
+    )
     def test_mi_lp_1(self):
         sth = STH.mi_lp_1()
         for affine in TestSlacks.AFF_LP_CASES:
-            TestSlacks.simulate_chain(sth.prob, affine, solver='ECOS_BB')
+            TestSlacks.simulate_chain(sth.prob, affine, solver=cp.HIGHS)
             sth.verify_objective(places=4)
             sth.verify_primal_values(places=4)
 
@@ -361,11 +366,11 @@ class TestSlacks(BaseTest):
     def test_mi_socp_1(self):
         sth = STH.mi_socp_1()
         for affine in TestSlacks.AFF_SOCP_CASES:
-            TestSlacks.simulate_chain(sth.prob, affine, solver='ECOS_BB')
+            TestSlacks.simulate_chain(sth.prob, affine, solver=cp.SCIPY)
             sth.verify_objective(places=4)
             sth.verify_primal_values(places=4)
 
-    @unittest.skipUnless([svr for svr in INSTALLED_MI if svr in MI_SOCP and svr != 'ECOS_BB'],
+    @unittest.skipUnless([svr for svr in INSTALLED_MI if svr in MI_SOCP],
                          'No appropriate mixed-integer SOCP solver is installed.')
     def test_mi_socp_2(self):
         sth = STH.mi_socp_2()
@@ -483,3 +488,289 @@ class TestPowND(BaseTest):
         sth.verify_primal_values(places=3)
         sth.check_complementarity(places=3)
         pass
+
+
+class TestRelEntrQuad(BaseTest):
+
+    def expcone_1(self) -> STH.SolverTestHelper:
+        """
+        min   3 * x[0] + 2 * x[1] + x[2]
+        s.t.  0.1 <= x[0] + x[1] + x[2] <= 1
+              x >= 0
+              and ...
+                x[0] >= x[1] * exp(x[2] / x[1])
+              equivalently ...
+                x[0] / x[1] >= exp(x[2] / x[1])
+                log(x[0] / x[1]) >= x[2] / x[1]
+                x[1] log(x[1] / x[0]) <= -x[2]
+        """
+        x = cp.Variable(shape=(3, 1))
+        cone_con = ExpCone(x[2], x[1], x[0]).as_quad_approx(5, 5)
+        constraints = [cp.sum(x) <= 1.0,
+                       cp.sum(x) >= 0.1,
+                       x >= 0,
+                       cone_con]
+        obj = cp.Minimize(3 * x[0] + 2 * x[1] + x[2])
+        obj_pair = (obj, 0.23534820622420757)
+        expect_exp = [np.array([-1.35348213]), np.array([-0.35348211]), np.array([0.64651792])]
+        con_pairs = [(constraints[0], 0),
+                     (constraints[1], 2.3534821130067614),
+                     (constraints[2], np.zeros(shape=(3, 1))),
+                     (constraints[3], expect_exp)]
+        expect_x = np.array([[0.05462721], [0.02609378], [0.01927901]])
+        var_pairs = [(x, expect_x)]
+        sth = STH.SolverTestHelper(obj_pair, var_pairs, con_pairs)
+        return sth
+
+    def test_expcone_1(self):
+        sth = self.expcone_1()
+        sth.solve(solver='CLARABEL')
+        sth.verify_primal_values(places=2)
+        sth.verify_objective(places=2)
+
+    def expcone_socp_1(self) -> STH.SolverTestHelper:
+        """
+        A random risk-parity portfolio optimization problem.
+        """
+        sigma = np.array([[1.83, 1.79, 3.22],
+                          [1.79, 2.18, 3.18],
+                          [3.22, 3.18, 8.69]])
+        L = np.linalg.cholesky(sigma)
+        c = 0.75
+        t = cp.Variable(name='t')
+        x = cp.Variable(shape=(3,), name='x')
+        s = cp.Variable(shape=(3,), name='s')
+        e = cp.Constant(np.ones(3, ))
+        objective = cp.Minimize(t - c * e @ s)
+        con1 = cp.norm(L.T @ x, p=2) <= t
+        con2 = ExpCone(s, e, x).as_quad_approx(5, 5)
+        # SolverTestHelper data
+        obj_pair = (objective, 4.0751197)
+        var_pairs = [
+            (x, np.array([0.57608346, 0.54315695, 0.28037716])),
+            (s, np.array([-0.55150, -0.61036, -1.27161])),
+        ]
+        con_pairs = [
+            (con1, 1.0),
+            (con2, [None,
+                    None,
+                    None]
+             )
+        ]
+        sth = STH.SolverTestHelper(obj_pair, var_pairs, con_pairs)
+        return sth
+
+    def test_expcone_socp_1(self):
+        sth = self.expcone_socp_1()
+        sth.solve(solver=cp.SCS)
+        sth.verify_primal_values(places=3)
+        sth.verify_objective(places=3)
+
+
+def sdp_ipm_installed():
+    viable = {cp.CVXOPT, cp.MOSEK, cp.COPT}.intersection(cp.installed_solvers())
+    return len(viable) > 0
+
+
+@unittest.skipUnless(sdp_ipm_installed(),
+                     'First-order solvers are too slow for the accuracy we need.')
+class TestOpRelConeQuad(BaseTest):
+
+    def setUp(self, n=3) -> None:
+        self.n = n
+        self.a = cp.Variable(shape=(n,), pos=True)
+        self.b = cp.Variable(shape=(n,), pos=True)
+        if hasattr(np.random, 'default_rng'):
+            self.rng = np.random.default_rng(0)
+        else:
+            self.rng = np.random.RandomState(0)
+        if hasattr(self.rng, 'random'):
+            rand_gen_func = self.rng.random
+        else:
+            rand_gen_func = self.rng.random_sample
+
+        self.a_lower = np.cumsum(rand_gen_func(n))
+        self.a_upper = self.a_lower + 0.05*rand_gen_func(n)
+        self.b_lower = np.cumsum(rand_gen_func(n))
+        self.b_upper = self.b_lower + 0.05*rand_gen_func(n)
+        self.base_cons = [
+            self.a_lower <= self.a,
+            self.a <= self.a_upper,
+            self.b_lower <= self.b,
+            self.b <= self.b_upper
+        ]
+        installed_solvers = cp.installed_solvers()
+        if cp.MOSEK in installed_solvers:
+            self.solver = cp.MOSEK
+        elif cp.CVXOPT in installed_solvers:
+            self.solver = cp.CVXOPT
+        elif cp.COPT in installed_solvers:
+            self.solver = cp.COPT
+        else:
+            raise RuntimeError('No viable solver installed.')
+        pass
+
+    @staticmethod
+    def Dop_commute(a: np.ndarray, b: np.ndarray, U: np.ndarray):
+        D = np.diag(a * np.log(a/b))
+        if np.iscomplexobj(U):
+            out = U @ D @ U.conj().T
+        else:
+            out = U @ D @ U.T
+        return out
+
+    @staticmethod
+    def sum_rel_entr_approx(a: cp.Expression, b: cp.Expression,
+                            apx_m: int, apx_k: int):
+        n = a.size
+        assert n == b.size
+        epi_vec = cp.Variable(shape=n)
+        con = cp.constraints.RelEntrConeQuad(a, b, epi_vec, apx_m, apx_k)
+        objective = cp.Minimize(cp.sum(epi_vec))
+        return objective, con
+
+    def oprelcone_1(self, apx_m, apx_k, real) -> STH.SolverTestHelper:
+        """
+        These tests construct two matrices that commute (imposing all eigenvectors equal)
+        and then use the fact that: T=Dop(A, B) for (A, B, T) in OpRelEntrConeQuad
+        i.e. T >> Dop(A, B) for an objective that is an increasing function of the
+        eigenvalues (which we here take to be the trace), we compute the reference
+        objective value as tr(Dop) whose correctness can be seen by writing out
+        tr(T)=tr(T-Dop)+tr(Dop), where tr(T-Dop)>=0 because of PSD-ness of (T-Dop),
+        and at optimality we have (T-Dop)=0 (the zero matrix of corresponding size)
+        For the case that the input matrices commute, Dop takes on a particularly
+        simplified form, i.e.: U @ diag(a * log(a/b)) @ U^{-1} (which is implemented
+        in the Dop_commute method above)
+        """
+        # Compute the expected optimal solution
+        temp_obj, temp_con = TestOpRelConeQuad.sum_rel_entr_approx(
+            self.a, self.b, apx_m, apx_k
+        )
+        temp_constraints = [con for con in self.base_cons]
+        temp_constraints.append(temp_con)
+        temp_prob = cp.Problem(temp_obj, temp_constraints)
+        temp_prob.solve()
+        expect_a = self.a.value
+        expect_b = self.b.value
+        expect_objective = temp_obj.value
+
+        # Next: create a matrix representation of the same problem,
+        # using operator relative entropy.
+        n = self.n
+        if real:
+            randmat = self.rng.normal(size=(n, n))
+            U = sp.linalg.qr(randmat)[0]
+            A = cp.symmetric_wrap(U @ cp.diag(self.a) @ U.T)
+            B = cp.symmetric_wrap(U @ cp.diag(self.b) @ U.T)
+            T = cp.Variable(shape=(n, n), symmetric=True)
+        else:
+            randmat = 1j * self.rng.normal(size=(n, n))
+            randmat += self.rng.normal(size=(n, n))
+            U = sp.linalg.qr(randmat)[0]
+            A = cp.hermitian_wrap(U @ cp.diag(self.a) @ U.conj().T)
+            B = cp.hermitian_wrap(U @ cp.diag(self.b) @ U.conj().T)
+            T = cp.Variable(shape=(n, n), hermitian=True)
+        main_con = cp.constraints.OpRelEntrConeQuad(A, B, T, apx_m, apx_k)
+        obj = cp.Minimize(trace(T))
+        expect_T = TestOpRelConeQuad.Dop_commute(expect_a, expect_b, U)
+
+        # Define the SolverTestHelper object
+        con_pairs = [(con, None) for con in self.base_cons]
+        con_pairs.append((main_con, None))
+        obj_pair = (obj, expect_objective)
+        var_pairs = [(T, expect_T)]
+        sth = STH.SolverTestHelper(obj_pair, var_pairs, con_pairs)
+        return sth
+
+    def test_oprelcone_1_m1_k3_real(self):
+        sth = self.oprelcone_1(1, 3, True)
+        sth.solve(self.solver)
+        sth.verify_primal_values(places=3)
+        sth.verify_objective(places=3)
+        pass
+
+    def test_oprelcone_1_m3_k1_real(self):
+        sth = self.oprelcone_1(3, 1, True)
+        sth.solve(self.solver)
+        sth.verify_primal_values(places=3)
+        sth.verify_objective(places=3)
+        pass
+
+    def test_oprelcone_1_m4_k4_real(self):
+        sth = self.oprelcone_1(4, 4, True)
+        sth.solve(self.solver)
+        sth.verify_primal_values(places=3)
+        sth.verify_objective(places=3)
+        pass
+
+    def test_oprelcone_1_m1_k3_complex(self):
+        sth = self.oprelcone_1(1, 3, False)
+        sth.solve(self.solver)
+        sth.verify_primal_values(places=3)
+        sth.verify_objective(places=3)
+        pass
+
+    def test_oprelcone_1_m3_k1_complex(self):
+        sth = self.oprelcone_1(3, 1, False)
+        sth.solve(self.solver)
+        sth.verify_primal_values(places=3)
+        sth.verify_objective(places=3)
+        pass
+
+    def oprelcone_2(self) -> STH.SolverTestHelper:
+        """
+        This test uses the same idea from the tests with commutative matrices,
+        instead, here, we make the input matrices to Dop, non-commutative,
+        the same condition as before i.e. T=Dop(A, B) for (A, B, T) in OpRelEntrConeQuad
+        (for an objective that is an increasing function of the eigenvalues) holds,
+        the difference here then, is in how we compute the reference values, which
+        has been done by assuming correctness of the original CVXQUAD matlab implementation
+        """
+        n, m, k = 4, 3, 3
+        # generate two sets of linearly orthogonal vectors
+        # Each to be set as the eigenvectors of a particular input matrix to Dop
+        U1 = np.array([[-0.05878522, -0.78378355, -0.49418311, -0.37149791],
+                       [0.67696027, -0.25733435, 0.59263364, -0.35254672],
+                       [0.43478177, 0.53648704, -0.54593428, -0.47444939],
+                       [0.59096015, -0.17788771, -0.32638042, 0.71595942]])
+        U2 = np.array([[-0.42499169, 0.6887562, 0.55846178, 0.18198188],
+                       [-0.55478633, -0.7091174, 0.3884544, 0.19613213],
+                       [-0.55591804, 0.14358541, -0.72444644, 0.38146522],
+                       [0.4500548, -0.04637494, 0.11135968, 0.88481584]])
+        a_diag = cp.Variable(shape=(n,), pos=True)
+        b_diag = cp.Variable(shape=(n,), pos=True)
+        A = U1 @ cp.diag(a_diag) @ U1.T
+        B = U2 @ cp.diag(b_diag) @ U2.T
+        T = cp.Variable(shape=(n, n), symmetric=True)
+        a_lower = np.array([0.40683013, 1.34514597, 1.60057343, 2.13373667])
+        a_upper = np.array([1.36158501, 1.61289351, 1.85065805, 3.06140939])
+        b_lower = np.array([0.06858235, 0.36798274, 0.95956627, 1.16286541])
+        b_upper = np.array([0.70446555, 1.16635299, 1.46126732, 1.81367755])
+        con1 = cp.constraints.OpRelEntrConeQuad(A, B, T, m, k)
+        con2 = a_lower <= a_diag
+        con3 = a_diag <= a_upper
+        con4 = b_lower <= b_diag
+        con5 = b_diag <= b_upper
+        con_pairs = [(con1, None),
+                     (con2, None),
+                     (con3, None),
+                     (con4, None),
+                     (con5, None)]
+        obj = cp.Minimize(trace(T))
+
+        expect_obj = 1.85476
+        expect_T = np.array([[0.49316819, 0.20845265, 0.60474713, -0.5820242],
+                             [0.20845265, 0.31084053, 0.2264112, -0.8442255],
+                             [0.60474713, 0.2264112, 0.4687153, -0.85667283],
+                             [-0.5820242, -0.8442255, -0.85667283, 0.58206723]])
+
+        obj_pair = (obj, expect_obj)
+        var_pairs = [(T, expect_T)]
+        sth = STH.SolverTestHelper(obj_pair, var_pairs, con_pairs)
+        return sth
+
+    def test_oprelcone_2(self):
+        sth = self.oprelcone_2()
+        sth.solve(self.solver)
+        sth.verify_primal_values(places=2)
+        sth.verify_objective(places=2)
