@@ -14,22 +14,22 @@ See the License for the specific language governing permissions and
 limitations under the License.
 """
 
+import unittest
+
+import numpy as np
 
 import cvxpy as cvx
 import cvxpy.interface as intf
 from cvxpy.tests.base_test import BaseTest
-from cvxpy.reductions.solvers.conic_solvers import ecos_conif
-import numpy as np
-import unittest
 
 
 class TestExamples(BaseTest):
     """ Unit tests using example problems. """
 
     # Find the largest Euclidean ball in the polyhedron.
-    def test_chebyshev_center(self):
+    def test_chebyshev_center(self) -> None:
         # The goal is to find the largest Euclidean ball (i.e. its center and
-        # radius) that lies in a polyhedron described by linear inequalites in this
+        # radius) that lies in a polyhedron described by linear inequalities in this
         # fashion: P = {x : a_i'*x <= b_i, i=1,...,m} where x is in R^2
 
         # Generate the input data
@@ -44,20 +44,20 @@ class TestExamples(BaseTest):
         x_c = cvx.Variable(2, name='x_c')
         obj = cvx.Maximize(r)
         constraints = [  # TODO have atoms compute values for constants.
-            a1.T*x_c + np.linalg.norm(a1)*r <= b[0],
-            a2.T*x_c + np.linalg.norm(a2)*r <= b[1],
-            a3.T*x_c + np.linalg.norm(a3)*r <= b[2],
-            a4.T*x_c + np.linalg.norm(a4)*r <= b[3],
+            a1.T @ x_c + np.linalg.norm(a1)*r <= b[0],
+            a2.T @ x_c + np.linalg.norm(a2)*r <= b[1],
+            a3.T @ x_c + np.linalg.norm(a3)*r <= b[2],
+            a4.T @ x_c + np.linalg.norm(a4)*r <= b[3],
         ]
 
         p = cvx.Problem(obj, constraints)
-        result = p.solve()
+        result = p.solve(solver=cvx.SCS)
         self.assertAlmostEqual(result, 0.447214)
         self.assertAlmostEqual(r.value, result)
         self.assertItemsAlmostEqual(x_c.value, [0, 0])
 
     # Test issue with numpy scalars.
-    def test_numpy_scalars(self):
+    def test_numpy_scalars(self) -> None:
         n = 6
         eps = 1e-6
         np.random.seed(10)
@@ -87,16 +87,16 @@ class TestExamples(BaseTest):
         slack = cvx.Variable()
         # Form the problem
         x = cvx.Variable(n)
-        objective = cvx.Minimize(0.5*cvx.quad_form(x, P0) + q0.T*x + r0 + slack)
-        constraints = [0.5*cvx.quad_form(x, P1) + q1.T*x + r1 <= slack,
-                       0.5*cvx.quad_form(x, P2) + q2.T*x + r2 <= slack,
-                       0.5*cvx.quad_form(x, P3) + q3.T*x + r3 <= slack,
+        objective = cvx.Minimize(0.5*cvx.quad_form(x, P0) + q0.T @ x + r0 + slack)
+        constraints = [0.5*cvx.quad_form(x, P1) + q1.T @ x + r1 <= slack,
+                       0.5*cvx.quad_form(x, P2) + q2.T @ x + r2 <= slack,
+                       0.5*cvx.quad_form(x, P3) + q3.T @ x + r3 <= slack,
                        ]
 
         # We now find the primal result and compare it to the dual result
         # to check if strong duality holds i.e. the duality gap is effectively zero
         p = cvx.Problem(objective, constraints)
-        p.solve()
+        p.solve(solver=cvx.SCS)
 
         # Note that since our data is random,
         # we may need to run this program multiple times to get a feasible primal
@@ -115,7 +115,7 @@ class TestExamples(BaseTest):
         self.assertEqual(intf.shape(dual_result), (1, 1))
 
     # Tests examples from the README.
-    def test_readme_examples(self):
+    def test_readme_examples(self) -> None:
         import numpy
         numpy.random.seed(1)
         # cvx.Problem data.
@@ -126,12 +126,12 @@ class TestExamples(BaseTest):
 
         # Construct the problem.
         x = cvx.Variable(n)
-        objective = cvx.Minimize(cvx.sum_squares(A*x - b))
+        objective = cvx.Minimize(cvx.sum_squares(A @ x - b))
         constraints = [0 <= x, x <= 1]
         p = cvx.Problem(objective, constraints)
 
         # The optimal objective is returned by p.solve().
-        p.solve()
+        p.solve(solver=cvx.SCS)
         # The optimal value for x is stored in x.value.
         print(x.value)
         # The optimal Lagrange multiplier for a constraint
@@ -190,13 +190,13 @@ class TestExamples(BaseTest):
 
         # Construct the problem.
         x = cvx.Variable(m)
-        objective = cvx.Minimize(cvx.sum_squares(A*x - b) + gamma*cvx.norm(x, 1))
+        objective = cvx.Minimize(cvx.sum_squares(A @ x - b) + gamma*cvx.norm(x, 1))
         p = cvx.Problem(objective)
 
         # Assign a value to gamma and find the optimal x.
         def get_x(gamma_value):
             gamma.value = gamma_value
-            p.solve()
+            p.solve(solver=cvx.SCS)
             return x.value
 
         gammas = np.logspace(-1, 2, num=2)
@@ -221,12 +221,12 @@ class TestExamples(BaseTest):
         # cvx.Variables:
         # x is a vector of stock holdings as fractions of total assets.
 
-        expected_return = mu*x
+        expected_return = mu @ x
         risk = cvx.quad_form(x, sigma)
 
         objective = cvx.Maximize(expected_return - gamma*risk)
         p = cvx.Problem(objective, [cvx.sum(x) == 1])
-        p.solve()
+        p.solve(solver=cvx.SCS)
 
         # The optimal expected return.
         print(expected_return.value)
@@ -252,24 +252,24 @@ class TestExamples(BaseTest):
         a = cvx.Variable(n)  # mi.SparseVar(n, nonzeros=6)
         b = cvx.Variable()
 
-        slack = [cvx.pos(1 - label*(sample.T*a - b)) for (label, sample) in data]
+        slack = [cvx.pos(1 - label*(sample.T @ a - b)) for (label, sample) in data]
         objective = cvx.Minimize(cvx.norm(a, 2) + gamma*sum(slack))
         p = cvx.Problem(objective)
         # Extensions can attach new solve methods to the CVXPY cvx.Problem class.
         # p.solve(method="admm")
-        p.solve()
+        p.solve(solver=cvx.SCS)
 
         # Count misclassifications.
         errors = 0
         for label, sample in data:
-            if label*(sample.T*a - b).value < 0:
+            if label*(sample.T @ a - b).value < 0:
                 errors += 1
 
         print("%s misclassifications" % errors)
         print(a.value)
         print(b.value)
 
-    def test_advanced1(self):
+    def test_advanced1(self) -> None:
         """Code from the advanced tutorial.
         """
         # Solving a problem with different solvers.
@@ -278,14 +278,9 @@ class TestExamples(BaseTest):
         constraints = [x >= 2]
         prob = cvx.Problem(obj, constraints)
 
-        # Solve with ECOS.
-        prob.solve(solver=cvx.ECOS)
-        print("optimal value with ECOS:", prob.value)
-        self.assertAlmostEqual(prob.value, 6)
-
-        # Solve with ECOS_BB.
-        prob.solve(solver=cvx.ECOS_BB)
-        print("optimal value with ECOS_BB:", prob.value)
+        # Solve with CLARABEL.
+        prob.solve(solver=cvx.CLARABEL)
+        print("optimal value with CLARABEL:", prob.value)
         self.assertAlmostEqual(prob.value, 6)
 
         # Solve with CVXOPT.
@@ -322,9 +317,15 @@ class TestExamples(BaseTest):
             print("optimal value with GUROBI:", prob.value)
             self.assertAlmostEqual(prob.value, 6)
 
+        if cvx.XPRESS in cvx.installed_solvers():
+            # Solve with the Xpress Optimizer.
+            prob.solve(solver=cvx.XPRESS)
+            print("optimal value with XPRESS:", prob.value)
+            self.assertAlmostEqual(prob.value, 6)
+
         print(cvx.installed_solvers())
 
-    def test_log_det(self):
+    def test_log_det(self) -> None:
         # Generate data
         x = np.array([[0.55, 0.0],
                       [0.25, 0.35],
@@ -340,12 +341,12 @@ class TestExamples(BaseTest):
         obj = cvx.Maximize(cvx.log_det(A))
         constraints = []
         for i in range(m):
-            constraints.append(cvx.norm(A*x[:, i] + b) <= 1)
+            constraints.append(cvx.norm(A @ x[:, i] + b) <= 1)
         p = cvx.Problem(obj, constraints)
-        result = p.solve()
+        result = p.solve(solver=cvx.SCS)
         self.assertAlmostEqual(result, 1.9746, places=2)
 
-    def test_portfolio_problem(self):
+    def test_portfolio_problem(self) -> None:
         """Test portfolio problem that caused dcp_attr errors.
         """
         import numpy as np
@@ -362,13 +363,13 @@ class TestExamples(BaseTest):
         Z = Z.dot(Z.T)
 
         x = cvx.Variable(n)
-        y = x.__rmul__(F)
+        y = F @ x
         # DCP attr causes error because not all the curvature
         # matrices are reduced to constants when an atom
         # is scalar.
-        cvx.square(cvx.norm(D*x)) + cvx.square(Z*y)
+        cvx.square(cvx.norm(D @ x)) + cvx.square(Z @ y)
 
-    def test_intro(self):
+    def test_intro(self) -> None:
         """Test examples from cvxpy.org introduction.
         """
         import numpy
@@ -382,12 +383,12 @@ class TestExamples(BaseTest):
 
         # Construct the problem.
         x = cvx.Variable(n)
-        objective = cvx.Minimize(cvx.sum_squares(A*x - b))
+        objective = cvx.Minimize(cvx.sum_squares(A @ x - b))
         constraints = [0 <= x, x <= 1]
         prob = cvx.Problem(objective, constraints)
 
         # The optimal objective is returned by p.solve().
-        prob.solve()
+        prob.solve(solver=cvx.SCS, eps=1e-6)
         # The optimal value for x is stored in x.value.
         print(x.value)
         # The optimal Lagrange multiplier for a constraint
@@ -409,7 +410,7 @@ class TestExamples(BaseTest):
 
         # Form and solve problem.
         prob = cvx.Problem(obj, constraints)
-        prob.solve()  # Returns the optimal value.
+        prob.solve(solver=cvx.SCS, eps=1e-6)  # Returns the optimal value.
         print("status:", prob.status)
         print("optimal value", prob.value)
         print("optimal var", x.value, y.value)
@@ -429,7 +430,7 @@ class TestExamples(BaseTest):
 
         # Form and solve problem.
         prob = cvx.Problem(obj, constraints)
-        prob.solve()  # Returns the optimal value.
+        prob.solve(solver=cvx.SCS, eps=1e-6)  # Returns the optimal value.
         print("status:", prob.status)
         print("optimal value", prob.value)
         print("optimal var", x.value, y.value)
@@ -443,7 +444,7 @@ class TestExamples(BaseTest):
 
         # Replace the objective.
         prob = cvx.Problem(cvx.Maximize(x + y), prob.constraints)
-        print("optimal value", prob.solve())
+        print("optimal value", prob.solve(solver=cvx.SCS, eps=1e-6))
 
         self.assertAlmostEqual(prob.value, 1.0, places=3)
 
@@ -451,7 +452,7 @@ class TestExamples(BaseTest):
         constraints = prob.constraints
         constraints[0] = (x + y <= 3)
         prob = cvx.Problem(prob.objective, constraints)
-        print("optimal value", prob.solve())
+        print("optimal value", prob.solve(solver=cvx.SCS, eps=1e-6))
 
         self.assertAlmostEqual(prob.value, 3.0, places=2)
 
@@ -461,7 +462,7 @@ class TestExamples(BaseTest):
 
         # An infeasible problem.
         prob = cvx.Problem(cvx.Minimize(x), [x >= 1, x <= 0])
-        prob.solve()
+        prob.solve(solver=cvx.SCS, eps=1e-6)
         print("status:", prob.status)
         print("optimal value", prob.value)
 
@@ -470,7 +471,7 @@ class TestExamples(BaseTest):
 
         # An unbounded problem.
         prob = cvx.Problem(cvx.Minimize(x))
-        prob.solve()
+        prob.solve(solver=cvx.CLARABEL)
         print("status:", prob.status)
         print("optimal value", prob.value)
 
@@ -500,11 +501,11 @@ class TestExamples(BaseTest):
 
         # Construct the problem.
         x = cvx.Variable(n)
-        objective = cvx.Minimize(cvx.sum_squares(A*x - b))
+        objective = cvx.Minimize(cvx.sum_squares(A @ x - b))
         constraints = [0 <= x, x <= 1]
         prob = cvx.Problem(objective, constraints)
 
-        print("Optimal value", prob.solve())
+        print("Optimal value", prob.solve(solver=cvx.SCS, eps=1e-6))
         print("Optimal var")
         print(x.value)  # A numpy matrix.
 
@@ -546,7 +547,7 @@ class TestExamples(BaseTest):
 
         # Construct the problem.
         x = cvx.Variable(m)
-        error = cvx.sum_squares(A*x - b)
+        error = cvx.sum_squares(A @ x - b)
         obj = cvx.Minimize(error + gamma*cvx.norm(x, 1))
         prob = cvx.Problem(obj)
 
@@ -557,7 +558,7 @@ class TestExamples(BaseTest):
         gamma_vals = numpy.logspace(-4, 6)
         for val in gamma_vals:
             gamma.value = val
-            prob.solve()
+            prob.solve(solver=cvx.SCS, eps=1e-6)
             # Use expr.value to get the numerical value of
             # an expression in the problem.
             sq_penalty.append(error.value)
@@ -573,7 +574,7 @@ class TestExamples(BaseTest):
         # Use expr.size to get the dimensions.
         print("dimensions of X:", X.size)
         print("dimensions of sum(X):", cvx.sum(X).size)
-        print("dimensions of A*X:", (A*X).size)
+        print("dimensions of A @ X:", (A @ X).size)
 
         # ValueError raised for invalid dimensions.
         try:
@@ -581,12 +582,12 @@ class TestExamples(BaseTest):
         except ValueError as e:
             print(e)
 
-    def test_inpainting(self):
+    def test_inpainting(self) -> None:
         """Test image in-painting.
         """
         import numpy as np
         np.random.seed(1)
-        rows, cols = 100, 100
+        rows, cols = 20, 20
         # Load the images.
         # Convert to arrays.
         Uorig = np.random.randint(0, 255, size=(rows, cols))
@@ -607,35 +608,7 @@ class TestExamples(BaseTest):
         prob = cvx.Problem(obj, constraints)
         prob.solve(solver=cvx.SCS)
 
-    def test_advanced2(self):
-        """Test code from the advanced section of the tutorial.
-        """
-        x = cvx.Variable()
-        prob = cvx.Problem(cvx.Minimize(cvx.square(x)), [x == 2])
-        # Get ECOS arguments.
-        data, chain, inverse = prob.get_problem_data(cvx.ECOS)
-
-        # Get ECOS_BB arguments.
-        data, chain, inverse = prob.get_problem_data(cvx.ECOS_BB)
-
-        # Get CVXOPT arguments.
-        if cvx.CVXOPT in cvx.installed_solvers():
-            data, chain, inverse = prob.get_problem_data(cvx.CVXOPT)
-
-        # Get SCS arguments.
-        data, chain, inverse = prob.get_problem_data(cvx.SCS)
-
-        import ecos
-        # Get ECOS arguments.
-        data, chain, inverse = prob.get_problem_data(cvx.ECOS)
-        # Call ECOS solver.
-        solution = ecos.solve(data["c"], data["G"], data["h"],
-                              ecos_conif.dims_to_solver_dict(data["dims"]),
-                              data["A"], data["b"])
-        # Unpack raw solver output.
-        prob.unpack_results(solution, chain, inverse)
-
-    def test_log_sum_exp(self):
+    def test_log_sum_exp(self) -> None:
         """Test log_sum_exp function that failed in Github issue.
         """
         import numpy as np
@@ -645,48 +618,12 @@ class TestExamples(BaseTest):
         X = np.ones((m, n))
         w = cvx.Variable(n)
 
-        expr2 = [cvx.log_sum_exp(cvx.hstack([0, X[i, :]*w])) for i in range(m)]
+        expr2 = [cvx.log_sum_exp(cvx.hstack([0, X[i, :] @ w])) for i in range(m)]
         expr3 = sum(expr2)
         obj = cvx.Minimize(expr3)
         p = cvx.Problem(obj)
         p.solve(solver=cvx.SCS, max_iters=1)
 
-    # # Risk return tradeoff curve
-    # def test_risk_return_tradeoff(self):
-    #     from math import sqrt
-    #     from cvxopt import matrix
-    #     from cvxopt.blas import dot
-    #     from cvxopt.solvers import qp, options
-    #     import scipy
-
-    #     n = 4
-    #     S = matrix( [[ 4e-2,  6e-3, -4e-3,   0.0 ],
-    #                  [ 6e-3,  1e-2,  0.0,    0.0 ],
-    #                  [-4e-3,  0.0,   2.5e-3, 0.0 ],
-    #                  [ 0.0,   0.0,   0.0,    0.0 ]] )
-    #     pbar = matrix([.12, .10, .07, .03])
-
-    #     N = 100
-    #     # CVXPY
-    #     Sroot = numpy.asmatrix(scipy.linalg.sqrtm(S))
-    #     x = cvx.Variable(n, name='x')
-    #     mu = cvx.Parameter(name='mu')
-    #     mu.value = 1 # TODO cvx.Parameter("positive")
-    #     objective = cvx.Minimize(-pbar*x + mu*quad_over_lin(Sroot*x,1))
-    #     constraints = [sum(x) == 1, x >= 0]
-    #     p = cvx.Problem(objective, constraints)
-
-    #     mus = [ 10**(5.0*t/N-1.0) for t in range(N) ]
-    #     xs = []
-    #     for mu_val in mus:
-    #         mu.value = mu_val
-    #         p.solve()
-    #         xs.append(x.value)
-    #     returns = [ dot(pbar,x) for x in xs ]
-    #     risks = [ sqrt(dot(x, S*x)) for x in xs ]
-
-    #     # QP solver
-
 
 if __name__ == '__main__':
     unittest.main()
