@@ -17,17 +17,28 @@ limitations under the License.
 import warnings
 
 import numpy as np
+import pytest
 import scipy.sparse as sp
+from hypothesis import assume, given
+from hypothesis.extra.numpy import arrays, basic_indices, integer_array_indices
 
 import cvxpy as cp
 import cvxpy.interface.matrix_utilities as intf
 import cvxpy.settings as s
 from cvxpy import Minimize, Problem
 from cvxpy.atoms.affine.add_expr import AddExpression
+from cvxpy.atoms.affine.wraps import (
+    hermitian_wrap,
+    nonneg_wrap,
+    nonpos_wrap,
+    psd_wrap,
+    skew_symmetric_wrap,
+    symmetric_wrap,
+)
 from cvxpy.expressions.constants import Constant, Parameter
 from cvxpy.expressions.variable import Variable
 from cvxpy.tests.base_test import BaseTest
-from cvxpy.utilities.eigvals import gershgorin_psd_check
+from cvxpy.utilities.linalg import gershgorin_psd_check
 
 
 class TestExpressions(BaseTest):
@@ -60,27 +71,14 @@ class TestExpressions(BaseTest):
         # self.assertEqual(x.canonical_form[0].shape, (2, 1))
         # self.assertEqual(x.canonical_form[1], [])
 
-        self.assertEqual(repr(self.x), "Variable((2,))")
-        self.assertEqual(repr(self.A), "Variable((2, 2))")
+        self.assertEqual(repr(self.x), "Variable((2,), x)")
+        self.assertEqual(repr(self.A), "Variable((2, 2), A)")
+        self.assertEqual(repr(cp.Variable(name='x', nonneg=True)),
+                         "Variable((), x, nonneg=True)")
+        self.assertTrue(repr(cp.Variable()).startswith("Variable((), var"))
 
-        # # Scalar variable
-        # coeff = self.a.coefficients()
-        # self.assertEqual(coeff[self.a.id], [1])
-
-        # # Vector variable.
-        # coeffs = x.coefficients()
-        # self.assertItemsEqual(coeffs.keys(), [x.id])
-        # vec = coeffs[x.id][0]
-        # self.assertEqual(vec.shape, (2,2))
-        # self.assertEqual(vec[0,0], 1)
-
-        # # Matrix variable.
-        # coeffs = self.A.coefficients()
-        # self.assertItemsEqual(coeffs.keys(), [self.A.id])
-        # self.assertEqual(len(coeffs[self.A.id]), 2) or 0 in self.shape
-        # mat = coeffs[self.A.id][1]
-        # self.assertEqual(mat.shape, (2,4))
-        # self.assertEqual(mat[0,2], 1)
+        # Test shape provided as list instead of tuple
+        self.assertEqual(cp.Variable(shape=[2], integer=True).shape, (2,))
 
         with self.assertRaises(Exception) as cm:
             Variable((2, 2), diag=True, symmetric=True)
@@ -216,6 +214,12 @@ class TestExpressions(BaseTest):
         # Test repr.
         self.assertEqual(repr(c), "Constant(CONSTANT, NONNEGATIVE, (2,))")
 
+        # Test name.
+        c = Constant(1, name="test")
+        self.assertEqual(str(c), "test")
+        self.assertEqual(c.name(), "test")
+        self.assertEqual(repr(c), "Constant(CONSTANT, NONNEGATIVE, ())")
+
     def test_constant_psd_nsd(self):
         n = 5
         np.random.randn(0)
@@ -276,6 +280,59 @@ class TestExpressions(BaseTest):
         self.assertFalse(gershgorin_psd_check(Q.value, s.EIGVAL_TOL))
         self.assertFalse(Q.is_psd())
 
+    def test_constant_skew_symmetric(self) -> None:
+        # Define inputs
+        M1_false = np.eye(3)
+        M2_true = np.zeros((3, 3))
+        M3_true = np.array([[0, 1], [-1, 0]])
+        M4_true = np.array([[0, -1], [1, 0]])
+        M5_false = np.array([[0, 1], [1,  0]])
+        M6_false = np.array([[1, 1], [-1, 0]])
+        M7_false = np.array([[0, 1], [-1.1, 0]])
+
+        # Test dense constants
+        C = Constant(M1_false)
+        self.assertFalse(C.is_skew_symmetric())
+        C = Constant(M2_true)
+        self.assertTrue(C.is_skew_symmetric())
+        C = Constant(M3_true)
+        self.assertTrue(C.is_skew_symmetric())
+        C = Constant(M4_true)
+        self.assertTrue(C.is_skew_symmetric())
+        C = Constant(M5_false)
+        self.assertFalse(C.is_skew_symmetric())
+        C = Constant(M6_false)
+        self.assertFalse(C.is_skew_symmetric())
+        C = Constant(M7_false)
+        self.assertFalse(C.is_skew_symmetric())
+
+        # Test sparse constants
+        C = Constant(sp.csc_matrix(M1_false))
+        self.assertFalse(C.is_skew_symmetric())
+        C = Constant(sp.csc_matrix(M2_true))
+        self.assertTrue(C.is_skew_symmetric())
+        C = Constant(sp.csc_matrix(M4_true))
+        self.assertTrue(C.is_skew_symmetric())
+        C = Constant(sp.csc_matrix(M5_false))
+        self.assertFalse(C.is_skew_symmetric())
+        C = Constant(sp.csc_matrix(M6_false))
+        self.assertFalse(C.is_skew_symmetric())
+        C = Constant(sp.csc_matrix(M7_false))
+        self.assertFalse(C.is_skew_symmetric())
+
+        # Test complex inputs: never recognized as skew-symmetric.
+        C = Constant(1j * M2_true)
+        self.assertFalse(C.is_skew_symmetric())
+        #   ^ From a mathematical standpoint one can argue that this should
+        #     be true, but I don't think there's precedent for CVXPY
+        #     automatically converting complex expressions with zero imaginary-part
+        #     into equivalent real expressions. -- Riley
+        C = Constant(1j * M3_true)
+        self.assertFalse(C.is_skew_symmetric())
+        C = Constant(1j * M4_true)
+        self.assertFalse(C.is_skew_symmetric())
+        pass
+
     def test_1D_array(self) -> None:
         """Test NumPy 1D arrays as constants.
         """
@@ -950,6 +1007,15 @@ class TestExpressions(BaseTest):
         self.assertEqual(exp.curvature, s.AFFINE)
         self.assertEqual(exp.shape, (1,))
 
+    def test_special_idx_str_repr(self) -> None:
+        idx = [i for i in range(178)]
+        exp = cp.Variable((200, 10), name="exp")[idx, 6]
+        self.assertEqual("exp[[0, 1, 2, ..., 175, 176, 177], 6]", str(exp))
+
+        idx = [i for i in range(5)]
+        exp = cp.Variable((10, 10), name="exp")[idx, 2:5]
+        self.assertEqual("exp[[0, 1, 2, 3, 4], 2:5]", str(exp))
+
     def test_none_idx(self) -> None:
         """Test None as index.
         """
@@ -978,7 +1044,7 @@ class TestExpressions(BaseTest):
         self.assertEqual(str(cm.exception), "Index -100 is out of bounds for axis 0 with size 2.")
 
         exp = self.x[:-100]
-        self.assertEqual(exp.size, (0,))
+        self.assertEqual(exp.size, 0)
         self.assertItemsAlmostEqual(exp.value, np.array([]))
 
         exp = self.C[100:2]
@@ -990,6 +1056,24 @@ class TestExpressions(BaseTest):
         exp = self.C[:, -199:-3]
         self.assertEqual(exp.shape, (3, 0))
 
+    def test_float_is_invalid_index(self) -> None:
+        with self.assertRaises(IndexError) as cm:
+            self.x[1.0]
+        self.assertEqual(str(cm.exception), "float is an invalid index type.")
+
+        with self.assertRaises(IndexError) as cm:
+            self.x[(1.0,)]
+        self.assertEqual(str(cm.exception), "float is an invalid index type.")
+
+        with self.assertRaises(IndexError) as cm:
+            self.C[: 2.:40]
+        self.assertEqual(str(cm.exception), "float is an invalid index type.")
+
+        with self.assertRaises(IndexError) as cm:
+            self.x[np.array([1.0, 2.0])]
+        self.assertEqual(str(cm.exception),
+                         "arrays used as indices must be of integer (or boolean) type")
+
     def test_neg_indices(self) -> None:
         """Test negative indices.
         """
@@ -1226,10 +1310,6 @@ class TestExpressions(BaseTest):
 
         col_scale = Variable(n)
 
-        with self.assertRaises(ValueError) as cm:
-            cp.multiply(A, col_scale)
-        self.assertEqual(str(cm.exception), "Cannot broadcast dimensions  (3, 4) (4,)")
-
         col_scale = Variable([1, n])
         C = cp.multiply(A, col_scale)
         self.assertEqual(C.shape, (m, n))
@@ -1258,10 +1338,6 @@ class TestExpressions(BaseTest):
 
         col_scale = Variable(n)
 
-        with self.assertRaises(ValueError) as cm:
-            A + col_scale
-        self.assertEqual(str(cm.exception), "Cannot broadcast dimensions  (3, 4) (4,)")
-
         col_scale = Variable([1, n])
         C = A + col_scale
         self.assertEqual(C.shape, (m, n))
@@ -1270,6 +1346,29 @@ class TestExpressions(BaseTest):
         R = A + row_scale
         self.assertEqual(R.shape, (m, n))
 
+    def test_curvatures(self) -> None:
+        """Test the curvatures property returns a list of valid curvatures"""
+        mat = np.array([[1, -1]])
+        self.assertEqual(cp.sum(mat @ cp.square(Variable(2))).curvatures, [s.UNKNOWN])
+
+        y = cp.Constant(42)
+        self.assertIn(s.CONSTANT, y.curvatures)
+
+        x = cp.Variable(pos=True)
+        self.assertEqual(x.curvatures, [s.AFFINE, s.CONVEX, s.CONCAVE,
+                                        s.LOG_LOG_AFFINE, s.LOG_LOG_CONVEX, s.LOG_LOG_CONCAVE,
+                                        s.QUASILINEAR, s.QUASICONVEX, s.QUASICONCAVE])
+
+        monomial = x*x*x
+        self.assertEqual(monomial.curvatures, [s.LOG_LOG_AFFINE,
+                                               s.LOG_LOG_CONVEX, s.LOG_LOG_CONCAVE])
+
+        posynomial = x*x*x + x
+        self.assertEqual(posynomial.curvatures, [s.LOG_LOG_CONVEX])
+
+        llcv = 1/(x*x*x + x)
+        self.assertEqual(llcv.curvatures, [s.LOG_LOG_CONCAVE])
+
     def test_log_log_curvature(self) -> None:
         """Test that the curvature string is populated for log-log expressions.
         """
@@ -1282,3 +1381,335 @@ class TestExpressions(BaseTest):
 
         llcv = 1/(x*x*x + x)
         assert llcv.curvature == s.LOG_LOG_CONCAVE
+
+    def test_quad_form_matmul(self) -> None:
+        """Test conversion of native x.T @ A @ x into QuadForms.
+        """
+
+        # Trivial quad form
+        x = Variable(shape=(2,))
+        A = Constant([[1, 0], [0, -1]])
+        expr = x.T.__matmul__(A).__matmul__(x)
+        assert isinstance(expr, cp.QuadForm)
+
+        # QuadForm inside nested expr: 0.5 * (x.T @ A @ x) + x.T @ x
+        x = Variable(shape=(2,))
+        A = Constant([[1, 0], [0, -1]])
+        expr = (1 / 2) * (x.T.__matmul__(A).__matmul__(x)) + x.T.__matmul__(x)
+        assert isinstance(expr.args[0].args[1], cp.QuadForm)
+        assert expr.args[0].args[1].args[0] is x
+
+        # QuadForm inside nested expr: (0.5 * c.T @ c) * (x.T @ A @ x) + x.T @ x
+        x = Variable(shape=(2,))
+        A = Constant([[1, 0], [0, -1]])
+        c = Constant([2, -2])
+        expr = (1 / 2 * c.T.__matmul__(c)) * (x.T.__matmul__(A).__matmul__(x)) + x.T.__matmul__(x)
+        assert isinstance(expr.args[0].args[1], cp.QuadForm)
+        assert expr.args[0].args[1].args[0] is x
+
+        # QuadForm with sparse matrices
+        x = Variable(shape=(2,))
+        A = Constant(sp.eye(2))
+        expr = x.T.__matmul__(A).__matmul__(x)
+        assert isinstance(expr, cp.QuadForm)
+
+        # QuadForm with mismatched dimensions raises error
+        x = Variable(shape=(2,))
+        A = Constant(np.eye(3))
+        with self.assertRaises(Exception) as _:
+            x.T.__matmul__(A).__matmul__(x)
+
+        # QuadForm with PSD-wrapped matrix
+        x = cp.Variable(shape=(2,))
+        A = cp.Constant([[1, 0], [0, 1]])
+        expr = x.T.__matmul__(psd_wrap(A)).__matmul__(x)
+        assert isinstance(expr, cp.QuadForm)
+
+        # QuadForm with nested subexpr
+        x = cp.Variable(shape=(2,))
+        A = cp.Constant([[2, 0, 0], [0, 0, 1]])
+        M = cp.Constant([[2, 0, 0], [0, 2, 0], [0, 0, 2]])
+        b = cp.Constant([1, 2, 3])
+
+        y = A.__matmul__(x) - b
+        expr = y.T.__matmul__(M).__matmul__(y)
+        assert isinstance(expr, cp.QuadForm)
+        assert expr.args[0] is y
+        assert expr.args[1] is M
+
+        # QuadForm with parameters
+        x = Variable(shape=(2,))
+        A = Parameter(shape=(2, 2), symmetric=True)
+        expr = x.T.__matmul__(A).__matmul__(x)
+        assert isinstance(expr, cp.QuadForm)
+
+        # Expect error for asymmetric/nonhermitian matrices
+        x = Variable(shape=(2,))
+        A = Constant([[1, 0], [1, 1]])
+        with self.assertRaises(ValueError) as _:
+            x.T.__matmul__(A).__matmul__(x)
+
+        x = Variable(shape=(2,))
+        A = Constant([[1, 1j], [1j, 1]])
+        with self.assertRaises(ValueError) as _:
+            x.T.__matmul__(A).__matmul__(x)
+
+        # Not a quad_form because x.T @ A @ y where x, y not necessarily equal
+        x = Variable(shape=(2,))
+        y = Variable(shape=(2,))
+        A = Constant([[1, 0], [0, -1]])
+        expr = x.T.__matmul__(A).__matmul__(y)
+        assert not isinstance(expr, cp.QuadForm)
+
+        # Not a quad_form because M is variable
+        x = Variable(shape=(2,))
+        M = Variable(shape=(2, 2))
+        expr = x.T.__matmul__(M).__matmul__(x)
+        assert not isinstance(expr, cp.QuadForm)
+
+        x = Constant([1, 0])
+        M = Variable(shape=(2, 2))
+        expr = x.T.__matmul__(M).__matmul__(x)
+        assert not isinstance(expr, cp.QuadForm)
+
+    def test_matmul_scalars(self) -> None:
+        """Test evaluating a matmul that reduces one argument internally to a scalar.
+        """
+        x = cp.Variable((2,))
+        quad = cp.quad_form(x, np.eye(2))
+        a = np.array([2])
+        expr = quad * a
+        x.value = np.array([1, 2])
+        P = np.eye(2)
+        true_val = (np.transpose(x.value) @ P @ x.value) * a
+        assert quad.shape == ()
+        self.assertEqual(expr.value, true_val)
+
+    def test_wraps(self) -> None:
+        """Test wrap classes."""
+        x = cp.Variable(2)
+        expr = nonneg_wrap(x)
+        assert expr.is_nonneg()
+
+        expr = nonpos_wrap(x)
+        assert expr.is_nonpos()
+
+        Z = cp.Variable((2, 2))
+        U = cp.Variable((2, 2), complex=True)
+        expr = psd_wrap(Z)
+        assert expr.is_psd()
+        assert not expr.is_complex()
+        assert expr.is_symmetric()
+        assert expr.is_hermitian()
+
+        expr = psd_wrap(U)
+        assert expr.is_psd()
+        assert expr.is_complex()
+        assert not expr.is_symmetric()
+        assert expr.is_hermitian()
+
+        expr = symmetric_wrap(Z)
+        assert expr.is_symmetric()
+        assert expr.is_hermitian()
+
+        expr = skew_symmetric_wrap(Z)
+        assert expr.is_skew_symmetric()
+
+        expr = hermitian_wrap(U)
+        assert expr.is_hermitian()
+
+    def test_expr_does_not_support_cpp_warning(self):
+        from cvxpy.atoms.affine.sum import Sum
+
+        class SumNotSupportedInCPP(Sum):
+            def _supports_cpp(self):
+                return False
+
+        x = Variable(2)
+        prob = Problem(Minimize(0), [SumNotSupportedInCPP(x) == 1])
+
+        with pytest.warns(
+            UserWarning,
+            match="The problem includes expressions that don't support "
+            "CPP backend. Defaulting to the SCIPY backend "
+            "for canonicalization.",
+        ):
+            prob.solve()
+
+
+    def test_expr_does_not_support_cpp_error(self):
+        from cvxpy.atoms.affine.sum import Sum
+
+        class SumNotSupportedInCPP(Sum):
+            def _supports_cpp(self):
+                return False
+
+        x = Variable(2)
+        prob = Problem(Minimize(0), [SumNotSupportedInCPP(x) == 1])
+
+        with pytest.raises(
+            ValueError,
+            match="The CPP backend cannot be used with problems "
+            "that have expressions which do not support it",
+        ):
+            prob.solve(canon_backend=cp.CPP_CANON_BACKEND)
+
+
+class TestND_Expressions():
+
+    @pytest.fixture(autouse=True)
+    def setup(self) -> None:
+        self.x = Variable((2,2,2), name='x')
+        self.target = (1+np.arange(8)).reshape(2,2,2)
+        self.obj = cp.Minimize(0)
+
+    def test_nd_variable(self) -> None:
+        prob = cp.Problem(self.obj, [self.x == self.target])
+        prob.solve(canon_backend=cp.SCIPY_CANON_BACKEND)
+        assert np.allclose(self.x.value, self.target)
+
+    def test_nd_variable_warning(self) -> None:
+        prob = cp.Problem(self.obj, [self.x == self.target])
+        warning_str = "The problem has an expression with dimension greater than 2. " \
+                    "Defaulting to the SCIPY backend for canonicalization."
+        with pytest.warns(UserWarning, match=warning_str):
+            prob.solve()
+
+    def test_nd_variable_value_error(self) -> None:
+        prob = cp.Problem(self.obj, [self.x == self.target])
+        error_str = "Only the SCIPY and NUMPY backends are supported " \
+                    "for problems with expressions of dimension greater than 2."
+        with pytest.raises(ValueError, match=error_str):
+            prob.solve(canon_backend=cp.CPP_CANON_BACKEND)
+
+    def test_nd_mul_elem(self) -> None:
+        expr = cp.multiply(self.x, 3)
+        prob = cp.Problem(self.obj, [expr == self.target])
+        prob.solve(canon_backend=cp.SCIPY_CANON_BACKEND)
+        assert np.allclose(expr.value, self.target)
+
+    def test_nd_div(self) -> None:
+        expr = self.x / self.target
+        prob = cp.Problem(self.obj, [expr == self.target])
+        prob.solve(canon_backend=cp.SCIPY_CANON_BACKEND)
+        assert np.allclose(expr.value, self.target)
+
+    def test_nd_vstack(self) -> None:
+        x = Variable((1, 2, 2))
+        z = Variable((1, 2, 2))
+        expr = cp.vstack([x,z])
+        prob = cp.Problem(self.obj, [expr == self.target])
+        prob.solve(canon_backend=cp.SCIPY_CANON_BACKEND)
+        assert np.allclose(expr.value, self.target)
+
+    def test_nd_concatenate(self) -> None:
+        x = cp.Variable((1, 2, 2))
+        z = cp.Variable((1, 2, 2))
+        expr = cp.concatenate([x,z], axis = 0)
+        prob = cp.Problem(self.obj, [expr == self.target])
+        prob.solve(canon_backend=cp.SCIPY_CANON_BACKEND)
+        assert np.allclose(expr.value, self.target)
+
+    def test_nd_sum_expr(self) -> None:
+        x = [cp.Variable((2,2,2)) for _ in range(10)]
+        expr = sum(x)
+        prob = cp.Problem(self.obj, [expr == self.target])
+        prob.solve(canon_backend=cp.SCIPY_CANON_BACKEND)
+        assert np.allclose(expr.value, self.target)
+
+    @pytest.mark.parametrize("axis", [(0),(1),(2),((0,1)),((0,2)),((2,1))])
+    def test_nd_sum(self, axis) -> None:
+        expr = cp.sum(self.x, axis=axis, keepdims=True)
+        y = self.target.sum(axis=axis, keepdims=True)
+        prob = cp.Problem(self.obj, [expr == y])
+        prob.solve(canon_backend=cp.SCIPY_CANON_BACKEND)
+        assert np.allclose(expr.value, y)
+
+    @pytest.mark.parametrize("axis", [(0),(1),(2),((0,1)),((0,2)),((2,1))])
+    def test_nd_parametrized_sum(self, axis) -> None:
+        param = cp.Parameter((2,2,2))
+        param.value = np.arange(8).reshape(2,2,2)
+        expr = cp.multiply(self.x, param).sum(axis=axis)
+        target = self.target.sum(axis=axis)
+        prob = cp.Problem(self.obj, [expr == target])
+        prob.solve(canon_backend=cp.SCIPY_CANON_BACKEND)
+        assert np.allclose(expr.value, target)
+
+    @pytest.mark.parametrize("axis", [(0,2,4,5),((4,5)),((0,2,3,1)),((5,3,1)), ((0,1,2,5))])
+    def test_nd_big_sum(self, axis) -> None:
+        in_shape = (6,5,4,3,2,1)
+        expr = cp.Variable(shape=in_shape).sum(axis=axis, keepdims=True)
+        y = np.ones(in_shape).sum(axis=axis, keepdims=True)
+        prob = cp.Problem(self.obj, [expr == y])
+        prob.solve(canon_backend=cp.SCIPY_CANON_BACKEND)
+        assert np.allclose(expr.value, y)
+
+    @given(integer_array_indices(shape=(2,2,2)))
+    def test_nd_integer_index(self, s) -> None:
+        expr = self.x[s]
+        y = self.target[s]
+        prob = cp.Problem(self.obj, [expr == y])
+        prob.solve(canon_backend=cp.SCIPY_CANON_BACKEND)
+        assert np.allclose(expr.value, y)
+
+    @given(axis=basic_indices(shape=(2,2,2), allow_newaxis=True))
+    def test_nd__basic_index(self, axis) -> None:
+        # Skip examples with 0-d output. TODO allow 0-d expressions in cvxpy.
+        def is_zero_dim_output(axis):
+            return 0 in self.target[axis].shape
+
+        assume(is_zero_dim_output(axis) is False)
+        expr = self.x[axis]
+        y = self.target[axis]
+        prob = cp.Problem(self.obj, [expr == y])
+        prob.solve(canon_backend=cp.SCIPY_CANON_BACKEND)
+        assert np.allclose(expr.value, y)
+
+    @given(axis=integer_array_indices(shape=(6,5,4,3,2,1)))
+    def test_nd_big_index(self, axis) -> None:
+        in_shape = (6,5,4,3,2,1)
+        expr = cp.Variable(shape=in_shape)[axis]
+        y = np.ones(in_shape)[axis]
+        prob = cp.Problem(self.obj, [expr == y])
+        prob.solve(canon_backend=cp.SCIPY_CANON_BACKEND)
+        assert np.allclose(expr.value, y)
+
+    @given(axis=arrays(shape=(2,2,2), dtype=bool))
+    def test_nd_bool_index(self, axis) -> None:
+        def is_zero_dim_output(axis):
+            return 0 in self.target[axis].shape
+
+        assume(is_zero_dim_output(axis) is False)
+        expr = self.x[axis]
+        y = self.target[axis]
+        prob = cp.Problem(self.obj, [expr == y])
+        prob.solve(canon_backend=cp.SCIPY_CANON_BACKEND)
+        assert np.allclose(expr.value, y)
+
+    def test_nd_index_sum(self) -> None:
+        expr = self.x[:,:,0].sum(axis=0)
+        y = self.target[:,:,0].sum(axis=0)
+        prob = cp.Problem(self.obj, [expr == y])
+        prob.solve(canon_backend=cp.SCIPY_CANON_BACKEND)
+        assert np.allclose(expr.value, y)
+
+    @pytest.mark.parametrize("order", ['C', 'F'])
+    @pytest.mark.parametrize("shape", [(20, 2, 30), (300, 2, 2),
+                                       (1, 24, 5, 10), (240, 5, 1)])
+    def test_nd_reshape(self, order, shape) -> None:
+        var = cp.Variable((5, 24, 10))
+        target = np.arange(1200).reshape((5, 24, 10))
+        expr = cp.reshape(var, shape, order=order)
+        y = target.reshape(shape, order=order)
+        prob = cp.Problem(self.obj, [expr == y])
+        prob.solve(canon_backend=cp.SCIPY_CANON_BACKEND)
+        assert np.allclose(expr.value, y)
+
+    def test_nd_transpose(self) -> None:
+        var = cp.Variable((5, 24, 10))
+        target = np.arange(1200).reshape((5, 24, 10))
+        expr = var.T
+        y = target.T
+        prob = cp.Problem(self.obj, [expr == y])
+        prob.solve(canon_backend=cp.SCIPY_CANON_BACKEND)
+        assert np.allclose(expr.value, y)
