@@ -16,8 +16,6 @@ limitations under the License.
 
 # Utility functions to handle indexing/slicing into an expression.
 
-from __future__ import division
-
 import numbers
 from typing import Optional, Tuple
 
@@ -40,6 +38,11 @@ def validate_key(key, shape: Tuple[int, ...]):
         Error: Index/slice out of bounds.
     """
     key = to_tuple(key)
+    if any(isinstance(k, float) or (isinstance(k, slice) and (
+            isinstance(k.start, float) or
+            isinstance(k.stop, float) or
+            isinstance(k.step, float))) for k in key):
+        raise IndexError("float is an invalid index type.")
     if len(key) == 0:
         raise IndexError("An index cannot be empty.")
     # Change single indices for vectors into double indices.
@@ -206,3 +209,24 @@ def is_special_slice(key) -> bool:
             return True
 
     return False
+
+
+def special_key_to_str(key):
+    """Converts a special key to a string representation."""
+    key_strs = []
+    for k in key:
+        if isinstance(k, (np.ndarray, list)):
+            key_strs.append(pprint_sequence(k))
+        elif isinstance(k, slice):
+            key_strs.append(slice_to_str(k))
+        else:
+            key_strs.append(str(k))
+    return ", ".join(key_strs)
+
+
+def pprint_sequence(seq, max_elems=6):
+    """Shorten the sequence (array or list) for pretty-printing."""
+    if len(seq) > max_elems:
+        half = max_elems // 2
+        return str(seq[:half])[:-1] + ', ..., ' + str(seq[-half:])[1:]
+    return str(seq)
