@@ -40,32 +40,18 @@ def get_diff_mat(dim: int, axis: int) -> sp.csc_matrix:
 
     Returns
     -------
-    SciPy CSC matrix
+    sp.csc_matrix
         A square matrix representing first order difference.
     """
-    # Construct a sparse matrix representation.
-    val_arr = []
-    row_arr = []
-    col_arr = []
-    for i in range(dim):
-        val_arr.append(1.)
-        row_arr.append(i)
-        col_arr.append(i)
-        if i > 0:
-            val_arr.append(-1.)
-            row_arr.append(i)
-            col_arr.append(i-1)
-
-    mat = sp.csc_matrix((val_arr, (row_arr, col_arr)),
-                        (dim, dim))
-    if axis == 0:
-        return mat
-    else:
-        return mat.T
+    mat = sp.diags([np.ones(dim), -np.ones(dim - 1)], [0, -1], 
+                   shape=(dim, dim), 
+                   format='csc')
+    return mat if axis == 0 else mat.T
 
 
 class cumsum(AffAtom, AxisAtom):
-    """Cumulative sum.
+    """
+    Cumulative sum of the elements of an expression.
 
     Attributes
     ----------
@@ -79,13 +65,13 @@ class cumsum(AffAtom, AxisAtom):
 
     @AffAtom.numpy_numeric
     def numeric(self, values):
-        """Convolve the two values.
+        """
+        Returns the cumulative sum of elements of an expression over an axis.
         """
         return np.cumsum(values[0], axis=self.axis)
 
     def shape_from_args(self) -> Tuple[int, ...]:
-        """The same as the input.
-        """
+        """The same as the input."""
         return self.args[0].shape
 
     def _grad(self, values):
@@ -99,12 +85,8 @@ class cumsum(AffAtom, AxisAtom):
         Returns:
             A list of SciPy CSC sparse matrices or None.
         """
-        # TODO inefficient
         dim = values[0].shape[self.axis]
-        mat = np.zeros((dim, dim))
-        for i in range(dim):
-            for j in range(i+1):
-                mat[i, j] = 1
+        mat = sp.tril(np.ones((dim, dim)))
         var = Variable(self.args[0].shape)
         if self.axis == 0:
             grad = MulExpression(mat, var)._grad(values)[1]
@@ -113,8 +95,7 @@ class cumsum(AffAtom, AxisAtom):
         return [grad]
 
     def get_data(self):
-        """Returns the axis being summed.
-        """
+        """Returns the axis being summed."""
         return [self.axis]
 
     def graph_implementation(
