@@ -35,6 +35,15 @@ class trace(AffAtom):
     def __init__(self, expr) -> None:
         super(trace, self).__init__(expr)
 
+    def sign_from_args(self) -> Tuple[bool, bool]:
+        """Trace is nonneg (nonpos) if its argument is elementwise nonneg
+        (nonpos) or psd (nsd).
+        """
+        is_nonneg = self.args[0].is_nonneg() or self.args[0].is_psd()
+        is_nonpos = self.args[0].is_nonpos() or self.args[0].is_nsd()
+
+        return is_nonneg, is_nonpos
+
     @AffAtom.numpy_numeric
     def numeric(self, values):
         """Sums the diagonal entries.
@@ -53,6 +62,12 @@ class trace(AffAtom):
         """
         return tuple()
 
+    def is_real(self) -> bool:
+        return self.args[0].is_real() or self.args[0].is_hermitian()
+
+    def is_complex(self) -> bool:
+        return not self.is_real()
+
     def is_atom_log_log_convex(self) -> bool:
         """Is the atom log-log convex?
         """
