@@ -14,8 +14,6 @@ See the License for the specific language governing permissions and
 limitations under the License.
 """
 
-from __future__ import division
-
 import warnings
 from typing import Tuple
 
@@ -23,9 +21,11 @@ import numpy as np
 import scipy.sparse as sp
 from scipy import linalg as LA
 
+from cvxpy.atoms.affine.wraps import psd_wrap
 from cvxpy.atoms.atom import Atom
 from cvxpy.expressions.expression import Expression
 from cvxpy.interface.matrix_utilities import is_sparse
+from cvxpy.utilities.linalg import sparse_cholesky
 
 
 class CvxPyDomainError(Exception):
@@ -36,6 +36,7 @@ class QuadForm(Atom):
     _allow_complex = True
 
     def __init__(self, x, P) -> None:
+        """Atom representing :math:`x^T P x`."""
         super(QuadForm, self).__init__(x, P)
 
     def numeric(self, values):
@@ -52,7 +53,7 @@ class QuadForm(Atom):
         if self.args[1].shape[1] != n or self.args[0].shape not in [(n, 1), (n,)]:
             raise ValueError("Invalid dimensions for arguments.")
         if not self.args[1].is_hermitian():
-            raise ValueError("P must be symmetric/Hermitian.")
+            raise ValueError("Quadratic form matrices must be symmetric/Hermitian.")
 
     def sign_from_args(self) -> Tuple[bool, bool]:
         """Returns sign (is positive, is negative) of the expression.
@@ -98,6 +99,11 @@ class QuadForm(Atom):
         """
         return True
 
+    def has_quadratic_term(self) -> bool:
+        """Always a quadratic term.
+        """
+        return True
+
     def is_pwl(self) -> bool:
         """Is the atom piecewise linear?
         """
@@ -111,11 +117,11 @@ class QuadForm(Atom):
     def _grad(self, values):
         x = np.array(values[0])
         P = np.array(values[1])
-        D = (P + np.conj(P.T)) @ x.T
-        return [sp.csc_matrix(D.ravel(order='F')).T]
+        D = (P + np.conj(P.T)) @ x
+        return [sp.csc_matrix(D.ravel(order="F")).T]
 
     def shape_from_args(self) -> Tuple[int, ...]:
-        return tuple() if self.args[0].ndim == 0 else (1, 1)
+        return tuple()
 
 
 class SymbolicQuadForm(Atom):
@@ -190,7 +196,15 @@ def decomp_quad(P, cond=None, rcond=None, lower=True, check_finite: bool = True)
 
     """
     if is_sparse(P):
-        P = np.array(P.todense())  # make dense (needs to happen for eigh).
+        # TODO: consider using QDLDL instead, if available.
+        try:
+            sign, L, p = sparse_cholesky(P)
+            if sign > 0:
+                return 1.0, L[p, :], np.empty((0, 0))
+            else:
+                return 1.0, np.empty((0, 0)), L[:, p]
+        except (ValueError, ModuleNotFoundError):
+            P = np.array(P.todense())  # make dense (needs to happen for eigh).
     w, V = LA.eigh(P, lower=lower, check_finite=check_finite)
 
     if rcond is not None:
@@ -215,9 +229,14 @@ def decomp_quad(P, cond=None, rcond=None, lower=True, check_finite: bool = True)
     return scale, M1, M2
 
 
-def quad_form(x, P):
+def quad_form(x, P, assume_PSD: bool = False):
     """ Alias for :math:`x^T P x`.
 
+    Parameters
+    ----------
+    x : vector argument.
+    P : matrix argument.
+    assume_PSD : P is assumed to be PSD without checking.
     """
     x, P = map(Expression.cast_to_const, (x, P))
     # Check dimensions.
@@ -226,6 +245,8 @@ def quad_form(x, P):
     if x.is_constant():
         return x.H @ P @ x
     elif P.is_constant():
+        if assume_PSD:
+            P = psd_wrap(P)
         return QuadForm(x, P)
     else:
         raise Exception(
