@@ -14,6 +14,7 @@ See the License for the specific language governing permissions and
 limitations under the License.
 """
 
-from cvxpy.interface.numpy_interface.ndarray_interface import NDArrayInterface
 from cvxpy.interface.numpy_interface.matrix_interface import MatrixInterface
-from cvxpy.interface.numpy_interface.sparse_matrix_interface import SparseMatrixInterface
+from cvxpy.interface.numpy_interface.ndarray_interface import NDArrayInterface
+from cvxpy.interface.numpy_interface.sparse_matrix_interface import (
+    SparseMatrixInterface,)
