@@ -3,6 +3,7 @@ import numpy as np
 import cvxpy.interface as intf
 import cvxpy.settings as s
 from cvxpy.reductions.solution import Solution, failure_solution
+from cvxpy.reductions.solvers import utilities
 from cvxpy.reductions.solvers.qp_solvers.qp_solver import QpSolver
 
 
@@ -26,20 +27,26 @@ class GUROBI(QpSolver):
 
     MIP_CAPABLE = True
 
+    # Keyword arguments for the CVXPY interface.
+    INTERFACE_ARGS = ["save_file", "reoptimize"]
+
     # Map of Gurobi status to CVXPY status.
     STATUS_MAP = {2: s.OPTIMAL,
                   3: s.INFEASIBLE,
+                  4: s.INFEASIBLE_OR_UNBOUNDED,  # Triggers reoptimize.
                   5: s.UNBOUNDED,
-                  4: s.INFEASIBLE_OR_UNBOUNDED,
-                  6: s.INFEASIBLE,
-                  7: s.SOLVER_ERROR,
-                  8: s.SOLVER_ERROR,
-                  9: s.USER_LIMIT,  # Maximum time expired
-                  # TODO could be anything.
-                  10: s.SOLVER_ERROR,
-                  11: s.SOLVER_ERROR,
-                  12: s.SOLVER_ERROR,
-                  13: s.OPTIMAL_INACCURATE}
+                  6: s.SOLVER_ERROR,
+                  7: s.USER_LIMIT, # ITERATION_LIMIT
+                  8: s.USER_LIMIT, # NODE_LIMIT
+                  9: s.USER_LIMIT,  # TIME_LIMIT
+                  10: s.USER_LIMIT, # SOLUTION_LIMIT
+                  11: s.USER_LIMIT, # INTERRUPTED
+                  12: s.SOLVER_ERROR, # NUMERIC
+                  13: s.USER_LIMIT, # SUBOPTIMAL
+                  14: s.USER_LIMIT, # INPROGRESS
+                  15: s.USER_LIMIT, # USER_OBJ_LIMIT
+                  16: s.USER_LIMIT, # WORK_LIMIT
+                  17: s.USER_LIMIT} # MEM_LIMIT
 
     def name(self):
         return s.GUROBI
@@ -48,6 +55,22 @@ class GUROBI(QpSolver):
         import gurobipy
         gurobipy
 
+    def apply(self, problem):
+        """
+        Construct QP problem data stored in a dictionary.
+        The QP has the following form
+
+            minimize      1/2 x' P x + q' x
+            subject to    A x =  b
+                          F x <= g
+
+        """
+        import gurobipy as grb
+        data, inv_data = super(GUROBI, self).apply(problem)
+        # Add initial guess.
+        data['init_value'] = utilities.stack_vals(problem.variables, grb.GRB.UNDEFINED)
+        return data, inv_data
+
     def invert(self, results, inverse_data):
         model = results["model"]
         x_grb = model.getVars()
@@ -152,82 +175,35 @@ class GUROBI(QpSolver):
                 old_x_grb = old_model.getVars()
                 for idx in range(len(x_grb)):
                     x_grb[idx].start = old_x_grb[idx].X
-        model.update()
-
-        x = np.array(model.getVars(), copy=False)
+        elif warm_start:
+            # Set the start value of Gurobi vars to user provided values.
+            for idx in range(len(x_grb)):
+                x_grb[idx].start = data['init_value'][idx]
 
         if A.shape[0] > 0:
-            if hasattr(model, 'addMConstrs'):
-                # We can pass all of A @ x == b at once, use stable API
-                # introduced with Gurobi v9
-                model.addMConstrs(A, None, grb.GRB.EQUAL, b)
-            elif hasattr(model, '_v811_addMConstrs'):
-                # We can pass all of A @ x == b at once, API only for Gurobi
-                # v811
-                A.eliminate_zeros()  # Work around bug in gurobipy v811
-                sense = np.repeat(grb.GRB.EQUAL, A.shape[0])
-                model._v811_addMConstrs(A, sense, b)
-            else:
-                # Add equality constraints: iterate over the rows of A
-                # adding each row into the model
-                for i in range(A.shape[0]):
-                    start = A.indptr[i]
-                    end = A.indptr[i+1]
-                    variables = x[A.indices[start:end]]
-                    coeff = A.data[start:end]
-                    expr = grb.LinExpr(coeff, variables)
-                    model.addConstr(expr, grb.GRB.EQUAL, b[i])
-        model.update()
+            # We can pass all of A @ x == b at once, use stable API
+            # introduced with Gurobi v9.5
+            model.addMConstr(A, None, grb.GRB.EQUAL, b)
 
         if F.shape[0] > 0:
-            if hasattr(model, 'addMConstrs'):
-                # We can pass all of F @ x <= g at once, use stable API
-                # introduced with Gurobi v9
-                model.addMConstrs(F, None, grb.GRB.LESS_EQUAL, g)
-            elif hasattr(model, '_v811_addMConstrs'):
-                # We can pass all of F @ x <= g at once, API only for Gurobi
-                # v811.
-                F.eliminate_zeros()  # Work around bug in gurobipy v811
-                sense = np.repeat(grb.GRB.LESS_EQUAL, F.shape[0])
-                model._v811_addMConstrs(F, sense, g)
-            else:
-                # Add inequality constraints: iterate over the rows of F
-                # adding each row into the model
-                for i in range(F.shape[0]):
-                    start = F.indptr[i]
-                    end = F.indptr[i+1]
-                    variables = x[F.indices[start:end]]
-                    coeff = F.data[start:end]
-                    expr = grb.LinExpr(coeff, variables)
-                    model.addConstr(expr, grb.GRB.LESS_EQUAL, g[i])
-        model.update()
+            # We can pass all of A @ x == b at once, use stable API
+            # introduced with Gurobi v9.5
+            model.addMConstr(F, None, grb.GRB.LESS_EQUAL, g)
 
         # Define objective
-        if hasattr(model, 'setMObjective'):
-            # Use stable API starting in Gurobi v9
-            P = P.tocoo()
-            model.setMObjective(0.5 * P, q, 0.0)
-        elif hasattr(model, '_v811_setMObjective'):
-            # Use temporary API for Gurobi v811 only
-            P = P.tocoo()
-            model._v811_setMObjective(0.5 * P, q)
-        else:
-            obj = grb.QuadExpr()
-            if P.count_nonzero():  # If there are any nonzero elms in P
-                P = P.tocoo()
-                obj.addTerms(0.5*P.data, vars=list(x[P.row]),
-                             vars2=list(x[P.col]))
-            obj.add(grb.LinExpr(q, x))  # Add linear part
-            model.setObjective(obj)  # Set objective
-        model.update()
+        # Use stable API starting in Gurobi v9
+        P = P.tocoo()
+        model.setMObjective(0.5 * P, q, 0.0)
 
         # Set parameters
         model.setParam("QCPDual", True)
         for key, value in solver_opts.items():
-            model.setParam(key, value)
+            # Ignore arguments unique to the CVXPY interface.
+            if key not in self.INTERFACE_ARGS:
+                model.setParam(key, value)
 
-        # Update model
-        model.update()
+        if 'save_file' in solver_opts:
+            model.write(solver_opts['save_file'])
 
         # Solve problem
         results_dict = {}
