@@ -15,12 +15,14 @@ limitations under the License.
 """
 
 import numpy as np
+import pytest
 import scipy.sparse as sp
 
 import cvxpy as cp
 from cvxpy import Minimize, Problem
 from cvxpy.expressions.constants import Constant, Parameter
 from cvxpy.expressions.variable import Variable
+from cvxpy.reductions.solvers.defines import INSTALLED_MI_SOLVERS
 from cvxpy.tests.base_test import BaseTest
 
 
@@ -238,6 +240,19 @@ class TestComplex(BaseTest):
         val = np.ones(2)*np.sqrt(2)
         self.assertItemsAlmostEqual(x.value, val + 1j*val)
 
+    def test_complex_ndarray(self) -> None:
+        """Test ndarray of type complex64 and complex128.
+        """
+        x = Variable()
+        z = np.full(1, 1j, dtype=np.complex64)
+        x.value = 0
+        expr = x + z
+        assert np.isclose(expr.value, z)
+
+        z = np.full(1, 1j, dtype=np.complex128)
+        expr = x + z
+        assert np.isclose(expr.value, z)
+
     def test_missing_imag(self) -> None:
         """Test problems where imaginary is missing.
         """
@@ -278,7 +293,7 @@ class TestComplex(BaseTest):
         """
         x = Variable((1, 2), complex=True)
         prob = Problem(cp.Maximize(cp.sum(cp.imag(x) + cp.real(x))), [cp.norm1(x) <= 2])
-        result = prob.solve(solver="ECOS")
+        result = prob.solve(solver=cp.CLARABEL)
         self.assertAlmostEqual(result, 2*np.sqrt(2))
         val = np.ones(2)*np.sqrt(2)/2
         # self.assertItemsAlmostEqual(x.value, val + 1j*val)
@@ -286,7 +301,7 @@ class TestComplex(BaseTest):
         x = Variable((2, 2), complex=True)
         prob = Problem(cp.Maximize(cp.sum(cp.imag(x) + cp.real(x))),
                        [cp.pnorm(x, p=2) <= np.sqrt(8)])
-        result = prob.solve(solver="ECOS")
+        result = prob.solve(solver=cp.CLARABEL)
         self.assertAlmostEqual(result, 8)
         val = np.ones((2, 2))
         self.assertItemsAlmostEqual(x.value, val + 1j*val)
@@ -314,11 +329,25 @@ class TestComplex(BaseTest):
         P = np.arange(9) - 2j*np.arange(9)
         P = np.reshape(P, (3, 3))
         P = np.conj(P.T).dot(P)/100 + np.eye(3)*.1
-        value = cp.log_det(P).value
+        logdet_value = cp.log_det(P).value
         X = Variable((3, 3), complex=True)
-        prob = Problem(cp.Maximize(cp.log_det(X)), [X == P])
+        objective = cp.log_det(X)
+        prob = Problem(cp.Maximize(objective), [X == P])
         result = prob.solve(solver=cp.SCS, eps=1e-6)
-        self.assertAlmostEqual(result, value, places=2)
+        self.assertAlmostEqual(result, logdet_value, places=2)
+        objective_value = objective.value
+        _, ld = np.linalg.slogdet(P)
+        self.assertAlmostEqual(objective_value, ld, places=3)
+
+        # Test case for Issue 1816.
+        #   The optimal solution is the identity matrix scaled by 3.
+        #   NumPy's slogdet function returns a sign "s" with a tiny complex
+        #   part which causes 1 == s to fail.
+        cons = [X >> 0, cp.real(cp.trace(X)) <= 9]
+        obj = cp.Maximize(cp.log_det(X))
+        prob = cp.Problem(objective=obj, constraints=cons)
+        prob.solve(solver=cp.SCS, eps=1e-6)
+        self.assertAlmostEqual(obj.value, 3*np.log(3))
 
     def test_eigval_atoms(self) -> None:
         """Test eigenvalue atoms.
@@ -361,7 +390,7 @@ class TestComplex(BaseTest):
         x = Variable(3, complex=False)
         value = cp.quad_form(b, P).value
         prob = Problem(cp.Minimize(cp.quad_form(x, P)), [x == b])
-        result = prob.solve(solver="ECOS")
+        result = prob.solve(solver="CLARABEL")
         self.assertAlmostEqual(result, value)
 
         # Solve a problem with complex variable
@@ -369,7 +398,7 @@ class TestComplex(BaseTest):
         x = Variable(3, complex=True)
         value = cp.quad_form(b, P).value
         prob = Problem(cp.Minimize(cp.quad_form(x, P)), [x == b])
-        result = prob.solve(solver="ECOS")
+        result = prob.solve(solver="CLARABEL")
         normalization = max(abs(result), abs(value))
         self.assertAlmostEqual(result / normalization, value / normalization, places=5)
 
@@ -379,7 +408,7 @@ class TestComplex(BaseTest):
         value = cp.quad_form(b, P).value
         expr = cp.quad_form(x, P)
         prob = Problem(cp.Minimize(expr), [x == b])
-        result = prob.solve(solver="ECOS")
+        result = prob.solve(solver="CLARABEL")
         normalization = max(abs(result), abs(value))
         self.assertAlmostEqual(result / normalization, value / normalization)
 
@@ -457,7 +486,7 @@ class TestComplex(BaseTest):
         obj = cp.Maximize(cp.real(cp.sum(v * np.ones((2, 2)))))
         con = [cp.norm(v) <= 1]
         prob = cp.Problem(obj, con)
-        result = prob.solve(solver="ECOS")
+        result = prob.solve(solver="CLARABEL")
         self.assertAlmostEqual(result, 4.0)
 
     def test_sparse(self) -> None:
@@ -499,7 +528,7 @@ class TestComplex(BaseTest):
         constraints = [f >> 0]
         for k in range(1, n):
             indices = [(i * n) + i - (n - k) for i in range(n - k, n)]
-            constraints += [cp.sum(cp.vec(f)[indices]) == c[n - k]]
+            constraints += [cp.sum(cp.vec(f, order='F')[indices]) == c[n - k]]
         # Form objective.
         obj = cp.Maximize(c[0] - cp.real(cp.trace(f)))
         # Form and solve problem.
@@ -523,6 +552,14 @@ class TestComplex(BaseTest):
             cp.sum_largest(x, 2)
         self.assertEqual(str(cm.exception), "Arguments to sum_largest cannot be complex.")
 
+        with self.assertRaises(Exception) as cm:
+            cp.dotsort(x, 2)
+        self.assertEqual(str(cm.exception), "Arguments to dotsort cannot be complex.")
+
+        with self.assertRaises(Exception) as cm:
+            cp.dotsort(cp.Variable(2), np.array([1+2j]))
+        self.assertEqual(str(cm.exception), "Arguments to dotsort cannot be complex.")
+
         x = Variable(2, complex=True)
         for atom in [cp.geo_mean, cp.log_sum_exp, cp.max,
                      cp.entr, cp.exp, cp.huber,
@@ -602,6 +639,10 @@ class TestComplex(BaseTest):
         print("P2 is complex:", cp.quad_form(x, P2).curvature)
         assert cp.quad_form(x, P2).is_dcp()
 
+    @pytest.mark.skipif(
+        "HIGHS" not in INSTALLED_MI_SOLVERS, 
+        reason='HiGHS solver is not installed.'
+    )
     def test_bool(self) -> None:
         # The purpose of this test is to make sure
         # that we don't try to recover dual variables
@@ -617,7 +658,7 @@ class TestComplex(BaseTest):
 
         obj = cp.Maximize(cp.real(complex_var))
         prob = cp.Problem(obj, constraints)
-        prob.solve(solver='ECOS_BB')
+        prob.solve(solver=cp.HIGHS)
         self.assertAlmostEqual(prob.value, 1, places=4)
 
     def test_partial_trace(self) -> None:
@@ -665,11 +706,11 @@ class TestComplex(BaseTest):
         np.random.seed(1)
 
         # Generate three test cases
-        rho_A = np.random.random((8, 8)) + 1j*np.random.random((8, 8))
+        rho_A = np.random.random((6, 6)) + 1j*np.random.random((6, 6))
         rho_A /= np.trace(rho_A)
-        rho_B = np.random.random((6, 6)) + 1j*np.random.random((6, 6))
+        rho_B = np.random.random((4, 4)) + 1j*np.random.random((4, 4))
         rho_B /= np.trace(rho_B)
-        rho_C = np.random.random((4, 4)) + 1j*np.random.random((4, 4))
+        rho_C = np.random.random((2, 2)) + 1j*np.random.random((2, 2))
         rho_C /= np.trace(rho_C)
 
         rho_TC = np.kron(np.kron(rho_A, rho_B), rho_C.T)
@@ -680,11 +721,82 @@ class TestComplex(BaseTest):
         rho_ABC = cp.Variable(shape=rho_ABC_val.shape, complex=True)
         cons = [
             rho_ABC_val == rho_ABC,
-            rho_TC == cp.partial_transpose(rho_ABC, [8, 6, 4], axis=2),
-            rho_TB == cp.partial_transpose(rho_ABC, [8, 6, 4], axis=1),
+            rho_TC == cp.partial_transpose(rho_ABC, (6, 4, 2), axis=2),
+            rho_TB == cp.partial_transpose(rho_ABC, (6, 4, 2), axis=1),
         ]
         prob = cp.Problem(cp.Minimize(0), cons)
         prob.solve()
 
-        print(rho_ABC_val)
         assert np.allclose(rho_ABC.value, rho_ABC_val)
+
+    def test_duals(self) -> None:
+        np.random.seed(0)
+        u_real = np.random.rand(3)
+        u_imag = np.random.rand(3)
+        u = u_real + 1j * u_imag
+
+        y = cp.Variable(shape=(3,))
+        helper_objective = cp.Minimize(cp.norm(y - u_real))
+
+        ####################################
+        #
+        #   Compute reference values
+        #
+        ####################################
+        con_a = cp.PowCone3D(y[0], y[1], y[2], [0.25])
+        helper_prob_a = cp.Problem(helper_objective, [con_a])
+        helper_prob_a.solve()
+        expect_dual_a = con_a.dual_value
+
+        con_b = cp.ExpCone(y[0], y[1], y[2])
+        helper_prob_b = cp.Problem(helper_objective, [con_b])
+        helper_prob_b.solve()
+        expect_dual_b = con_b.dual_value
+
+        con_c = cp.SOC(y[2], y[:2])
+        helper_prob_c = cp.Problem(helper_objective, [con_c])
+        helper_prob_c.solve()
+        expect_dual_c = con_c.dual_value
+
+        ####################################
+        #
+        #   Run tests
+        #
+        ####################################
+        x = cp.Variable(shape=(3,), complex=True)
+        actual_objective = cp.Minimize(cp.norm(x - u))
+        coupling_con = cp.real(x) == y
+
+        con_a_test = con_a.copy()
+        prob_a = cp.Problem(actual_objective, [coupling_con, con_a_test])
+        prob_a.solve()
+        actual_dual_a = con_a_test.dual_value
+        self.assertItemsAlmostEqual(actual_dual_a, expect_dual_a, places=2)
+
+        con_b_test = con_b.copy()
+        prob_b = cp.Problem(actual_objective, [coupling_con, con_b_test])
+        prob_b.solve()
+        actual_dual_b = con_b_test.dual_value
+        self.assertItemsAlmostEqual(actual_dual_b, expect_dual_b, places=2)
+
+        con_c_test = con_c.copy()
+        prob_c = cp.Problem(actual_objective, [coupling_con, con_c_test])
+        prob_c.solve()
+        actual_dual_c = con_c_test.dual_value
+        self.assertItemsAlmostEqual(actual_dual_c[0], expect_dual_c[0], places=2)
+        self.assertItemsAlmostEqual(actual_dual_c[1], expect_dual_c[1], places=2)
+
+    def test_illegal_complex_args(self) -> None:
+        x = cp.Variable(shape=(3,), complex=True)
+        with self.assertRaises(ValueError):
+            cp.ExpCone(x[0], x[1], x[2])
+        with self.assertRaises(ValueError):
+            cp.PowCone3D(x[0], x[1], x[2], [0.5])
+        with self.assertRaises(ValueError):
+            cp.PowConeND(x[0], x[:2], [0.5, 0.5])
+        with self.assertRaises(ValueError):
+            cp.RelEntrConeQuad(x[0], x[1], x[2], 1, 1)
+        with self.assertRaises(ValueError):
+            cp.NonNeg(x)
+        with self.assertRaises(ValueError):
+            cp.NonPos(x)
