@@ -13,7 +13,6 @@ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 """
-import sys
 import unittest
 from typing import Tuple
 
@@ -25,8 +24,6 @@ from cvxpy.constraints.constraint import Constraint
 from cvxpy.expressions.expression import Expression
 from cvxpy.expressions.variable import Variable
 
-PY35 = sys.version_info >= (3, 5)
-
 
 class TestMatrices(unittest.TestCase):
     """ Unit tests for testing different forms of matrices as constants. """
@@ -130,8 +127,7 @@ class TestMatrices(unittest.TestCase):
         self.assertExpression(B @ var, (4, 2))
         self.assertExpression(var - A, (4, 2))
         self.assertExpression(A - A - var, (4, 2))
-        if PY35:
-            self.assertExpression(var.__rmatmul__(B), (4, 2))
+        self.assertExpression(var.__rmatmul__(B), (4, 2))
         # self.assertExpression(var <= A, (4, 2))
         # self.assertExpression(A <= var, (4, 2))
         # self.assertExpression(var == A, (4, 2))
